Package: lavinteract
Type: Package
Title: Post-Estimation Utilities for 'lavaan' Fitted Models
Version: 0.5.3
Authors@R: 
    person(
      "Giuseppe", "Corbelli",
      email = "giuseppe.corbelli@uninettunouniversity.net",
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-2864-3548")
    )
Description: Companion toolbox for structural equation models fitted with 'lavaan'. Provides post-estimation diagnostics and graphics that operate directly on a fitted object using its estimates and covariance, and refits auxiliary models when needed. The package relies on 'lavaan' (Rosseel, 2012) <doi:10.18637/jss.v048.i02>.
URL: https://github.com/g-corbelli/lavinteract
BugReports: https://github.com/g-corbelli/lavinteract/issues
License: GPL-3
Encoding: UTF-8
Imports: lavaan, rlang, ggplot2, stats, utils
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Language: en-US
NeedsCompilation: no
Maintainer: Giuseppe Corbelli <giuseppe.corbelli@uninettunouniversity.net>
Config/testthat/edition: 3
RoxygenNote: 7.3.2
Packaged: 2026-08-20 12:42:29 UTC; giuse
Author: Giuseppe Corbelli [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2864-3548>)
Repository: CRAN
Date/Publication: 2026-08-21 05:42:24 UTC
Built: R 4.6.1; ; 2026-08-21 09:33:40 UTC; unix

Package: lavinteract
Type: Package
Title: Post-Estimation Utilities for 'lavaan' Fitted Models
Version: 0.5.1
Authors@R: 
    person(
      "Giuseppe", "Corbelli",
      email = "giuseppe.corbelli@uninettunouniversity.net",
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-2864-3548")
    )
Description: Companion toolbox for structural equation models fitted with 'lavaan'. Provides post-estimation diagnostics and graphics that operate directly on a fitted object using its estimates and covariance, and refits auxiliary models when needed. The package relies on 'lavaan' (Rosseel, 2012) <doi:10.18637/jss.v048.i02>.
URL: https://github.com/g-corbelli/lavinteract
BugReports: https://github.com/g-corbelli/lavinteract/issues
License: GPL-3
Encoding: UTF-8
Imports: lavaan, rlang, ggplot2, stats, utils
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Language: en-US
NeedsCompilation: no
Maintainer: Giuseppe Corbelli <giuseppe.corbelli@uninettunouniversity.net>
Config/testthat/edition: 3
RoxygenNote: 7.3.2
Packaged: 2026-04-28 09:19:19 UTC; giuse
Author: Giuseppe Corbelli [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2864-3548>)
Repository: CRAN
Date/Publication: 2026-04-28 14:20:02 UTC
Built: R 4.5.2; ; 2026-04-28 15:33:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,lav_localfit)
S3method(print,lav_cv)
S3method(print,lav_deltaR2)
S3method(print,lav_fdr)
S3method(print,lav_jn)
S3method(print,lav_localfit)
S3method(print,lav_slopes)
S3method(print,lav_vif)
S3method(summary,lav_cv)
S3method(summary,lav_deltaR2)
S3method(summary,lav_fdr)
S3method(summary,lav_jn)
S3method(summary,lav_localfit)
S3method(summary,lav_slopes)
S3method(summary,lav_vif)
export(lav_cv)
export(lav_deltaR2)
export(lav_fdr)
export(lav_jn)
export(lav_localfit)
export(lav_slopes)
export(lav_vif)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(lavaan,lavInspect)
importFrom(lavaan,lavNames)
importFrom(lavaan,lavResiduals)
importFrom(lavaan,parTable)
importFrom(lavaan,parameterEstimates)
importFrom(rlang,"%||%")
importFrom(rlang,sym)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,sd)
importFrom(stats,vcov)
importFrom(utils,head)

#' @noRd
.onAttach <- function(libname, pkgname) {
  packageStartupMessage("lavinteract: post-estimation utilities for 'lavaan' fitted models.")
  packageStartupMessage("For more information, type ?lavinteract.")
  packageStartupMessage(
    "For suggestions or to report issues, please contact Giuseppe Corbelli at ",
    "giuseppe.corbelli@uninettunouniversity.net."
  )
}

#' Local fit diagnostics for fitted \code{lavaan} models
#'
#' Evaluate local fit in a fitted \code{lavaan} model from residual-based
#' diagnostics. The function extracts residual summaries, computes standardized 
#' covariance and mean residual diagnostics, identifies the largest local 
#' discrepancies, and builds a heatmap of covariance residual misfit. 
#' By default, the function computes Bentler-standardized residual summaries and
#' standardized covariance residual diagnostics suitable for screening localized
#' strain in the model. 
#' The default plot is a traffic-light heatmap of the absolute standardized
#' covariance residuals.
#'
#' @usage
#' lav_localfit(
#'   fit,
#'   group = 1L,
#'   type = "cor.bentler",
#'   thresholds = c(1.96, 2.58),
#'   top_n = 10L,
#'   triangle = "lower",
#'   include_diagonal = FALSE,
#'   plot = TRUE,
#'   plot_style = "trafficlight",
#'   show_values = FALSE,
#'   digits = 3L,
#'   good_color = "#009E73",
#'   moderate_color = "#F0E442",
#'   poor_color = "#D55E00",
#'   neg_color = "#3B4CC0",
#'   pos_color = "#B40426",
#'   na_color = "grey90",
#'   return_data = FALSE
#' )
#'
#' @param fit A fitted \code{lavaan} object.
#' @param group Group to inspect in a multigroup model. May be either a numeric
#'   group index or a character group label. Ignored for single-group models.
#' @param type Character string passed to \code{lavaan::lavResiduals()}.
#'   Common choices are \code{"cor.bentler"} (default), \code{"raw"},
#'   \code{"cor"}, and \code{"cor.bollen"}.
#' @param thresholds Numeric vector of length 2 giving the descriptive cutoffs
#'   used for standardized residual magnitudes. The first value separates
#'   \code{"minor"} from \code{"moderate"} discrepancies; the second separates
#'   \code{"moderate"} from \code{"notable"} discrepancies. Used for the
#'   traffic-light heatmap and for printed summaries. Defaults to
#'   \code{c(1.96, 2.58)}, as per common z-based guidelines.
#' @param top_n Integer. Number of largest absolute standardized covariance
#'   residuals to report in the output table.
#' @param triangle Character string. Either \code{"lower"} or \code{"full"}.
#'   If \code{"lower"}, only the lower triangle of the covariance residual
#'   matrix is used in the heatmap and top-residual table.
#' @param include_diagonal Logical. If \code{TRUE}, include diagonal elements
#'   (variance residuals) in the covariance residual diagnostics.
#' @param plot Logical. If \code{TRUE}, produce a heatmap of covariance residual
#'   misfit.
#' @param plot_style Character string. Either \code{"trafficlight"} or
#'   \code{"signed"}. The default \code{"trafficlight"} plots the absolute
#'   standardized covariance residuals in descriptive magnitude categories. The
#'   \code{"signed"} option plots signed standardized residuals on a diverging
#'   color scale.
#' @param show_values Logical. If \code{TRUE}, overlay numeric residual values on
#'   the heatmap tiles.
#' @param digits Non-negative integer giving the default number of digits used
#'   in printed output and numeric labels.
#' @param good_color Fill color for \code{"minor"} discrepancies in the
#'   traffic-light heatmap.
#' @param moderate_color Fill color for \code{"moderate"} discrepancies in the
#'   traffic-light heatmap.
#' @param poor_color Fill color for \code{"notable"} discrepancies in the
#'   traffic-light heatmap.
#' @param neg_color Fill color for negative residuals in the signed heatmap.
#' @param pos_color Fill color for positive residuals in the signed heatmap.
#' @param na_color Fill color for cells omitted from the heatmap, such as the
#'   upper triangle when \code{triangle = "lower"}.
#' @param return_data Logical. If \code{TRUE}, include the long-format heatmap
#'   data frame in the returned object.
#'
#' @return A list of class \code{"lav_localfit"} with elements:
#' \itemize{
#'   \item \code{summary}: residual summary returned by
#'     \code{lavaan::lavResiduals()} for the selected group;
#'   \item \code{cov_residuals}: covariance residual matrix for the selected
#'     group;
#'   \item \code{cov_z}: standardized covariance residual matrix;
#'   \item \code{mean_residuals}: mean residual vector, if available;
#'   \item \code{mean_z}: standardized mean residual vector, if available;
#'   \item \code{top_cov_residuals}: data frame of the largest absolute
#'     standardized covariance residuals;
#'   \item \code{top_mean_residuals}: data frame of the largest absolute
#'     standardized mean residuals;
#'   \item \code{counts}: list with counts of covariance residuals exceeding the
#'     descriptive thresholds, the maximum absolute standardized covariance
#'     residual, and the corresponding variable pair;
#'   \item \code{plot}: a \code{ggplot} heatmap object, or \code{NULL} if
#'     \code{plot = FALSE};
#'   \item \code{group}: numeric index of the selected group;
#'   \item \code{group_label}: character label of the selected group;
#'   \item \code{type}: residual type used;
#'   \item \code{thresholds}: thresholds used for descriptive classification;
#'   \item \code{digits}: default number of digits for printing;
#'   \item \code{call}: matched function call;
#'   \item \code{plot_data}: only when \code{return_data = TRUE}, the long-format
#'     data used to build the heatmap.
#' }
#'
#' @section Notes:
#' In this function, local fit is evaluated from residuals of the sample summary
#' statistics rather than from casewise prediction errors. Standardized residuals
#' are used to screen where the model reproduces specific covariances or means
#' poorly. The default thresholds are descriptive aids only. They should not be
#' treated as formal decision rules or as substitutes for theory-based model
#' evaluation. When the fitted object uses bootstrap standard errors,
#' standardized residual diagnostics are obtained from an auxiliary refit of the
#' same model without bootstrap.
#'
#' @references
#' Rosseel, Y. (2012). lavaan: An R package for structural equation modeling.
#' \emph{Journal of Statistical Software}, \emph{48}(2), 1-36.
#' \doi{10.18637/jss.v048.i02}
#'
#' Schermelleh-Engel, K., Moosbrugger, H., & Muller, H. (2003). Evaluating the
#' fit of structural equation models: Tests of significance and descriptive
#' goodness-of-fit measures. \emph{Methods of Psychological Research Online},
#' \emph{8}(2), 23-74.
#'
#' @seealso \code{\link[lavaan]{lavResiduals}} for residual extraction and
#'   \code{\link{lav_deltaR2}} for local structural effect-size diagnostics.
#'
#' @examples
#' model <- "
#' ind60 =~ x1 + x2 + x3
#' dem60 =~ y1 + y2 + y3 + y4
#' dem65 =~ y5 + y6 + y7 + y8
#' dem60 ~ ind60
#' dem65 ~ ind60 + dem60
#' y1 ~~ y5
#' y2 ~~ y6
#' "
#' fit <- lavaan::sem(
#' model = model, 
#' data = lavaan::PoliticalDemocracy,
#' std.lv = TRUE, 
#' estimator = "MLR", 
#' meanstructure = TRUE)
#'
#' lav_localfit(fit)
#'
#' @importFrom lavaan lavInspect lavResiduals
#' @importFrom ggplot2 ggplot aes geom_tile geom_text scale_fill_manual
#'   scale_fill_gradient2 coord_equal labs theme_minimal theme element_text
#'   element_blank
#' @importFrom rlang %||%
#' @importFrom utils head
#' @export
lav_localfit <- function(
    fit,
    group = 1L,
    type = "cor.bentler",
    thresholds = c(1.96, 2.58),
    top_n = 10L,
    triangle = "lower",
    include_diagonal = FALSE,
    plot = TRUE,
    plot_style = "trafficlight",
    show_values = FALSE,
    digits = 3L,
    good_color = "#009E73",
    moderate_color = "#F0E442",
    poor_color = "#D55E00",
    neg_color = "#3B4CC0",
    pos_color = "#B40426",
    na_color = "grey90",
    return_data = FALSE
) {
  
  if (!inherits(fit, "lavaan")) {
    stop("'fit' must be a fitted 'lavaan' object.", call. = FALSE)
  }
  
  fit_converged <- tryCatch(lavaan::lavInspect(fit, "converged"),
                            error = function(e) FALSE)
  if (!isTRUE(fit_converged)) {
    stop("'fit' did not converge.", call. = FALSE)
  }
  
  if (!is.character(type) || length(type) != 1L || is.na(type)) {
    stop("'type' must be a single character string.", call. = FALSE)
  }
  
  triangle <- match.arg(triangle, choices = c("lower", "full"))
  plot_style <- match.arg(plot_style, choices = c("trafficlight", "signed"))
  
  if (!is.numeric(thresholds) || length(thresholds) != 2L || any(is.na(thresholds))) {
    stop("'thresholds' must be a numeric vector of length 2.", call. = FALSE)
  }
  thresholds <- as.numeric(thresholds)
  if (thresholds[1L] <= 0 || thresholds[2L] <= thresholds[1L]) {
    stop("'thresholds' must be positive and strictly increasing.", call. = FALSE)
  }
  
  if (!is.numeric(top_n) || length(top_n) != 1L || is.na(top_n) || top_n < 1) {
    stop("'top_n' must be a positive integer.", call. = FALSE)
  }
  top_n <- as.integer(top_n)
  
  if (!is.logical(include_diagonal) || length(include_diagonal) != 1L || is.na(include_diagonal)) {
    stop("'include_diagonal' must be TRUE or FALSE.", call. = FALSE)
  }
  
  if (!is.logical(plot) || length(plot) != 1L || is.na(plot)) {
    stop("'plot' must be TRUE or FALSE.", call. = FALSE)
  }
  
  if (!is.logical(show_values) || length(show_values) != 1L || is.na(show_values)) {
    stop("'show_values' must be TRUE or FALSE.", call. = FALSE)
  }
  
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) || digits < 0) {
    stop("'digits' must be a non-negative integer.", call. = FALSE)
  }
  digits <- as.integer(digits)
  
  ngroups <- tryCatch(lavaan::lavInspect(fit, "ngroups"),
                      error = function(e) 1L)
  group_labels <- tryCatch(lavaan::lavInspect(fit, "group.label"),
                           error = function(e) NULL)
  
  if (is.null(group_labels) || length(group_labels) != ngroups) {
    group_labels <- as.character(seq_len(ngroups))
  } else {
    group_labels <- as.character(group_labels)
  }
  
  if (ngroups == 1L) {
    group_index <- 1L
    group_label <- group_labels[1L]
  } else {
    if (is.numeric(group) && length(group) == 1L && !is.na(group)) {
      group_index <- as.integer(group)
      if (group_index < 1L || group_index > ngroups) {
        stop("'group' is outside the available group range.", call. = FALSE)
      }
      group_label <- group_labels[group_index]
    } else if (is.character(group) && length(group) == 1L && !is.na(group)) {
      pos <- match(group, group_labels)
      if (is.na(pos)) {
        stop("'group' does not match any available group label.", call. = FALSE)
      }
      group_index <- pos
      group_label <- group_labels[group_index]
    } else {
      stop("'group' must be a single numeric index or a single group label.", call. = FALSE)
    }
  }
  
  fmt_num <- function(x, k = digits, apa = FALSE) {
    if (length(x) == 0L || is.na(x)) return("NA")
    out <- formatC(x, digits = k, format = "f")
    if (apa) {
      out <- sub("^(-?)0\\.", "\\1.", out)
    }
    out
  }
  
  mag_label <- function(abs_z, th = thresholds) {
    if (is.na(abs_z)) {
      return(NA_character_)
    }
    if (abs_z < th[1L]) {
      return("minor")
    }
    if (abs_z < th[2L]) {
      return("moderate")
    }
    "notable"
  }
  
  residual_type_label <- function(x) {
    if (identical(x, "cor.bentler")) return("Bentler-standardized residuals")
    if (identical(x, "cor.bollen")) return("Bollen-standardized residuals")
    if (identical(x, "cor")) return("correlation residuals")
    if (identical(x, "raw")) return("raw residuals")
    paste(x, "residuals")
  }
  
  extract_main_metric <- function(summary_obj, type) {
    if (is.null(summary_obj)) {
      return(list(name = NA_character_, value = NA_real_))
    }
    
    sm <- as.matrix(summary_obj)
    rn <- rownames(sm)
    cn <- colnames(sm)
    
    target_name <- if (identical(type, "cor.bentler")) {
      "srmr"
    } else if (identical(type, "raw")) {
      "rmr"
    } else if (identical(type, "cor.bollen")) {
      "crmr"
    } else {
      NA_character_
    }
    
    if (!is.na(target_name) && target_name %in% rn) {
      target_col <- if ("total" %in% cn) "total" else cn[length(cn)]
      return(list(name = toupper(target_name), value = as.numeric(sm[target_name, target_col])))
    }
    
    if (length(rn) > 0L && length(cn) > 0L) {
      target_col <- if ("total" %in% cn) "total" else cn[length(cn)]
      return(list(name = rn[1L], value = as.numeric(sm[1L, target_col])))
    }
    
    list(name = NA_character_, value = NA_real_)
  }
  
  recover_fit_data <- function(fit) {
    out <- tryCatch(lavaan::lavInspect(fit, "data"), error = function(e) NULL)
    if (is.null(out)) {
      out <- tryCatch(lavaan::lavInspect(fit, "data.original"), error = function(e) NULL)
    }
    if (is.null(out)) {
      stop("Could not recover data from `fit` for auxiliary residual refit.", call. = FALSE)
    }
    
    ov_names <- tryCatch(lavaan::lavNames(fit, type = "ov"), error = function(e) NULL)
    
    if (is.data.frame(out)) {
      out <- as.data.frame(out)
    } else if (is.matrix(out)) {
      out <- as.data.frame(out)
      if (!is.null(ov_names) && length(ov_names) == ncol(out)) {
        names(out) <- ov_names
      }
    } else if (is.list(out)) {
      ok_list <- all(vapply(out, function(x) is.data.frame(x) || is.matrix(x), logical(1)))
      if (!ok_list) {
        stop("Recovered grouped data could not be coerced to a data frame.", call. = FALSE)
      }
      
      group_labels <- tryCatch(lavaan::lavInspect(fit, "group.label"), error = function(e) NULL)
      group_var <- tryCatch(fit@Options$group, error = function(e) NULL)
      if (length(group_var) != 1L || is.na(group_var) || !nzchar(group_var)) {
        group_var <- NULL
      }
      
      out_list <- vector("list", length(out))
      for (g in seq_along(out)) {
        dg <- out[[g]]
        if (is.matrix(dg)) {
          dg <- as.data.frame(dg)
          if (!is.null(ov_names) && length(ov_names) == ncol(dg)) {
            names(dg) <- ov_names
          }
        } else {
          dg <- as.data.frame(dg)
        }
        
        if (!is.null(group_var) && !(group_var %in% names(dg))) {
          lab_g <- if (!is.null(group_labels) && length(group_labels) >= g) group_labels[g] else g
          num_lab <- suppressWarnings(as.numeric(lab_g))
          if (!is.na(num_lab)) {
            dg[[group_var]] <- num_lab
          } else {
            dg[[group_var]] <- as.character(lab_g)
          }
        }
        
        out_list[[g]] <- dg
      }
      
      out <- do.call(rbind, out_list)
      rownames(out) <- NULL
    } else {
      stop("Recovered data is not a supported object type.", call. = FALSE)
    }
    
    out
  }
  
  make_aux_resid_fit <- function(fit) {
    OPT <- fit@Options
    data_in <- recover_fit_data(fit)
    ordered_vars <- tryCatch(lavaan::lavNames(fit, type = "ov.ord"), error = function(e) character(0))
    
    refit_args <- list(
      model = lavaan::parTable(fit),
      data = data_in,
      group = if (!is.null(OPT$group) && nzchar(OPT$group)) OPT$group else NULL,
      estimator = OPT$estimator,
      missing = OPT$missing,
      std.lv = isTRUE(OPT$std.lv),
      meanstructure = isTRUE(OPT$meanstructure),
      parameterization = OPT$parameterization,
      fixed.x = OPT$fixed.x,
      conditional.x = OPT$conditional.x,
      orthogonal = isTRUE(OPT$orthogonal),
      ordered = if (length(ordered_vars)) ordered_vars else NULL,
      se = "standard",
      test = "standard",
      warn = FALSE
    )
    
    refit_args <- refit_args[!vapply(refit_args, is.null, logical(1L))]
    
    do.call(lavaan::lavaan, refit_args)
  }
  
  fit_for_resid <- fit
  use_aux_fit <- isTRUE(identical(fit@Options$se, "bootstrap"))
  
  if (use_aux_fit) {
    fit_for_resid <- make_aux_resid_fit(fit)
  }
  
  res_try <- tryCatch(
    {
      if (ngroups == 1L) {
        lavaan::lavResiduals(
          fit_for_resid,
          type = type,
          zstat = TRUE,
          summary = TRUE
        )
      } else {
        tmp <- lavaan::lavResiduals(
          fit_for_resid,
          type = type,
          zstat = TRUE,
          summary = TRUE,
          drop.list.single.group = FALSE
        )
        tmp[[group_index]]
      }
    },
    error = function(e) NULL
  )
  
  if (is.null(res_try)) {
    stop(
      "Could not compute standardized residual diagnostics for this fitted object. ",
      "This often occurs when lavaan cannot compute the residual asymptotic covariance matrix.",
      call. = FALSE
    )
  }
  
  res_obj <- res_try
  
  cov_res <- tryCatch(as.matrix(res_obj$cov), error = function(e) NULL)
  cov_z <- tryCatch(as.matrix(res_obj$cov.z), error = function(e) NULL)
  mean_res <- tryCatch(as.numeric(res_obj$mean), error = function(e) NULL)
  mean_z <- tryCatch(as.numeric(res_obj$mean.z), error = function(e) NULL)
  if (!is.null(mean_res) && !is.null(names(res_obj$mean))) {
    names(mean_res) <- names(res_obj$mean)
  }
  if (!is.null(mean_z) && !is.null(names(res_obj$mean.z))) {
    names(mean_z) <- names(res_obj$mean.z)
  }
  sum_obj <- tryCatch(res_obj$summary, error = function(e) NULL)
  
  if (is.null(cov_res) || is.null(cov_z)) {
    stop("Could not extract covariance residual matrices from 'fit'.", call. = FALSE)
  }
  
  var_names <- rownames(cov_z)
  if (is.null(var_names)) {
    var_names <- colnames(cov_z)
  }
  if (is.null(var_names)) {
    var_names <- paste0("V", seq_len(nrow(cov_z)))
  }
  rownames(cov_res) <- var_names
  colnames(cov_res) <- var_names
  rownames(cov_z) <- var_names
  colnames(cov_z) <- var_names
  
  # build long table for covariance residuals
  long_rows <- list()
  idx_row <- 0L
  
  nr <- nrow(cov_z)
  nc <- ncol(cov_z)
  
  for (i in seq_len(nr)) {
    for (j in seq_len(nc)) {
      
      if (triangle == "lower" && j > i) next
      if (!include_diagonal && i == j) next
      
      idx_row <- idx_row + 1L
      z_val <- cov_z[i, j]
      raw_val <- cov_res[i, j]
      abs_z <- abs(z_val)
      
      long_rows[[idx_row]] <- data.frame(
        var1 = rownames(cov_z)[i],
        var2 = colnames(cov_z)[j],
        row_index = i,
        col_index = j,
        residual = as.numeric(raw_val),
        z_resid = as.numeric(z_val),
        abs_z = as.numeric(abs_z),
        magnitude = mag_label(abs_z),
        stringsAsFactors = FALSE
      )
    }
  }
  
  cov_long <- do.call(rbind, long_rows)
  rownames(cov_long) <- NULL
  cov_long <- cov_long[order(cov_long$abs_z, decreasing = TRUE), , drop = FALSE]
  
  if (nrow(cov_long) > 0L) {
    top_cov <- head(cov_long, top_n)
  } else {
    top_cov <- data.frame(
      var1 = character(0),
      var2 = character(0),
      row_index = integer(0),
      col_index = integer(0),
      residual = numeric(0),
      z_resid = numeric(0),
      abs_z = numeric(0),
      magnitude = character(0),
      stringsAsFactors = FALSE
    )
  }
  
  # mean residual table
  if (!is.null(mean_res) && !is.null(mean_z) && length(mean_res) == length(mean_z)) {
    mean_df <- data.frame(
      variable = names(mean_z) %||% paste0("V", seq_along(mean_z)),
      residual = as.numeric(mean_res),
      z_resid = as.numeric(mean_z),
      abs_z = abs(as.numeric(mean_z)),
      magnitude = vapply(abs(as.numeric(mean_z)), mag_label, character(1L)),
      stringsAsFactors = FALSE
    )
    mean_df <- mean_df[order(mean_df$abs_z, decreasing = TRUE), , drop = FALSE]
    top_mean <- head(mean_df, top_n)
  } else {
    mean_df <- data.frame(
      variable = character(0),
      residual = numeric(0),
      z_resid = numeric(0),
      abs_z = numeric(0),
      magnitude = character(0),
      stringsAsFactors = FALSE
    )
    top_mean <- mean_df
  }
  
  n_cov_ge_t1 <- sum(cov_long$abs_z >= thresholds[1L], na.rm = TRUE)
  n_cov_ge_t2 <- sum(cov_long$abs_z >= thresholds[2L], na.rm = TRUE)
  
  max_abs_cov <- if (nrow(cov_long)) max(cov_long$abs_z, na.rm = TRUE) else NA_real_
  max_cov_pair <- if (nrow(cov_long)) {
    c(cov_long$var1[1L], cov_long$var2[1L])
  } else {
    c(NA_character_, NA_character_)
  }
  
  counts <- list(
    n_cov_total = nrow(cov_long),
    n_cov_abs_ge_threshold1 = n_cov_ge_t1,
    n_cov_abs_ge_threshold2 = n_cov_ge_t2,
    max_abs_cov_z = max_abs_cov,
    max_cov_pair = max_cov_pair
  )
  
  # build plot data
  plot_df <- NULL
  if (isTRUE(plot)) {
    
    plot_df <- cov_long
    # complete matrix cells for plotting when lower triangle requested
    all_cells <- expand.grid(
      row_index = seq_len(nr),
      col_index = seq_len(nc),
      stringsAsFactors = FALSE
    )
    all_cells$var1 <- rownames(cov_z)[all_cells$row_index]
    all_cells$var2 <- colnames(cov_z)[all_cells$col_index]
    
    plot_df <- merge(
      all_cells,
      plot_df,
      by = c("var1", "var2", "row_index", "col_index"),
      all.x = TRUE,
      sort = FALSE
    )
    
    # blank unwanted cells
    if (triangle == "lower") {
      plot_df$masked <- plot_df$col_index > plot_df$row_index
    } else {
      plot_df$masked <- FALSE
    }
    if (!include_diagonal) {
      plot_df$masked <- plot_df$masked | (plot_df$row_index == plot_df$col_index)
    }
    
    plot_df$fill_value <- if (plot_style == "trafficlight") {
      as.character(plot_df$magnitude)
    } else {
      plot_df$z_resid
    }
    
    plot_df$fill_value[plot_df$masked] <- NA
    plot_df$label_value <- if (plot_style == "trafficlight") {
      vapply(plot_df$abs_z, fmt_num, character(1L), k = digits, apa = FALSE)
    } else {
      vapply(plot_df$z_resid, fmt_num, character(1L), k = digits, apa = FALSE)
    }
    plot_df$label_value[plot_df$masked] <- ""
    
    plot_df$var1 <- factor(plot_df$var1, levels = rev(var_names))
    plot_df$var2 <- factor(plot_df$var2, levels = var_names)
    
    if (plot_style == "trafficlight") {
      plot_df$fill_value <- factor(
        plot_df$fill_value,
        levels = c("minor", "moderate", "notable")
      )
      
      p_out <- ggplot2::ggplot(
        plot_df,
        ggplot2::aes(x = !!rlang::sym("var2"), y = !!rlang::sym("var1"))
      ) +
        ggplot2::geom_tile(
          ggplot2::aes(fill = !!rlang::sym("fill_value")),
          color = "white",
          linewidth = 0.4
        ) +
        ggplot2::scale_fill_manual(
          values = c(
            "minor" = good_color,
            "moderate" = moderate_color,
            "notable" = poor_color
          ),
          na.value = na_color,
          name = "|z| magnitude"
        )
    } else {
      p_out <- ggplot2::ggplot(
        plot_df,
        ggplot2::aes(x = !!rlang::sym("var2"), y = !!rlang::sym("var1"))
      ) +
        ggplot2::geom_tile(
          ggplot2::aes(fill = !!rlang::sym("fill_value")),
          color = "white",
          linewidth = 0.4
        ) +
        ggplot2::scale_fill_gradient2(
          low = neg_color,
          mid = "white",
          high = pos_color,
          midpoint = 0,
          na.value = na_color,
          name = "z residual"
        )
    }
    
    if (isTRUE(show_values)) {
      p_out <- p_out +
        ggplot2::geom_text(
          ggplot2::aes(label = !!rlang::sym("label_value")),
          size = 3
        )
    }
    
    plot_title <- if (plot_style == "trafficlight") {
      "Absolute standardized covariance residuals"
    } else {
      "Signed standardized covariance residuals"
    }
    
    if (ngroups > 1L) {
      plot_subtitle <- paste0("Group ", group_label)
    } else {
      plot_subtitle <- NULL
    }
    
    p_out <- p_out +
      ggplot2::coord_equal() +
      ggplot2::labs(
        title = plot_title,
        subtitle = plot_subtitle,
        x = NULL,
        y = NULL
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        panel.grid = ggplot2::element_blank(),
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        axis.title = ggplot2::element_blank()
      )
  } else {
    p_out <- NULL
  }
  
  summary_df <- NULL
  if (!is.null(sum_obj)) {
    sm <- as.matrix(sum_obj)
    summary_df <- data.frame(
      metric = rownames(sm),
      sm,
      row.names = NULL,
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  } else {
    summary_df <- data.frame(
      metric = character(0),
      stringsAsFactors = FALSE
    )
  }
  
  res <- list(
    summary = summary_df,
    cov_residuals = cov_res,
    cov_z = cov_z,
    mean_residuals = mean_res,
    mean_z = mean_z,
    top_cov_residuals = top_cov,
    top_mean_residuals = top_mean,
    counts = counts,
    plot = p_out,
    group = group_index,
    group_label = group_label,
    type = type,
    thresholds = thresholds,
    digits = digits,
    call = match.call()
  )
  
  if (isTRUE(return_data)) {
    res$plot_data <- plot_df
  }
  
  class(res) <- "lav_localfit"
  return(res)
}


# --- S3 methods: print, summary, plot ---------------------------------------

#' @param x A \code{"lav_localfit"} object.
#' @param ... Additional arguments; unused.
#' @rdname lav_localfit
#' @export
print.lav_localfit <- function(x, ...) {
  digits <- x$digits %||% 3L
  
  fmt <- function(v, k = digits) {
    if (length(v) == 0L || is.na(v)) "NA" else formatC(v, digits = k, format = "f")
  }
  
  fmt_apa <- function(v, k = digits) {
    if (length(v) == 0L || is.na(v)) return("NA")
    out <- formatC(v, digits = k, format = "f")
    sub("^(-?)0\\.", "\\1.", out)
  }
  
  if (!is.null(x$summary) && nrow(x$summary) > 0L) {
    main_metric_name <- if ("metric" %in% names(x$summary)) {
      x$summary$metric[1L]
    } else {
      NA_character_
    }
  } else {
    main_metric_name <- NA_character_
  }
  
  cat("\nLocal fit diagnostics from residual-based evaluation\n")
  cat(rep("-", 74), "\n", sep = "")
  cat("Residual type: ", x$type, "\n", sep = "")
  cat("Group: ", x$group_label, "\n", sep = "")
  
  if (!is.null(x$summary) && nrow(x$summary) > 0L) {
    sm <- x$summary
    total_col <- if ("total" %in% names(sm)) "total" else names(sm)[ncol(sm)]
    if ("metric" %in% names(sm) && nrow(sm) > 0L) {
      cat("Primary residual summary: ",
          toupper(sm$metric[1L]),
          " = ",
          fmt_apa(sm[[total_col]][1L]),
          "\n", sep = "")
    }
  }
  
  cat("Thresholds: |z| < ", fmt(x$thresholds[1L], k = 2),
      " = minor; ", fmt(x$thresholds[1L], k = 2), " to < ",
      fmt(x$thresholds[2L], k = 2), " = moderate; >= ",
      fmt(x$thresholds[2L], k = 2), " = notable\n", sep = "")
  cat("Covariance residuals screened: ", x$counts$n_cov_total, "\n", sep = "")
  cat("Count with |z| >= ", fmt(x$thresholds[1L], k = 2), ": ",
      x$counts$n_cov_abs_ge_threshold1, "\n", sep = "")
  cat("Count with |z| >= ", fmt(x$thresholds[2L], k = 2), ": ",
      x$counts$n_cov_abs_ge_threshold2, "\n", sep = "")
  
  if (!is.na(x$counts$max_abs_cov_z)) {
    cat("Largest absolute standardized covariance residual: ",
        fmt_apa(x$counts$max_abs_cov_z),
        " (", x$counts$max_cov_pair[1L], " with ", x$counts$max_cov_pair[2L], ")\n",
        sep = "")
  }
  
  cat(rep("-", 74), "\n", sep = "")
  
  if (!is.null(x$top_cov_residuals) && nrow(x$top_cov_residuals) > 0L) {
    cat("Largest absolute standardized covariance residuals\n")
    top_cov <- x$top_cov_residuals
    
    var1_txt <- as.character(top_cov$var1)
    var2_txt <- as.character(top_cov$var2)
    z_txt <- vapply(top_cov$z_resid, fmt_apa, character(1L), k = digits)
    abs_txt <- vapply(top_cov$abs_z, fmt_apa, character(1L), k = digits)
    mag_txt <- as.character(top_cov$magnitude)
    
    hdr1 <- "Var1"
    hdr2 <- "Var2"
    hdr3 <- "z"
    hdr4 <- "Magnitude"
    
    w1 <- max(nchar(c(hdr1, var1_txt), type = "width"))
    w2 <- max(nchar(c(hdr2, var2_txt), type = "width"))
    w3 <- max(nchar(c(hdr3, z_txt), type = "width"))
    w4 <- max(nchar(c(hdr4, mag_txt), type = "width"))
    
    row_fmt <- paste0(
      "%-", w1, "s  ",
      "%-", w2, "s  ",
      "%",  w3, "s  ",
      "%-", w4, "s\n"
    )
    
    hdr_line <- sprintf(row_fmt, hdr1, hdr2, hdr3, hdr4)
    
    rule <- paste(rep("-", nchar(sub("\n$", "", hdr_line), type = "width")),
                  collapse = "")
    
    cat(rule, "\n", sep = "")
    cat(hdr_line)
    cat(rule, "\n", sep = "")
    
    for (i in seq_len(nrow(top_cov))) {
      cat(sprintf(
        row_fmt,
        var1_txt[i],
        var2_txt[i],
        z_txt[i],
        mag_txt[i]
      ))
    }
    cat(rule, "\n", sep = "")
  }
  
  # solo se esiste almeno un res delle medie
  if (!is.null(x$top_mean_residuals) &&
      nrow(x$top_mean_residuals) > 0L &&
      any(abs(x$top_mean_residuals$z_resid) > 0, na.rm = TRUE)) { 
    cat("Largest absolute standardized mean residuals\n")
    top_mean <- x$top_mean_residuals
    
    var_txt <- as.character(top_mean$variable)
    z_txt <- vapply(top_mean$z_resid, fmt_apa, character(1L), k = digits)
    abs_txt <- vapply(top_mean$abs_z, fmt_apa, character(1L), k = digits)
    mag_txt <- as.character(top_mean$magnitude)
    
    hdr1 <- "Variable"
    hdr2 <- "z"
    hdr3 <- "Magnitude"
    
    w1 <- max(nchar(c(hdr1, var_txt), type = "width"))
    w2 <- max(nchar(c(hdr2, z_txt), type = "width"))
    w3 <- max(nchar(c(hdr3, mag_txt), type = "width"))
    
    row_fmt <- paste0(
      "%-", w1, "s  ",
      "%",  w2, "s  ",
      "%-", w3, "s\n"
    )
    
    hdr_line <- sprintf(row_fmt, hdr1, hdr2, hdr3)
    
    rule <- paste(rep("-", nchar(sub("\n$", "", hdr_line), type = "width")),
                  collapse = "")
    
    cat(rule, "\n", sep = "")
    cat(hdr_line)
    cat(rule, "\n", sep = "")
    
    for (i in seq_len(nrow(top_mean))) {
      cat(sprintf(
        row_fmt,
        var_txt[i],
        z_txt[i],
        mag_txt[i]
      ))
    }
    cat(rule, "\n", sep = "")
  }
  
  if (!is.null(x$plot)) {
    print(x$plot)
  }
  
  invisible(x)
}

#' @param object A \code{"lav_localfit"} object.
#' @param ... Additional arguments; unused.
#' @rdname lav_localfit
#' @export
summary.lav_localfit <- function(object, ...) {
  print.lav_localfit(object, ...)
  invisible(object)
}

#' @param x A \code{"lav_localfit"} object.
#' @param y Ignored.
#' @param ... Additional arguments; unused.
#' @rdname lav_localfit
#' @export
plot.lav_localfit <- function(x, y = NULL, ...) {
  if (is.null(x$plot)) {
    stop("No plot is stored in this object. Refit with 'plot = TRUE'.", call. = FALSE)
  }
  print(x$plot)
  invisible(x)
}

#' Incremental effect sizes for structural predictors in fitted \code{lavaan} models
#'
#' Compute outcome-specific incremental effect sizes for structural predictors in
#' a fitted \code{lavaan} model by comparing the fitted model with nested reduced
#' models in which one predictor, or a block of predictors, is fixed to zero.
#' For each tested predictor set, the function returns the reduction in explained
#' variance (\eqn{\Delta R^2}), the corresponding part \eqn{R^2}, and Cohen's
#' \eqn{f^2 = \Delta R^2 / (1 - R^2_{full})}.
#'
#' Reduced models preserve all untargeted model parameters and differ from the
#' fitted model only in that the tested structural regression path(s) are fixed
#' to zero. Accordingly, the resulting \eqn{\Delta R^2} quantifies the unique
#' contribution of the tested predictor or predictor block to the explained
#' variance of a given endogenous variable, conditional on the rest of the model.
#'
#' If \code{data} is supplied, reduced models are refit from that data set.
#' Otherwise, the function reuses the internal data and fitting options stored in
#' \code{fit}. The function is intended for converged \code{lavaan} models with
#' free structural regressions.
#'
#' @usage
#' lav_deltaR2(
#'   fit,
#'   data = NULL,
#'   outcome = NULL,
#'   terms = NULL,
#'   block = NULL,
#'   quiet = TRUE,
#'   digits = 3L
#' )
#'
#' @param fit A fitted \code{lavaan} object.
#' @param data Optional data frame used to fit \code{fit}. If supplied, this data
#'   frame is used to refit reduced models. If \code{NULL}, reduced models are
#'   refit using the internal data and options stored in \code{fit}.
#' @param outcome Optional character vector naming endogenous variables for which
#'   incremental effect sizes should be computed. If \code{NULL}, all endogenous
#'   variables with at least one free structural predictor are used.
#' @param terms Optional character vector naming predictors to test one at a
#'   time. Ignored if \code{block} is supplied. If \code{NULL}, all eligible
#'   structural predictors are tested individually.
#' @param block Optional character vector naming predictors to remove jointly.
#'   When supplied, one reduced-model comparison is computed per selected outcome
#'   for the entire predictor block. A block is evaluated only for outcomes in
#'   which all named predictors appear as free structural regressors.
#' @param quiet Logical. If \code{TRUE}, suppress reduced-model refit messages.
#' @param digits Non-negative integer giving the default number of digits used in
#'   printing.
#'
#' @return A list with:
#' \itemize{
#'   \item \code{delta_r2_table}: A data frame containing one row per
#'   outcome-by-test comparison, with columns:
#'   \itemize{
#'     \item \code{outcome}: endogenous variable whose \eqn{R^2} is evaluated;
#'     \item \code{tested_set}: predictor or predictor block fixed to zero in the
#'       reduced model;
#'     \item \code{test_type}: either \code{"single"} or \code{"block"};
#'     \item \code{n_terms}: number of predictors removed jointly;
#'     \item \code{group}: numeric group index;
#'     \item \code{group_label}: group label, if available;
#'     \item \code{r2_full}: \eqn{R^2} from the fitted model;
#'     \item \code{r2_reduced}: \eqn{R^2} from the reduced model;
#'     \item \code{delta_r2}: \eqn{R^2_{full} - R^2_{reduced}};
#'     \item \code{part_r2}: equal to \code{delta_r2} in this implementation;
#'     \item \code{f2}: Cohen's \eqn{f^2};
#'     \item \code{f2_magnitude}: conventional descriptive interpretation of
#'       \code{f2} based on Cohen's benchmarks (\code{"negligible"},
#'       \code{"small"}, \code{"medium"}, or \code{"large"});
#'     \item \code{converged}: logical indicator of whether the reduced model
#'       converged.
#'   }
#'   \item \code{settings}: list of user-supplied function settings;
#'   \item \code{group_var}: grouping variable name, or \code{NULL};
#'   \item \code{group_labels}: group labels, if available;
#'   \item \code{call}: matched function call;
#'   \item \code{digits}: default number of digits used for printing.
#' }
#'
#' @section Notes:
#' For a given endogenous variable, \eqn{\Delta R^2} is the reduction in
#' explained variance obtained when the tested predictor or predictor block is
#' fixed to zero, and the reduced model is refit with all untargeted parameters
#' left free.
#' In this implementation, part \eqn{R^2} is numerically identical to \eqn{\Delta R^2}. 
#' Cohen's \eqn{f^2} expresses the same quantity relative to the residual variance of the
#' full model. The \code{f2_magnitude} column applies Cohen's conventional 
#' benchmarks to \eqn{f^2} and is intended only as a descriptive aid.
#'
#' @references
#' Cohen, J. (1988). \emph{Statistical power analysis for the behavioral sciences}
#' (2nd ed.). Lawrence Erlbaum Associates.
#'
#' Rosseel, Y. (2012). lavaan: An R package for structural equation modeling.
#' \emph{Journal of Statistical Software}, \emph{48}(2), 1-36.
#' \doi{10.18637/jss.v048.i02}
#'
#' @seealso \code{\link{lav_cv}} for cross-validated \eqn{R^2}, \code{\link{lav_vif}}
#'   for variance inflation factors.
#'
#' @examples
#' library("lavaan")
#' data("PoliticalDemocracy", 
#' package = "lavaan")
#' model <- "
#' ind60 =~ x1 + x2 + x3
#' dem60 =~ y1 + y2 + y3 + y4
#' dem65 =~ y5 + y6 + y7 + y8
#' dem60 ~ ind60
#' dem65 ~ ind60 + dem60
#' y1 ~~ y5
#' y2 ~~ y4 + y6
#' y3 ~~ y7
#' y4 ~~ y8
#' y6 ~~ y8
#' "
#' fit <- lavaan::sem(model, 
#' data = PoliticalDemocracy)
#' lav_deltaR2(fit = fit)
#' lav_deltaR2(fit = fit, outcome = "dem65")
#' lav_deltaR2(fit = fit, outcome = "dem65", terms = "ind60")
#' lav_deltaR2(fit = fit, outcome = "dem65", block = c("ind60", "dem60"))
#'
#' @importFrom lavaan lavInspect lavNames parTable
#' @importFrom rlang %||%
#' @export
lav_deltaR2 <- function(
    fit,
    data = NULL,
    outcome = NULL,
    terms = NULL,
    block = NULL,
    quiet = TRUE,
    digits = 3L
) {
  
  if (!inherits(fit, "lavaan")) {
    stop("'fit' must be a fitted 'lavaan' object.", call. = FALSE)
  }
  
  fit_converged <- tryCatch(lavaan::lavInspect(fit, "converged"),
                            error = function(e) FALSE)
  if (!isTRUE(fit_converged)) {
    stop("'fit' did not converge.", call. = FALSE)
  }
  
  if (!is.null(outcome) && !is.character(outcome)) {
    stop("'outcome' must be NULL or a character vector.", call. = FALSE)
  }
  
  if (!is.null(terms) && !is.character(terms)) {
    stop("'terms' must be NULL or a character vector.", call. = FALSE)
  }
  
  if (!is.null(block) && !is.character(block)) {
    stop("'block' must be NULL or a character vector.", call. = FALSE)
  }
  
  if (!is.null(block) && !is.null(terms)) {
    stop("Supply either 'terms' or 'block', not both.", call. = FALSE)
  }
  
  if (!is.logical(quiet) || length(quiet) != 1L || is.na(quiet)) {
    stop("'quiet' must be TRUE or FALSE.", call. = FALSE)
  }
  
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) || digits < 0) {
    stop("'digits' must be a non-negative integer.", call. = FALSE)
  }
  
  digits <- as.integer(digits)
  
  data_in <- NULL
  if (!is.null(data)) {
    data_in <- as.data.frame(data)
  }
  
  OPT <- fit@Options
  estimator <- OPT$estimator
  missing_opt <- OPT$missing
  
  as_one_string_or_null <- function(x) {
    if (is.character(x) && length(x) == 1L && !is.na(x) && nzchar(x)) {
      return(x)
    }
    NULL
  }
  
  group_var <- as_one_string_or_null(OPT$group)
  cluster_var <- as_one_string_or_null(OPT$cluster)
  sampling_weights <- as_one_string_or_null(OPT$sampling.weights)
  
  group_labels <- tryCatch(lavaan::lavInspect(fit, "group.label"),
                           error = function(e) NULL)
  
  ordered_vars <- tryCatch(lavaan::lavNames(fit, type = "ov.ord"),
                           error = function(e) character(0))
  
  # trova regr. strutturali
  pt <- lavaan::parTable(fit)
  
  is_struct <- pt$op == "~" & pt$rhs != "1" & pt$free != 0L
  struct <- pt[is_struct, c("lhs", "rhs", "group"), drop = FALSE]
  
  if (!nrow(struct)) {
    stop("No free structural regressions found.", call. = FALSE)
  }
  
  struct <- unique(struct)
  outcomes_all <- unique(struct$lhs)
  
  if (is.null(outcome)) {
    outcome_use <- outcomes_all
  } else {
    outcome_use <- unique(outcome[outcome %in% outcomes_all])
    if (!length(outcome_use)) {
      stop("None of the requested 'outcome' values are endogenous variables ",
           "with free structural predictors.", call. = FALSE)
    }
  }
  
  struct <- struct[struct$lhs %in% outcome_use, , drop = FALSE]
  
  if (!is.null(block)) {
    block <- unique(block)
    tests <- list(block)
    test_labels <- paste(block, collapse = " + ")
    test_type <- "block"
  } else {
    terms_all <- unique(struct$rhs)
    
    if (is.null(terms)) {
      term_use <- terms_all
    } else {
      term_use <- unique(terms[terms %in% terms_all])
      if (!length(term_use)) {
        stop("None of the requested 'terms' appear as free structural ",
             "predictors in the selected outcomes.", call. = FALSE)
      }
    }
    
    tests <- as.list(term_use)
    test_labels <- term_use
    test_type <- "single"
  }
  
  r2_to_df <- function(x) {
    if (is.list(x)) {
      rows <- list()
      for (g in seq_along(x)) {
        v <- x[[g]]
        if (is.null(v) || !length(v)) next
        rows[[length(rows) + 1L]] <- data.frame(
          outcome = names(v),
          group   = g,
          r2      = as.numeric(v),
          stringsAsFactors = FALSE
        )
      }
      if (length(rows)) {
        out <- do.call(rbind, rows)
      } else {
        out <- data.frame(outcome = character(0), group = integer(0),
                          r2 = numeric(0), stringsAsFactors = FALSE)
      }
    } else {
      out <- data.frame(outcome = names(x), group = 1L,
                        r2 = as.numeric(x), stringsAsFactors = FALSE)
    }
    rownames(out) <- NULL
    out
  }
  
  r2_full <- r2_to_df(lavaan::lavInspect(fit, "r2"))
  r2_full <- r2_full[r2_full$outcome %in% outcome_use, , drop = FALSE]
  
  out_rows <- list()
  idx_row  <- 0L
  
  for (y in outcome_use) {
    avail <- unique(struct$rhs[struct$lhs == y])
    
    for (j in seq_along(tests)) {
      remove_terms <- unique(tests[[j]])
      
      if (test_type == "block") {
        if (!all(remove_terms %in% avail)) next
      } else {
        if (!(remove_terms[1L] %in% avail)) next
      }
      
      idx_fix <- which(
        pt$op == "~" &
          pt$lhs == y &
          pt$rhs %in% remove_terms &
          pt$rhs != "1" &
          pt$free != 0L
      )
      
      if (!length(idx_fix)) next
      
      pt_red <- pt
      pt_red$free[idx_fix] <- 0L
      if ("est"    %in% names(pt_red)) pt_red$est[idx_fix]    <- 0
      if ("ustart" %in% names(pt_red)) pt_red$ustart[idx_fix] <- 0
      if ("start"  %in% names(pt_red)) pt_red$start[idx_fix]  <- 0
      if ("label"  %in% names(pt_red)) pt_red$label[idx_fix]  <- ""
      
      pt_red <- pt_red[pt_red$op != ":=", , drop = FALSE]
      
      refit_error <- NULL
      red_fit <- NULL
      
      if (is.null(data_in)) {
        refit_opt <- fit@Options
        refit_opt$se <- "none"
        refit_opt$test <- "none"
        
        red_fit <- tryCatch(
          lavaan::lavaan(
            model       = pt_red,
            slotData    = fit@Data,
            slotOptions = refit_opt,
            warn        = !quiet
          ),
          error = function(e) {
            refit_error <<- conditionMessage(e)
            NULL
          }
        )
      } else {
        refit_args <- list(
          model = pt_red,
          data = data_in,
          group = group_var,
          cluster = cluster_var,
          sampling.weights = sampling_weights,
          estimator = estimator,
          missing = missing_opt,
          fixed.x = OPT$fixed.x,
          conditional.x = OPT$conditional.x,
          meanstructure = OPT$meanstructure,
          ordered = if (length(ordered_vars)) ordered_vars else NULL,
          se = "none",
          test = "none",
          warn = !quiet
        )
        
        keep_arg <- !vapply(refit_args, is.null, logical(1L))
        refit_args <- refit_args[keep_arg]
        
        red_fit <- tryCatch(
          do.call(lavaan::lavaan, refit_args),
          error = function(e) {
            refit_error <<- conditionMessage(e)
            NULL
          }
        )
      }
      
      conv <- FALSE
      if (!is.null(red_fit)) {
        conv <- isTRUE(tryCatch(lavaan::lavInspect(red_fit, "converged"),
                                error = function(e) FALSE))
      }
      
      if (!conv && !quiet && !is.null(refit_error)) {
        message("lav_deltaR2: reduced model for '", y, " ~ ",
                paste(remove_terms, collapse = " + "),
                "' failed: ", refit_error)
      }
      
      if (conv) {
        r2_red <- r2_to_df(lavaan::lavInspect(red_fit, "r2"))
        r2_red <- r2_red[r2_red$outcome == y, , drop = FALSE]
      } else {
        r2_red <- data.frame(outcome = character(0), group = integer(0),
                             r2 = numeric(0), stringsAsFactors = FALSE)
      }
      
      full_sub   <- r2_full[r2_full$outcome == y, , drop = FALSE]
      all_groups <- sort(unique(full_sub$group))
      if (!length(all_groups)) all_groups <- 1L
      
      for (g in all_groups) {
        pos_full <- match(g, full_sub$group)
        pos_red  <- match(g, r2_red$group)
        
        r2f <- if (is.na(pos_full)) NA_real_ else full_sub$r2[pos_full]
        r2r <- if (is.na(pos_red))  NA_real_ else r2_red$r2[pos_red]
        if (!conv) r2r <- NA_real_
        
        delta <- if (!is.na(r2f) && !is.na(r2r)) r2f - r2r else NA_real_
        tol <- 0.5 * 10^(-digits)
        if (is.finite(delta) && abs(delta) < tol) {
          delta <- 0
        }
        
        denom <- if (!is.na(r2f)) 1 - r2f else NA_real_
        f2 <- if (!is.na(delta) && !is.na(denom) && denom > 0) {
          delta / denom
        } else { NA_real_ }
        
        f2_magnitude <- if (is.na(f2)) {
          NA_character_
        } else if (f2 < 0.02) {
          "negligible"
        } else if (f2 < 0.15) {
          "small"
        } else if (f2 < 0.35) {
          "medium"
        } else {
          "large"
        }
        
        idx_row <- idx_row + 1L
        out_rows[[idx_row]] <- data.frame(
          outcome = y,
          tested_set = if (length(test_labels) == 1L && test_type == "block") {
            test_labels
          } else { test_labels[j] },
          test_type = if (length(remove_terms) == 1L) "single" else "block",
          n_terms = length(remove_terms),
          group = g,
          group_label = if (!is.null(group_labels) && length(group_labels) >= g) {
            as.character(group_labels[g])
          } else { as.character(g) },
          r2_full = r2f,
          r2_reduced = r2r,
          delta_r2 = delta,
          part_r2 = delta,
          f2 = f2,
          f2_magnitude = f2_magnitude,
          converged = conv,
          stringsAsFactors = FALSE
        )
      }
    }
  }
  
  if (!length(out_rows)) {
    stop("No eligible reduced-model comparisons could be formed from the ",
         "requested arguments.", call. = FALSE)
  }
  
  delta_r2_table <- do.call(rbind, out_rows)
  rownames(delta_r2_table) <- NULL
  
  delta_r2_table <- delta_r2_table[
    order(delta_r2_table$group, delta_r2_table$outcome,
          delta_r2_table$tested_set), , drop = FALSE
  ]
  
  res <- list(
    delta_r2_table = delta_r2_table,
    settings = list(
      outcome = if (is.null(outcome)) NULL else outcome,
      terms = if (is.null(block)) terms else NULL,
      block = if (is.null(block)) NULL else block,
      quiet = quiet
    ),
    group_var = group_var,
    group_labels = if (is.null(group_labels)) NULL else group_labels,
    call = match.call(),
    digits = digits
  )
  
  class(res) <- "lav_deltaR2"
  return(res)
}


# --- S3 methods: print and summary ------------------------------------------

#' @param x A 'lav_deltaR2' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_deltaR2
#' @export
print.lav_deltaR2 <- function(x, ...) {
  tbl <- x$delta_r2_table
  digits <- x$digits %||% 3L
  u2 <- if (isTRUE(l10n_info()[["UTF-8"]])) "\u00B2" else "^2"
  
  if (!nrow(tbl)) {
    cat("No incremental effect sizes in delta_r2_table.\n")
    return(invisible(x))
  }
  
  fmt_num <- function(z, k = digits) {
    if (is.na(z)) "NA" else formatC(z, digits = k, format = "f")
  }
  
  outcome_txt <- as.character(tbl$outcome)
  tested_txt <- as.character(tbl$tested_set)
  group_txt <- if (!is.null(x$group_var)) {
    as.character(tbl$group_label)
  } else {
    as.character(tbl$group)
  }
  r2_full_txt <- vapply(tbl$r2_full, fmt_num, character(1L), k = digits)
  r2_red_txt <- vapply(tbl$r2_reduced, fmt_num, character(1L), k = digits)
  part_r2_txt <- vapply(tbl$part_r2, fmt_num, character(1L), k = digits)
  f2_txt <- vapply(tbl$f2, fmt_num, character(1L), k = digits)
  
  if ("f2_magnitude" %in% names(tbl)) {
    mag_txt <- ifelse(is.na(tbl$f2_magnitude), "NA", as.character(tbl$f2_magnitude))
  } else {
    mag_txt <- rep("", nrow(tbl))
  }
  
  hdr_outcome <- "Outcome"
  hdr_tested <- "Tested"
  hdr_group <- "Group"
  hdr_r2_full <- paste0("R", u2, " full")
  hdr_r2_red <- paste0("R", u2, " reduced")
  hdr_part_r2 <- paste0("part R", u2)
  hdr_f2 <- paste0("f", u2)
  hdr_mag <- "Magnitude"
  
  w_outcome <- max(nchar(c(hdr_outcome, outcome_txt), type = "width"))
  w_tested <- max(nchar(c(hdr_tested, tested_txt), type = "width"))
  w_group <- max(nchar(c(hdr_group, group_txt), type = "width"))
  w_r2_full <- max(nchar(c(hdr_r2_full, r2_full_txt), type = "width"))
  w_r2_red <- max(nchar(c(hdr_r2_red, r2_red_txt), type = "width"))
  w_part_r2 <- max(nchar(c(hdr_part_r2, part_r2_txt), type = "width"))
  w_f2 <- max(nchar(c(hdr_f2, f2_txt), type = "width"))
  w_mag <- max(nchar(c(hdr_mag, mag_txt), type = "width"))
  
  row_fmt <- paste0(
    "%-", w_outcome, "s  ",
    "%-", w_tested, "s  ",
    "%-", w_group, "s  ",
    "%",  w_r2_full, "s  ",
    "%",  w_r2_red, "s  ",
    "%",  w_part_r2, "s  ",
    "%",  w_f2, "s  ",
    "%-", w_mag, "s\n"
  )
  
  header_line <- sprintf(
    row_fmt,
    hdr_outcome,
    hdr_tested,
    hdr_group,
    hdr_r2_full,
    hdr_r2_red,
    hdr_part_r2,
    hdr_f2,
    hdr_mag
  )
  
  line_width <- nchar(sub("\n$", "", header_line), type = "width")
  rule <- paste(rep("-", line_width), collapse = "")
  
  cat("\nIncremental effect sizes from reduced-model comparisons\n")
  cat(rule, "\n", sep = "")
  cat(header_line)
  cat(rule, "\n", sep = "")
  
  for (i in seq_len(nrow(tbl))) {
    cat(sprintf(
      row_fmt,
      outcome_txt[i],
      tested_txt[i],
      group_txt[i],
      r2_full_txt[i],
      r2_red_txt[i],
      part_r2_txt[i],
      f2_txt[i],
      mag_txt[i]
    ))
  }
  
  cat(rule, "\n", sep = "")
  delta_chr <- if (isTRUE(l10n_info()[["UTF-8"]])) "\u0394" else "Delta"
  
  cat(
    paste0(
      "part R", u2, " equals ", delta_chr, "R", u2,
      ". Cohen's f", u2, " = ", delta_chr, "R", u2,
      " / (1 - R", u2, " full).\n"
    )
  )
  
  cat("\n")
  invisible(x)
}

#' @param object A 'lav_deltaR2' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_deltaR2
#' @export
summary.lav_deltaR2 <- function(object, ...) {
  print.lav_deltaR2(object, ...)
  invisible(object)
}

#' Post-Estimation Utilities for 'lavaan' Fitted Models
#' 
#' @description
#' \if{html}{\figure{logo.png}{options: style="float: right; margin: 0 0 8px 12px;" alt="lavinteract logo" width="110"}}
#' 
#' Post-estimation tools for structural equation models fitted with 'lavaan'.
#' Provides methods for probing observed and latent interactions, diagnosing local
#' misfit and multicollinearity, quantifying incremental effect sizes for 
#' structural predictors, assessing predictive performance by repeated holdout 
#' cross-validation, and adjusting selected parameter p-values for multiple 
#' testing. Functions operate from a fitted model object and, when needed, refit 
#' auxiliary or reduced models while preserving the original SEM specification.
#' 
#' @details 
#' The functions are:
#' \itemize{
#'   \item \code{\link{lav_slopes}}: simple slopes and interaction plots from a fitted 'lavaan' model.
#'   \item \code{\link{lav_jn}}: Johnson-Neyman regions of significance for continuous moderators in a fitted 'lavaan' model.
#'   \item \code{\link{lav_deltaR2}}: incremental effect sizes (part \eqn{R^2} and Cohen's \eqn{f^2}) for structural predictors via reduced-model comparisons.
#'   \item \code{\link{lav_localfit}}: residual-based local fit diagnostics and heatmaps for fitted 'lavaan' models.
#'   \item \code{\link{lav_vif}}: variance inflation factors for structural predictors with measurement preserved.
#'   \item \code{\link{lav_cv}}: repeated holdout (Monte Carlo) cross-validation of \eqn{R^2} for SEM outcomes.
#'   \item \code{\link{lav_fdr}}: false discovery rate correction for selected 'lavaan' parameter p-values.
#' }
#'
#' @section Note:
#' The development of this package grew from ongoing discussions and interactions (sic)
#' with colleagues, in particular Dr. Cataldo Giuliano Gemmano, whose steady 
#' feedback and support helped shape it. 
#'
#' @author
#' Giuseppe Corbelli (<giuseppe.corbelli@uninettunouniversity.net>)
#'
#' @docType package
#' @name lavinteract
#' @aliases lavinteract-package lavinteract
#' @keywords SEM lavaan moderation interactions diagnostics localfit residuals multicollinearity plotting R2 multipletesting validation
"_PACKAGE"

#' Simple slopes and interaction plots for fitted 'lavaan' models
#'
#' Computes conditional (simple) slopes of a focal predictor across values
#' of a moderator from a fitted 'lavaan' model that includes their explicit 
#' product term. Plots predicted lines with Wald confidence ribbons and prints
#' an APA-style test of the interaction for easy reporting and interpretation,
#' together with a simple slopes table.
#' 
#' The model should include a main effect for the predictor, a main effect for
#' the moderator, and one explicit product term between them.
#' 
#' The moderator must enter the fitted model as a single numeric
#' variable, which may be continuous, a binary observed moderator coded as a
#' single numeric dummy variable, an observed numeric moderator with a small
#' number of values treated as discrete probe points, or a latent moderator
#' treated as a single continuous latent variable. 
#' 
#' Standard errors use the delta method with the model covariance matrix of the
#' estimates. When moderator values are derived automatically for latent
#' moderators, probe points are based on the estimated latent mean and
#' model-implied latent standard deviation.
#' 
#' @usage
#' lav_slopes(
#'   fit,
#'   outcome,
#'   pred,
#'   modx,
#'   interaction,
#'   data = NULL,
#'   modx.values = NULL,
#'   modx.labels = NULL,
#'   pred.range = NULL,
#'   conf.level = 0.95,
#'   x.label = NULL,
#'   y.label = NULL,
#'   legend.title = NULL,
#'   colors = NULL,
#'   line.size = 0.80,
#'   alpha = 0.20,
#'   table = TRUE,
#'   digits = 3L,
#'   modx_n_unique_cutoff = 4L,
#'   return_data = FALSE
#' )
#'
#' @param fit A fitted 'lavaan' object that includes the product term (required).
#' @param outcome Character. Name of the dependent variable in \code{fit} (required).
#' @param pred Character. Name of the focal predictor whose simple slopes are probed (required).
#' @param modx Character. Name of the moderator. The moderator must appear in
#' the fitted model as a single numeric variable. General nominal moderators
#' with more than two categories are not supported.
#' @param interaction Character. Name of the product term in \code{fit} (e.g., \code{"X_Z"}) (required).
#' @param data \code{data.frame}. Optional raw data (not needed, retained for backward compatibility). 
#' The function automatically recovers observed data from \code{fit} when needed.   
#' @param modx.values Numeric vector of moderator values at which to compute
#' simple slopes. If \code{NULL} and \code{modx} is observed and numeric, the
#' function uses mean minus 1 SD, the mean, and mean plus 1 SD for moderators
#' with more than \code{modx_n_unique_cutoff} unique values, and otherwise
#' uses the observed numeric values as discrete probe points. If \code{modx}
#' is latent in a single-group model, the function uses the estimated latent
#' mean minus 1 latent SD, the latent mean, and the latent mean plus 1 latent
#' SD. 
#' @param modx.labels Character vector. Legend and table labels for \code{modx.values}.
#' By default, the labels are \code{c("-1 SD", "Mean", "+1 SD")} when values are derived as 
#' mean plus or minus 1 SD, and \code{as.character(modx.values)} otherwise.
#' @param pred.range Numeric vector of length 2. Range \code{c(min, max)} for the x-axis
#' for the focal predictor. If \code{NULL} and \code{pred} is observed, the observed 
#' range recovered from \code{fit} is used. If \code{pred} is latent in a single-group model, 
#' the function uses the estimated latent mean minus 2 latent SD and the latent mean plus 2 
#' latent SD. Otherwise, \code{c(-2, 2)} is used.
#' @param conf.level Numeric in (0,1). Confidence level for Wald confidence intervals and ribbons (default: 0.95).
#' @param x.label Character. X-axis label (default: \code{pred}).
#' @param y.label Character. Y-axis label (default: \code{outcome}).
#' @param legend.title Character. Legend title; if \code{NULL}, the legend shows only levels (default: NULL).
#' @param colors Character vector. Colors for lines and ribbons; named vector recommended with names matching \code{modx.labels} (default: Okabe-Ito palette).
#' @param line.size Numeric > 0. Line width (default: 0.80).
#' @param alpha Numeric in (0,1). Ribbon opacity (default 0.20).
#' @param table Logical. Print APA-style interaction test and simple-slopes table (default: \code{TRUE}).
#' @param digits Integer \code{>= 0}. Decimal digits in printed output (default: 3).
#' @param modx_n_unique_cutoff Integer \code{>= 1}. Threshold for treating a numeric moderator
#' as continuous and using mean ± SD (default: 4).
#' @param return_data Logical. If \code{TRUE}, include the plotting data.frame in the returned list (default: FALSE).
#'
#' @return A list of class \code{"lav_slopes"} with elements:
#' \describe{
#'   \item{\code{plot}}{\code{ggplot} object with lines and confidence ribbons.}
#'   \item{\code{slope_table}}{Data frame with moderator levels, simple slopes, SE, z, and CI.}
#'   \item{\code{plot_data}}{Only when \code{return_data = TRUE}: data used to build the plot.}
#' }
#'
#' @section Notes:
#' Estimates are unstandardized; a standardized coefficient for the interaction is
#' also reported for reference. Wald tests assume large-sample normality of the
#' parameter estimates. Multigroup fitted models are not supported. 
#' 
#' @references
#' Aiken, L. S., & West, S. G. (1991). \emph{Multiple regression: Testing and
#' interpreting interactions}. Sage.
#'
#' Preacher, K. J., Curran, P. J., & Bauer, D. J. (2006). Computational tools
#' for probing interactions in multiple linear regression, multilevel modeling,
#' and latent curve analysis. \emph{Journal of Educational and Behavioral
#' Statistics}, \emph{31}(4), 437-448.
#' \doi{10.3102/10769986031004437}
#'
#' Rogosa, D. (1980). Comparing nonparallel regression lines.
#' \emph{Psychological Bulletin}, \emph{88}(2), 307-321.
#' \doi{10.1037/0033-2909.88.2.307}
#'
#' @seealso \code{\link{lav_jn}} for Johnson-Neyman regions of significance.
#' 
#' @examples
#' set.seed(42)
#' X <- rnorm(100); Z <- rnorm(100); X_Z <- X*Z
#' Y <- 0.6*X + 0.6*Z + 0.3*X_Z + rnorm(100, sd = 0.7) 
#' dataset <- data.frame(Y, X, Z, X_Z)
#' fit <- lavaan::sem("Y ~ X + Z + X_Z", data = dataset)
#' lav_slopes(
#' fit = fit, 
#' outcome = "Y", 
#' pred = "X", 
#' modx = "Z", 
#' interaction = "X_Z")
#'
#' @importFrom lavaan parameterEstimates parTable lavInspect lavNames
#' @importFrom stats vcov qnorm sd pnorm
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon scale_colour_manual scale_fill_manual labs
#' @importFrom rlang sym %||%
#' @export
lav_slopes <- function(
    fit,
    outcome,
    pred,
    modx,
    interaction,
    data = NULL,
    modx.values = NULL,
    modx.labels = NULL,
    pred.range = NULL,
    conf.level = 0.95,
    x.label = NULL,
    y.label = NULL,
    legend.title = NULL,
    colors = NULL,
    line.size = 0.80,
    alpha = 0.20,
    table = TRUE,
    digits = 3L,
    modx_n_unique_cutoff = 4L,
    return_data = FALSE
) {
  if (!inherits(fit, "lavaan")) stop("'fit' must be a 'lavaan' object.", call. = FALSE)
  if (missing(interaction) || is.null(interaction))
    stop("'interaction' (product-term name) must be supplied.", call. = FALSE)
  
  ngroups <- tryCatch(lavaan::lavInspect(fit, "ngroups"), error = function(e) 1L)
  if (length(ngroups) != 1L || is.na(ngroups)) ngroups <- 1L
  if (ngroups > 1L) {
    stop("'lav_slopes' does not currently support multigroup fitted models. Please fit one group at a time.", call. = FALSE)
  }
  
# recupera i dati!!!
  fetch_data <- function(f) {
    out <- tryCatch(lavaan::lavInspect(f, "data"), error = function(e) NULL)
    if (is.null(out)) {
      out <- tryCatch(lavaan::lavInspect(f, "data.original"), error = function(e) NULL)
    }
    if (is.null(out)) {
      out <- tryCatch(f@Data@X, error = function(e) NULL)
    }
    if (is.null(out)) {
      return(NULL)
    }
    ov_names <- tryCatch(lavaan::lavNames(f, type = "ov"), error = function(e) NULL)
    if (is.data.frame(out)) {
      out <- as.data.frame(out)
      return(out)
    }
    if (is.matrix(out)) {
      out <- as.data.frame(out)
      if (!is.null(ov_names) && length(ov_names) == ncol(out)) {
        names(out) <- ov_names
      }
      return(out)
    }
    if (is.list(out)) {
      ok_list <- all(vapply(out, function(x) is.data.frame(x) || is.matrix(x), logical(1)))
      if (!ok_list) {
        return(NULL)
      }
      group_labels <- tryCatch(lavaan::lavInspect(f, "group.label"), error = function(e) NULL)
      group_var <- NULL
      group_call <- tryCatch(f@call$group, error = function(e) NULL)
      if (!is.null(group_call)) {
        group_var <- paste(as.character(group_call), collapse = "")
        group_var <- gsub('^"|"$', "", group_var)
      }
      if (is.null(group_var) || length(group_var) != 1L || is.na(group_var) || !nzchar(group_var)) {
        tmp_group <- tryCatch(f@Options$group, error = function(e) NULL)
        if (length(tmp_group) == 1L && !is.na(tmp_group) && nzchar(tmp_group)) {
          group_var <- tmp_group
        } else {
          group_var <- NULL
        }
      }
      out_list <- vector("list", length(out))
      for (g in seq_along(out)) {
        dg <- out[[g]]
        if (is.matrix(dg)) {
          dg <- as.data.frame(dg)
          if (!is.null(ov_names) && length(ov_names) == ncol(dg)) {
            names(dg) <- ov_names
          }
        } else {
          dg <- as.data.frame(dg)
        }
        if (!is.null(group_var) && nzchar(group_var) && !(group_var %in% names(dg))) {
          if (!is.null(group_labels) && length(group_labels) >= g) {
            lab_g <- group_labels[g]
          } else {
            lab_g <- g
          }
          num_lab <- suppressWarnings(as.numeric(lab_g))
          if (!is.na(num_lab)) {
            dg[[group_var]] <- num_lab
          } else {
            dg[[group_var]] <- as.character(lab_g)
          }
        }
        out_list[[g]] <- dg
      }
      out <- do.call(rbind, out_list)
      rownames(out) <- NULL
      return(out)
    }
    NULL
  }
  dat <- fetch_data(fit)
  
  # solo per compatibilita
  if (is.null(dat) && !is.null(data)) {
    dat <- as.data.frame(data)
  }
  
  # estrazione pick a point delle latenti in automatico!!!
  lv_names <- tryCatch(lavaan::lavNames(fit, type = "lv"), error = function(e) character(0))
  # media e ds dilatente dal modello stimato:
  prendi_media_sd_latente <- function(fit, lv) {
    mean_lv <- tryCatch(lavaan::lavInspect(fit, "mean.lv"), error = function(e) NULL)
    cov_lv  <- tryCatch(lavaan::lavInspect(fit, "cov.lv"), error = function(e) NULL)
    if (is.list(mean_lv)) {
      mean_lv <- mean_lv[[1L]]
    }
    if (is.list(cov_lv)) {
      cov_lv <- cov_lv[[1L]]
    }
    if (is.matrix(mean_lv) && nrow(mean_lv) >= 1L) {
      tmp <- as.numeric(mean_lv[1L, ])
      names(tmp) <- colnames(mean_lv)
      mean_lv <- tmp
    }
    mu <- NA_real_
    var_lv <- NA_real_
    if (!is.null(mean_lv) && length(mean_lv) > 0L && lv %in% names(mean_lv)) {
      mu <- as.numeric(mean_lv[lv])
    }
    if (!is.null(cov_lv) && is.matrix(cov_lv)) {
      rn <- rownames(cov_lv)
      cn <- colnames(cov_lv)
      if (!is.null(rn) && !is.null(cn) && lv %in% rn && lv %in% cn) {
        var_lv <- as.numeric(cov_lv[lv, lv])
      }
    }
    if (!is.finite(mu)) {
      mu <- 0
    }
    if (!is.finite(var_lv) || var_lv < 0) {
      return(NULL)
    }
    list(mean = mu, sd = sqrt(var_lv))
  }
# estrai moderatori e label 
  if (is.null(modx.values)) {
    # caso 1: moderatore osservato!!!
    if (!is.null(dat) && modx %in% names(dat)) {
      z <- dat[[modx]]
      if (is.numeric(z) && length(unique(z)) > modx_n_unique_cutoff) {
        m <- mean(z, na.rm = TRUE)
        s <- sd(z, na.rm = TRUE)
        modx.values <- round(c(m - s, m, m + s), 2)
        modx.labels <- modx.labels %||% c("-1 SD", "Mean", "+1 SD")
      } else {
        modx.values <- sort(unique(z))
        modx.labels <- modx.labels %||% as.character(modx.values)
      }
      # caso 2: moderatore latente!!!!
    } else if (modx %in% lv_names) {
      lv_info <- prendi_media_sd_latente(fit, modx)
      if (is.null(lv_info)) {
        stop(
          "Could not derive latent mean and variance for moderator `", modx, "`. ",
          "Supply `modx.values` explicitly.",
          call. = FALSE
        )
      }
      modx.values <- round(
        c(
          lv_info$mean - lv_info$sd,
          lv_info$mean,
          lv_info$mean + lv_info$sd
        ),
        2
      )
      modx.labels <- modx.labels %||% c("-1 SD", "Mean", "+1 SD")
      # caso 3: moderatore non trovato!!!!!!!
    } else {
      stop(
        "`modx` was not found as either an observed or latent variable in `fit`. ",
        "Supply a valid moderator name or specify `modx.values` explicitly.",
        call. = FALSE
      )
    }
  }
  if (!is.null(modx.labels) && length(modx.labels) != length(modx.values))
    stop("'modx.labels' must match length of 'modx.values'.", call. = FALSE)
  modx.labels <- modx.labels %||% as.character(modx.values)
  
  if (!is.numeric(modx.values)) {
    stop(
      "'modx.values' must be numeric. General nominal moderators with more than two categories are not supported; binary observed moderators must be represented as a single numeric dummy-coded variable.",
      call. = FALSE
    )
  }

# range del predittore focale:
  if (is.null(pred.range)) {
    if (!is.null(dat) && pred %in% names(dat)) {
      pred.range <- range(dat[[pred]], na.rm = TRUE)
    } else if (pred %in% lv_names) {
      lv_info_pred <- prendi_media_sd_latente(fit, pred)
      if (!is.null(lv_info_pred)) {
        pred.range <- c(
          lv_info_pred$mean - 2 * lv_info_pred$sd,
          lv_info_pred$mean + 2 * lv_info_pred$sd
        )
      } else {
        pred.range <- c(-2, 2)
      }
    } else {
      pred.range <- c(-2, 2)
    }
  }
  x_seq <- seq(pred.range[1L], pred.range[2L], length.out = 1000L)
  
# stime dei parametri e vcov:
  pe <- lavaan::parameterEstimates(fit, standardized = FALSE)
  pt <- lavaan::parTable(fit)
  vc <- tryCatch(stats::vcov(fit),
                 error = function(e) lavaan::lavInspect(fit, "vcov"))
  if (!is.matrix(vc)) stop("Could not retrieve covariance matrix from 'fit'.", call. = FALSE)
  
  find_par <- function(lhs, rhs, op = "~", rhs_regex = NULL,
                       required = TRUE, default_est = 0) {
    idx_pe <- which(pe$lhs == lhs & pe$rhs == rhs & pe$op == op)
    if (length(idx_pe) == 0L && !is.null(rhs_regex))
      idx_pe <- which(pe$lhs == lhs & pe$op == op & grepl(rhs_regex, pe$rhs))
    if (length(idx_pe) == 0L) {
      if (!required) { est_val <- default_est; idx_pe <- NA_integer_ } else {
        stop(sprintf("Parameter '%s %s %s' not found.", lhs, op, rhs), call. = FALSE)
      }
    } else {
      idx_pe  <- idx_pe[1L]; est_val <- pe$est[idx_pe]
    }
    idx_pt <- which(pt$lhs == lhs & pt$rhs == rhs & pt$op == op)
    if (length(idx_pt) == 0L && !is.null(rhs_regex))
      idx_pt <- which(pt$lhs == lhs & pt$op == op & grepl(rhs_regex, pt$rhs))
    free_idx <- if (length(idx_pt) == 0L) 0L else {
      fi <- pt$free[idx_pt[1L]]; if (is.na(fi)) 0L else as.integer(fi)
    }
    list(est = est_val, free = free_idx, idx_pe = idx_pe)
  }
  
# estrai inetercetta + effetti principali + interazione
  p0 <- find_par(outcome, "",   op = "~1", required = FALSE, default_est = 0)
  p1 <- find_par(outcome, pred, op = "~")
  p2 <- find_par(outcome, modx, op = "~")
  p3 <- find_par(outcome, interaction, op = "~",
                 rhs_regex = paste0("(", pred, ").*(", modx, ")|(",
                                    modx, ").*(", pred, ")"))
  
  safe_free <- function(x) { if (is.na(x) || length(x) == 0L) 0L else as.integer(x) }
  p0$free <- safe_free(p0$free); p1$free <- safe_free(p1$free)
  p2$free <- safe_free(p2$free); p3$free <- safe_free(p3$free)
  
  vc_ <- function(a, b) {
    a <- safe_free(a); b <- safe_free(b)
    if (a <= 0L || b <= 0L) return(0)
    if (a > nrow(vc) || b > ncol(vc)) return(0)
    val <- vc[a, b]; if (is.na(val)) 0 else val
  }
  
  var_b1   <- vc_(p1$free, p1$free)
  var_b3   <- vc_(p3$free, p3$free)
  cov_b1b3 <- vc_(p1$free, p3$free)
  
  var_b0   <- vc_(p0$free, p0$free)
  var_b2   <- vc_(p2$free, p2$free)
  cov_b0b2 <- vc_(p0$free, p2$free)
  cov_b0b1 <- vc_(p0$free, p1$free)
  cov_b0b3 <- vc_(p0$free, p3$free)
  cov_b1b2 <- vc_(p1$free, p2$free)
  cov_b2b3 <- vc_(p2$free, p3$free)
  
# slope e anche predizioni
  z_crit <- stats::qnorm(1 - (1 - conf.level) / 2)
  slope_tbl <- data.frame(
    Moderator = modx.labels,
    Mod_value = modx.values,
    Slope = NA_real_, SE = NA_real_, z = NA_real_,
    CI_low = NA_real_, CI_high = NA_real_,
    stringsAsFactors = FALSE
  )
  plot_df <- NULL
  
  for (i in seq_along(modx.values)) {
    z0 <- modx.values[i]
    
    slope     <- p1$est + p3$est * z0
    var_slope <- var_b1 + 2 * z0 * cov_b1b3 + (z0 ^ 2) * var_b3
    se_slope  <- sqrt(max(var_slope, 0))
    
    slope_tbl$Slope[i] <- slope
    slope_tbl$SE[i] <- se_slope
    slope_tbl$z[i] <- if (se_slope > 0) slope / se_slope else NA_real_
    slope_tbl$CI_low[i] <- slope - z_crit * se_slope
    slope_tbl$CI_high[i] <- slope + z_crit * se_slope
    
    intercept <- p0$est + p2$est * z0
    var_int <- max(var_b0 + 2 * z0 * cov_b0b2 + (z0 ^ 2) * var_b2, 0)
    
    y_hat <- intercept + slope * x_seq
    se_yhat <- sqrt(pmax((x_seq ^ 2) * var_slope + var_int +
                           2 * x_seq * (cov_b0b1 + z0 * cov_b0b3 +
                                          z0 * cov_b1b2 + (z0 ^ 2) * cov_b2b3), 0))
    
    plot_df <- rbind(plot_df,
                     data.frame(
                       X = x_seq, Y_hat = y_hat,
                       CI_low = y_hat - z_crit * se_yhat,
                       CI_high= y_hat + z_crit * se_yhat,
                       LegendGroup = factor(modx.labels[i], levels = modx.labels),
                       stringsAsFactors = FALSE
                     ))
  }
  
# colori okabe ito
  if (is.null(colors)) {
    okabe_ito <- c("#000000", "#E69F00", "#56B4E9", "#009E73",
                   "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
    colors <- okabe_ito[seq_along(modx.labels)]
  }
  if (is.null(names(colors)) || !all(modx.labels %in% names(colors)))
    names(colors) <- modx.labels
  
# plotting con ggplot 
  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(
      x = !!rlang::sym("X"),
      y = !!rlang::sym("Y_hat"),
      colour = !!rlang::sym("LegendGroup"),
      fill   = !!rlang::sym("LegendGroup")
    )
  ) +
    ggplot2::geom_line(linewidth = line.size) +
    ggplot2::geom_ribbon(
      ggplot2::aes(
        ymin = !!rlang::sym("CI_low"),
        ymax = !!rlang::sym("CI_high")
      ),
      alpha = alpha, colour = NA
    ) +
    ggplot2::scale_colour_manual(values = colors, name = legend.title) +
    ggplot2::scale_fill_manual(values = colors, name = legend.title) +
    ggplot2::labs(x = x.label %||% pred, y = y.label %||% outcome)
  
  
  # output in console 
  
  # risultati per gli S3
  interaction_test <- list(
    b = p3$est,
    se = if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_,
    z = {
      se <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      if (!is.na(se) && se > 0) p3$est / se else NA_real_
    },
    p = {
      se <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      if (!is.na(se) && se > 0) 2 * stats::pnorm(abs(p3$est / se), lower.tail = FALSE) else NA_real_
    },
    ci = {
      se <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      c(p3$est - z_crit * se, p3$est + z_crit * se)
    },
    beta_std = {
      pe_std <- lavaan::parameterEstimates(fit, standardized = TRUE)
      idx_std <- which(pe_std$lhs == outcome & pe_std$rhs == interaction & pe_std$op == "~")
      if (length(idx_std)) pe_std$std.all[idx_std[1L]] else NA_real_
    }
  )
  
  res <- list(
    plot = p,
    slope_table  = slope_tbl,
    labels = list(outcome = outcome, pred = pred, modx = modx, interaction = interaction),
    conf.level = conf.level,
    digits = digits,
    interaction = interaction_test,
    print_table = table,
    call = match.call()
  )
  if (isTRUE(return_data)) res$plot_data <- plot_df
  
  class(res) <- "lav_slopes"
  return(res)
}

### metodi S3 print e summary:

#' @param x A 'lav_slopes' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_slopes
#' @export
print.lav_slopes <- function(x, ...) {
  lab <- x$labels
  it <- x$interaction
  digits <- x$digits
  cl <- x$conf.level
  
  fmt <- function(v, k = digits) {
    if (is.na(v)) "NA" else formatC(v, format = "f", digits = k)
  }
  
  fmtp <- function(p, k = digits) {
    if (is.na(p)) return("NA")
    cutoff <- 10^(-k)
    cutoff_txt <- sub("^0\\.", ".", formatC(cutoff, digits = k, format = "f"))
    if (p < cutoff) return(paste0("< ", cutoff_txt))
    out <- formatC(p, digits = k, format = "f")
    sub("^0\\.", ".", out)
  }
  
  zfmt <- function(z) {
    if (is.na(z)) "NA" else formatC(z, format = "f", digits = digits)
  }
  
  cat(
    "\nInteraction effect (", lab$pred, " * ", lab$modx, " -> ", lab$outcome, "): ",
    "b = ", fmt(it$b), ", SE = ", fmt(it$se),
    ", beta = ", fmt(it$beta_std), ", z = ", zfmt(it$z),
    ", p = ", fmtp(it$p),
    ", ", sprintf("%.0f%% CI", cl * 100), " [", fmt(it$ci[1L]), ", ", fmt(it$ci[2L]), "]\n",
    sep = ""
  )
  
  if (isTRUE(x$print_table)) {
    tbl <- x$slope_table
    
    cat(
      "\nSimple slopes of", lab$pred, "predicting", lab$outcome,
      "at levels of", lab$modx, sprintf("(%.0f%% CI)", cl * 100), "\n",
      sep = " "
    )
    
    mod_txt <- as.character(tbl$Moderator)
    modval_txt <- as.character(tbl$Mod_value)
    slope_txt <- vapply(tbl$Slope, fmt, character(1L))
    se_txt <- vapply(tbl$SE, fmt, character(1L))
    z_txt <- vapply(tbl$z, zfmt, character(1L))
    ci_low_txt <- vapply(tbl$CI_low, fmt, character(1L))
    ci_high_txt <- vapply(tbl$CI_high, fmt, character(1L))
    ci_txt <- paste0("[", ci_low_txt, ", ", ci_high_txt, "]")
    
    hdr_mod <- "Moderator"
    hdr_modval <- "Mod. value"
    hdr_slope <- "Slope"
    hdr_se <- "SE"
    hdr_z <- "z"
    hdr_ci <- sprintf("%.0f%% CI", cl * 100)
    
    w_mod <- max(nchar(c(hdr_mod, mod_txt), type = "width"))
    w_modval <- max(nchar(c(hdr_modval, modval_txt), type = "width"))
    w_slope <- max(nchar(c(hdr_slope, slope_txt), type = "width"))
    w_se <- max(nchar(c(hdr_se, se_txt), type = "width"))
    w_z <- max(nchar(c(hdr_z, z_txt), type = "width"))
    w_ci <- max(nchar(c(hdr_ci, ci_txt), type = "width"))
    
    row_fmt <- paste0(
      "%-", w_mod, "s  ",
      "%-", w_modval, "s  ",
      "%",  w_slope, "s  ",
      "%",  w_se, "s  ",
      "%",  w_z, "s  ",
      "%-", w_ci, "s\n"
    )
    
    header_line <- sprintf(
      row_fmt,
      hdr_mod, hdr_modval, hdr_slope, hdr_se, hdr_z, hdr_ci
    )
    
    rule_width <- nchar(sub("\n$", "", header_line), type = "width") + 3L
    rule <- paste(rep("-", rule_width), collapse = "")
    
    cat(rule, "\n", sep = "")
    cat(header_line)
    cat(rule, "\n", sep = "")
    
    for (i in seq_len(nrow(tbl))) {
      cat(sprintf(
        row_fmt,
        mod_txt[i],
        modval_txt[i],
        slope_txt[i],
        se_txt[i],
        z_txt[i],
        ci_txt[i]
      ))
    }
    
    cat(rule, "\n", sep = "")
  }
  
  cat("\n")
  print(x$plot)
  invisible(x)
}

#' @param object A 'lav_slopes' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_slopes
#' @export
summary.lav_slopes <- function(object, ...) {
  print.lav_slopes(object, ...)
  invisible(object)
}

#' Repeated holdout (Monte Carlo) cross-validation of \eqn{R^2} for structural equation models ('lavaan' objects)
#'
#' Estimate out-of-sample predictive performance for structural relations in a
#' fitted 'lavaan' model using repeated holdout (Monte Carlo cross-validation,
#' leave-group-out CV). At each repetition, the model is refitted on a random
#' training subset and evaluated on a disjoint test subset.
#'
#' For observed outcomes, \eqn{R^2} is computed by comparing test-set observed values
#' with predictions obtained by applying the training-set structural coefficients
#' to the test-set predictors.
#' 
#' For latent outcomes, the outcome is not directly observed in the test set.
#' Factor scores for the outcome are first computed in the test set using the
#' measurement model learned on the training set; these scores serve as the
#' outcome values. Predictions are then formed by applying the training-set
#' structural coefficients to the test-set predictors (including factor scores
#' for any latent predictors). \eqn{R^2} is computed by comparing the test-set factor
#' scores of the outcome with these predicted scores.
#' 
#' The in-sample baseline \eqn{R^2} is computed on the full dataset using the same
#' metric as in cross-validation: observed outcomes use observed-versus-predicted
#' \eqn{R^2}; latent outcomes use score-versus-predicted-score R^2.
#' 
#' By default, repetitions continue until the running mean \eqn{R^2} for each outcome
#' stabilizes within a specified tolerance over a trailing window of successful
#' splits, or until a maximum number of splits is reached.
#' 
#' The summary table reports the in-sample baseline \eqn{R^2}, the median cross-validated
#' \eqn{R^2}, its standard deviation, and the percent drop (baseline vs. median CV) with
#' heuristic threshold markers. The percent drop is suppressed when the in-sample
#' \eqn{R^2} is very small. 
#'
#' @usage
#' lav_cv(
#'   fit,
#'   data = NULL,
#'   times = "auto",
#'   train_prop = 0.8,
#'   seed = 42L,
#'   quiet = TRUE,
#'   digits = 3L,
#'   plot = TRUE,
#'   tol = 0.001,
#'   window = 50L,
#'   max_times = 3000L,
#'   min_r2_for_pct = 0.05
#' )
#'
#' @param fit A fitted 'lavaan' object (required).
#' @param data The data frame used to fit the model; if NULL, it is extracted from 'fit' when available (default: NULL).
#' @param times Integer indicating the number of random splits, or "auto" for stabilization-based early stopping (default: "auto").
#' @param train_prop Numeric in (0,1). Proportion of cases in the training split for each repetition (default: 0.8).
#' @param seed Integer. Random seed for reproducibility of the splits (default: 42).
#' @param quiet Logical. Suppress 'lavaan' refit messages when TRUE (default: TRUE).
#' @param digits Integer. Number of digits to print in summaries (default: 3).
#' @param plot Logical. Show convergence plots of the running mean \eqn{R^2} per outcome (default: TRUE).
#' @param tol Numeric. Tolerance for the auto-stop rule on the running mean (default: 0.001).
#' @param window Integer. Trailing window size (number of successful splits) used by the auto-stop rule (default: 50).
#' @param max_times Integer. Maximum number of splits when \code{times} = "auto" (default: 3000).
#' @param min_r2_for_pct Numeric in (0,1). Minimum in-sample \eqn{R^2} required to compute percent drop; below this, \%_drop is set to NA (default: 0.05).
#'
#' @return A list with class 'lav_cv' and elements:
#' \describe{
#'   \item{\code{table}}{Data frame with columns:
#'     \code{outcome}, \code{type} ("observed" or "latent"),
#'     \code{r2_in}, \code{r2_cv_mean}, \code{r2_cv_median}, \code{r2_cv_sd},
#'     \code{drop_mean_pct}, \code{drop_med_pct}, \code{splits_used}.}
#'   \item{\code{split_matrix}}{Matrix of split-wise test-set \eqn{R^2} values (rows = splits, columns = outcomes).}
#'   \item{\code{times}}{Character or integer indicating the number of splits used (e.g., \code{"auto(534)"} or \code{500}).}
#'   \item{\code{train_prop}}{Numeric. Training proportion used in each split.}
#'   \item{\code{N}}{Integer. Number of rows in the input data.}
#'   \item{\code{seed}}{Integer. Random seed used to generate the splits.}
#'   \item{\code{tol}}{Numeric. Tolerance used by the auto-stop rule.}
#'   \item{\code{window}}{Integer. Trailing window size for the auto-stop rule.}
#'   \item{\code{min_r2_for_pct}}{Numeric. Minimum in-sample \eqn{R^2} required to compute percent drop.}
#'   \item{\code{call}}{\code{match.call()} of the function call.}
#'   \item{\code{digits}}{Integer. Default number of digits for printing.}
#' }
#' 
#' @seealso \code{\link[lavaan]{sem}}, \code{\link[lavaan]{lavPredict}},
#'   \code{\link[lavaan]{inspect}}
#'   
#' @references
#' Cudeck, R., & Browne, M. W. (1983). Cross-validation of covariance
#' structures. \emph{Multivariate Behavioral Research}, \emph{18}(2), 147-167.
#' \doi{10.1207/s15327906mbr1802_2}
#'
#' Hastie, T., Tibshirani, R., & Friedman, J. (2001). \emph{The elements of
#' statistical learning: Data mining, inference, and prediction}. Springer.
#' \doi{10.1007/978-0-387-21606-5}
#'
#' Kvålseth, T. O. (1985). Cautionary note about \eqn{R^2}.
#' \emph{The American Statistician}, \emph{39}(4), 279-285.
#' \doi{10.1080/00031305.1985.10479448}
#'
#' Shmueli, G. (2010). To explain or to predict?
#' \emph{Statistical Science}, \emph{25}(3), 289-310.
#' \doi{10.1214/10-STS330}
#'
#' Yarkoni, T., & Westfall, J. (2017). Choosing prediction over explanation in
#' psychology: Lessons from machine learning. \emph{Perspectives on
#' Psychological Science}, \emph{12}(6), 1100-1122.
#' \doi{10.1177/1745691617693393}
#'
#' @examples 
#' set.seed(42)
#' model <- "
#' ind60 =~ x1 + x2 + x3
#' dem60 =~ y1 + y2 + y3 + y4
#' dem65 =~ y5 + y6 + y7 + y8
#' 
#' dem60 ~ ind60
#' dem65 ~ ind60 + dem60
#' 
#' y1 ~~ y5
#' y2 ~~ y6
#' "
#' fit <- lavaan::sem(
#' model = model, 
#' data = lavaan::PoliticalDemocracy,
#' std.lv = TRUE, 
#' estimator = "MLR", 
#' meanstructure = TRUE)
#' result <- lav_cv(
#' fit = fit, 
#' data = lavaan::PoliticalDemocracy, 
#' times = 5)
#' print(result)
#'
#' @importFrom rlang %||%
#' @export
lav_cv <- function(
    fit,
    data = NULL,
    times = "auto",
    train_prop = 0.8,
    seed = 42L,
    quiet = TRUE,
    digits = 3L,
    plot = TRUE,
    tol = 0.001,
    window = 50L,
    max_times = 3000L,
    min_r2_for_pct = 0.05   
) {
  if (!inherits(fit, "lavaan")) stop("`fit` must be a fitted lavaan object.", call. = FALSE)
  if (!(is.numeric(times) || identical(times, "auto"))) stop("`times` must be integer or 'auto'.", call. = FALSE)
  if (train_prop <= 0 || train_prop >= 1) stop("`train_prop` must be in (0,1).", call. = FALSE)
  OPT <- fit@Options
  if (!is.null(OPT$group) && nzchar(OPT$group)) stop("Multigroup models are not supported yet.", call. = FALSE)
  
  data_in <- data %||%
    tryCatch(lavaan::lavInspect(fit, "data"), error = function(e) NULL) %||%
    tryCatch(lavaan::lavInspect(fit, "data.original"), error = function(e) NULL)
  if (is.null(data_in)) stop("Could not locate data; pass it via `data =`.", call. = FALSE)
  if (is.matrix(data_in)) data_in <- as.data.frame(data_in)
  
  pt <- lavaan::parTable(fit)
  is_struct <- pt$op == "~" & pt$rhs != "1"
  is_interaction <- grepl(":", pt$rhs, fixed = TRUE) | pt$op == "XWITH"
  struct <- pt[is_struct & !is_interaction, c("lhs", "rhs")]
  outcomes_all <- unique(struct$lhs)
  if (!length(outcomes_all)) stop("No structural regressions found.", call. = FALSE)
  
  latents_all <- tryCatch(lavaan::lavNames(fit, type = "lv"), error = function(e) character(0))
  outcome_is_latent <- outcomes_all %in% latents_all
  any_latents <- length(latents_all) > 0L
  
  estimator_use <- OPT$estimator
  missing_opt <- OPT$missing
  std_lv <- isTRUE(OPT$std.lv)
  parameterization <- OPT$parameterization
  ordered_vars <- tryCatch(lavaan::lavNames(fit, type = "ov.ord"), error = function(e) character(0))
  model_obj <- lavaan::parTable(fit)
  
  r2_fun <- function(y, yhat) {
    ok <- is.finite(y) & is.finite(yhat)
    y <- y[ok]; yhat <- yhat[ok]
    if (length(y) < 3L) return(NA_real_)
    ss_res <- sum((y - yhat)^2)
    ss_tot <- sum((y - mean(y))^2)
    if (ss_tot <= 0) return(NA_real_)
    1 - ss_res / ss_tot
  }
  get_struct_coefs <- function(f) {
    pe <- lavaan::parameterEstimates(f, standardized = FALSE)
    out <- lapply(outcomes_all, function(y) {
      b  <- pe[pe$op == "~" & pe$lhs == y, c("rhs", "est")]
      ic <- pe[pe$op == "~1" & pe$lhs == y, "est"]; if (!length(ic)) ic <- 0
      list(betas = stats::setNames(b$est, b$rhs), intercept = as.numeric(ic))
    })
    names(out) <- outcomes_all
    out
  }
  compute_lv_scores <- function(fit_obj, newdata) {
    if (!any_latents) return(NULL)
    fs <- tryCatch(lavaan::lavPredict(fit_obj, newdata = newdata, type = "lv", method = "regression"),
                   error = function(e) NULL)
    if (is.null(fs)) fs <- tryCatch(lavaan::lavPredict(fit_obj, newdata = newdata, type = "lv", method = "Bartlett"),
                                    error = function(e) NULL)
    fs
  }
  build_X <- function(pred_names, fs, newdata) {
    M <- NULL
    if (any_latents && !is.null(fs)) {
      X_lat <- intersect(pred_names, colnames(fs))
      if (length(X_lat)) M <- cbind(M, fs[, X_lat, drop = FALSE])
    }
    X_obs <- intersect(pred_names, colnames(newdata))
    if (length(X_obs)) M <- cbind(M, as.matrix(newdata[, X_obs, drop = FALSE]))
    M
  }
  
  # estrazione R^2 di baseline
  coefs_full <- get_struct_coefs(fit)
  fs_full <- if (any_latents) compute_lv_scores(fit, data_in) else NULL
  baseline_r2 <- stats::setNames(rep(NA_real_, length(outcomes_all)), outcomes_all) # ok call diretta da namespace
  for (y in outcomes_all) {
    cY <- coefs_full[[y]]
    preds <- names(cY$betas)
    X <- build_X(preds, fs_full, data_in)
    if (!is.null(X)) {
      yhat <- drop(cY$intercept + X %*% cY$betas[colnames(X)])
      if (!outcome_is_latent[match(y, outcomes_all)]) {
        if (y %in% colnames(data_in)) baseline_r2[y] <- r2_fun(data_in[[y]], yhat)
      } else if (!is.null(fs_full) && y %in% colnames(fs_full)) {
        baseline_r2[y] <- r2_fun(fs_full[, y], yhat)
      }
    }
  }
  
  # aiuto, funziona questo! lascia cosi
  set.seed(seed)
  N <- nrow(data_in); n_train <- max(3L, floor(train_prop * N))
  maxN <- if (is.numeric(times)) as.integer(times) else as.integer(max_times)
  res_mat <- matrix(NA_real_, nrow = maxN, ncol = length(outcomes_all))
  colnames(res_mat) <- outcomes_all
  used <- 0L
  done <- function(vmat) {
    for (j in seq_len(ncol(vmat))) {
      v <- vmat[, j]; v <- v[is.finite(v)]
      if (length(v) < window + 5L) return(FALSE)
      cm <- cumsum(v)/seq_along(v)
      if (abs(cm[length(cm)] - cm[length(cm) - window]) > tol) return(FALSE)
    }
    TRUE
  }
  target_splits <- if (is.numeric(times)) as.integer(times) else as.integer(max_times)
  
  while (used < target_splits) {
    tr_idx <- sort(sample.int(N, size = n_train, replace = FALSE))
    te_idx <- setdiff(seq_len(N), tr_idx); if (length(te_idx) < 3L) next
    dat_train <- data_in[tr_idx, , drop = FALSE]
    dat_test <- data_in[te_idx,  , drop = FALSE]
    
    fit_train <- tryCatch(
      lavaan::sem(
        model = model_obj, data = dat_train,
        estimator = estimator_use, missing = missing_opt,
        se = "none", 
        test = "none",
        std.lv = std_lv, parameterization = parameterization,
        ordered = if (length(ordered_vars)) ordered_vars else NULL,
        warn = !quiet
      ),
      error = function(e) NULL
    )
    if (is.null(fit_train) || !isTRUE(lavaan::inspect(fit_train, "converged"))) next
    
    used <- used + 1L
    
    coefs_tr <- get_struct_coefs(fit_train)
    fs_te <- if (any_latents) compute_lv_scores(fit_train, dat_test) else NULL
    
    for (j in seq_along(outcomes_all)) {
      y <- outcomes_all[j]
      cY <- coefs_tr[[y]]
      preds <- names(cY$betas)
      X <- build_X(preds, fs_te, dat_test)
      if (is.null(X)) { res_mat[used, j] <- NA_real_; next }
      yhat <- drop(cY$intercept + X %*% cY$betas[colnames(X)])
      if (!outcome_is_latent[j]) {
        if (y %in% colnames(dat_test)) res_mat[used, j] <- r2_fun(dat_test[[y]], yhat)
      } else {
        if (!is.null(fs_te) && y %in% colnames(fs_te)) res_mat[used, j] <- r2_fun(fs_te[, y], yhat)
      }
    }
    
    if (identical(times, "auto") && used >= (window + 5L)) {
      if (done(res_mat[seq_len(used), , drop = FALSE])) break
    }
  }
  
  if (used < nrow(res_mat)) res_mat <- res_mat[seq_len(used), , drop = FALSE]
  
# estrai sommari (esplicitare stats senno fa casino non so perche)
  r2_cv_mean <- apply(res_mat, 2L, function(v) { v <- v[is.finite(v)]; if (!length(v)) NA_real_ else mean(v) })
  r2_cv_median <- apply(res_mat, 2L, function(v) { v <- v[is.finite(v)]; if (!length(v)) NA_real_ else stats::median(v) })
  r2_cv_sd <- apply(res_mat, 2L, function(v) { v <- v[is.finite(v)]; if (!length(v)) NA_real_ else stats::sd(v) })
  n_ok <- apply(res_mat, 2L, function(v) sum(is.finite(v)))
  
  drop_mean_pct   <- rep(NA_real_, length(outcomes_all))
  drop_median_pct <- rep(NA_real_, length(outcomes_all))
  for (j in seq_along(outcomes_all)) {
    if (is.finite(baseline_r2[j]) && baseline_r2[j] >= min_r2_for_pct) {
      drop_mean_pct[j]   <- 100 * (baseline_r2[j] - r2_cv_mean[j])   / baseline_r2[j]
      drop_median_pct[j] <- 100 * (baseline_r2[j] - r2_cv_median[j]) / baseline_r2[j]
    } 
  }
  
  # ok maybe it works...spero v5
  out_type <- ifelse(outcome_is_latent, "latent", "observed")
  res_table <- data.frame(
    outcome = outcomes_all,
    type = out_type,
    r2_in = as.numeric(baseline_r2),
    r2_cv_mean = as.numeric(r2_cv_mean),
    r2_cv_median = as.numeric(r2_cv_median),
    r2_cv_sd = as.numeric(r2_cv_sd),
    drop_mean_pct = as.numeric(drop_mean_pct),
    drop_med_pct = as.numeric(drop_median_pct),
    splits_used = as.integer(n_ok),
    stringsAsFactors = FALSE
  )
  
  res <- list(
    table = res_table[order(res_table$type, res_table$outcome), ],
    split_matrix = res_mat,
    times = if (identical(times, "auto")) paste0("auto(", used, ")") else as.integer(times),
    train_prop = train_prop,
    N = nrow(data_in),
    seed = seed,
    digits = digits,
    call = match.call(),
    tol = tol,
    window = window,
    min_r2_for_pct = min_r2_for_pct
  )
  class(res) <- c("lav_cv", "list")
  
  # devi indicare graphics::!
  if (isTRUE(plot)) {
    op <- graphics::par(no.readonly = TRUE); on.exit(graphics::par(op), add = TRUE)
    for (y in res$table$outcome) {
      v <- res$split_matrix[, y]; v <- v[is.finite(v)]
      if (!length(v)) next
      m  <- seq_along(v)
      cm <- cumsum(v)/m
      s2 <- c(NA_real_, cumsum((v[-1] - cm[-length(cm)]) * (v[-1] - cm[-1])) / pmax(1, m[-1] - 1))
      se <- sqrt(s2 / m); se[1] <- NA_real_
      upper <- cm + 1.96 * se
      lower <- cm - 1.96 * se
      graphics::plot(m, cm, type = "l", xlab = "Splits used", ylab = "Running mean of R^2",
           main = paste0("CV convergence: ", y))
      graphics::lines(m, upper, lty = 2)
      graphics::lines(m, lower, lty = 2)
      graphics::abline(h = utils::tail(cm, 1L), lty = 3) # mancava la call da utils!!!
    }
  }
  
  res
}

### metodi S3 print e summary:

#' @param x A 'lav_cv' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_cv
#' @export
print.lav_cv <- function(x, digits = x$digits %||% 3L, ...) {
  stopifnot(is.list(x), !is.null(x$table))
  u2 <- if (isTRUE(l10n_info()[["UTF-8"]])) "\u00B2" else "^2"
  
  # header con i dettagli giusti!!
  auto_txt <- if (grepl("^auto\\(", as.character(x$times))) {
    paste0("auto-stop at ", gsub("[^0-9]", "", x$times), " splits")
  } else {
    paste0(x$times, " splits")
  }
  cat(sprintf(
    "\nRepeated holdout (Monte Carlo) cross-validation of R%s\n", u2
  ))
  cat(sprintf("Splits: %s (train proportion = %.2f; N = %d; seed = %s)\n\n",
              auto_txt, x$train_prop, x$N, as.character(x$seed)))
  
  tbl <- x$table
  
  star_fun <- function(d){
    if (is.na(d)) "" else if (d >= 50) "***" else if (d >= 30) "**" else if (d >= 10) "*" else ""
  }
  stars <- vapply(tbl$drop_med_pct, star_fun, character(1))
  
  dash <- "-"  # al posto di NA meglio un trattino dai
  fmt  <- function(z, d = digits) {
    ifelse(is.na(z), dash, formatC(z, digits = d, format = "f"))
  }
  fmt1 <- function(z) {
    ifelse(is.na(z), dash, formatC(z, digits = 1, format = "f"))
  }
  
  for (tp in c("observed", "latent")) {
    sub <- tbl[tbl$type == tp, , drop = FALSE]
    if (!nrow(sub)) next
    
    out_names <- sub$outcome
    r2_infit <- round(sub$r2_in, digits)
    r2_med <- round(sub$r2_cv_median, digits)
    sd_cv <- round(sub$r2_cv_sd, digits)
    drop_pct <- ifelse(is.finite(sub$drop_med_pct), round(sub$drop_med_pct, 1), NA)
    stars_tp <- stars[tbl$type == tp]
    
    cat(sprintf("%s outcomes\n", tools::toTitleCase(tp)))
    cat(rep("-", 86), "\n", sep = "")
    cat(sprintf("%-16s %14s %16s %10s %9s %3s\n",
                "Outcome", paste0("R", u2, " in-sample"),
                paste0("CV R", u2), "CV SD", "% drop", ""))
    cat(rep("-", 86), "\n", sep = "")
    for (i in seq_len(nrow(sub))) {
      cat(sprintf("%-16s %14s %16s %10s %9s %3s\n",
                  out_names[i],
                  fmt(r2_infit[i]),
                  fmt(r2_med[i]),
                  fmt(sd_cv[i]),
                  fmt1(drop_pct[i]),
                  stars_tp[i]))
    }
    cat("\n")
  }
  
  # notine a pie tabella
  cat(sprintf("Auto-stop rule: window = %d, tol = %.3f (applied when times = 'auto').\n", x$window, x$tol))
  cat("Stars indicate % drop thresholds: * >=10%, ** >=30%, *** >=50%.\n")
  cat(sprintf("%% drop compares in-sample R%s to median CV R%s; suppressed when in-sample R%s < %.2f.\n",
              u2, u2, u2, x$min_r2_for_pct %||% 0.05))
  cat(sprintf("CV SD is the standard deviation of split-wise R%s across successful splits.\n", u2))
  invisible(x)
}

#' @param object A 'lav_cv' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_cv
#' @export
summary.lav_cv <- function(object, ...) { 
  print.lav_cv(object, ...); 
  invisible(object) 
  }

#' Johnson-Neyman regions of significance for fitted 'lavaan' models
#'
#' Computes the Johnson-Neyman (JN) interval for a continuous moderator from a
#' fitted 'lavaan' model that includes an explicit product term. 
#' Plots the conditional slope with a confidence band, shaded significance
#' regions, and an observed-moderator density strip.
#'
#' The model should include a main effect for the predictor, a main effect for
#' the moderator, and their product term. Standard errors are obtained via the
#' delta method using the model-implied covariance matrix of the parameter
#' estimates. If the model was fitted with a robust estimator (e.g., MLR),
#' the robust covariance matrix returned by \code{vcov(fit)} is used
#' automatically, so no separate correction is needed.
#'
#' The JN boundary values are the real roots of the quadratic equation that
#' equates the squared conditional slope to the squared critical value times
#' its variance. When the discriminant is negative, the conditional slope is
#' either significant or non-significant across the entire moderator range and
#' no finite boundary exists.
#'
#' This function is restricted to continuous moderators. For categorical
#' moderators, use \code{\link{lav_slopes}}.
#'
#' @usage
#' lav_jn(
#'   fit,
#'   outcome,
#'   pred,
#'   modx,
#'   interaction,
#'   data = NULL,
#'   conf.level = 0.95,
#'   modx.range = NULL,
#'   n_grid = 1000L,
#'   x.label = NULL,
#'   y.label = NULL,
#'   sig.color = "#009E73",
#'   nonsig.color = "#D55E00",
#'   line.color = "#0072B2",
#'   alpha = 0.20,
#'   line.size = 0.80,
#'   rug = TRUE,
#'   plot = TRUE,
#'   digits = 3L,
#'   return_data = FALSE
#' )
#'
#' @param fit A fitted 'lavaan' object that includes the product term (required).
#' @param outcome Character. Name of the dependent variable in \code{fit} (required).
#' @param pred Character. Name of the focal predictor whose conditional slope
#'   is probed (required).
#' @param modx Character. Name of the continuous moderator (required). If
#'   \code{modx} is categorical (fewer than 4 unique values), the function
#'   stops with a message directing the user to \code{\link{lav_slopes}}.
#' @param interaction Character. Name of the product term in \code{fit}
#'   (e.g., \code{"X_Z"}) (required).
#' @param data \code{data.frame}. Optional raw data (not needed; retained for
#'   backward compatibility). The function automatically recovers observed data
#'   from \code{fit} when needed.
#' @param conf.level Numeric in (0, 1). Confidence level for the Wald band and
#'   the JN critical value (default: 0.95).
#' @param modx.range Numeric vector of length 2. Range \code{c(min, max)} for
#'   the moderator axis. If \code{NULL}, the observed range of \code{modx} in
#'   the data is used, extended by 5\% on each side for visual padding
#'   (default: NULL).
#' @param n_grid Integer. Number of moderator values at which the conditional
#'   slope is evaluated for plotting (default: 1000).
#' @param x.label Character. X-axis label (default: \code{modx}).
#' @param y.label Character. Y-axis label (default: \code{"Conditional slope of <pred>"}).
#' @param sig.color Character. Fill color for significant-slope regions
#'   (default: \code{"#009E73"}, Okabe-Ito bluish green).
#' @param nonsig.color Character. Fill color for non-significant regions
#'   (default: \code{"#D55E00"}, Okabe-Ito vermillion).
#' @param line.color Character. Color for the conditional-slope line
#'   (default: \code{"#0072B2"}, Okabe-Ito blue).
#' @param alpha Numeric in (0, 1). Opacity of the confidence ribbon and the
#'   region shading (default: 0.20).
#' @param line.size Numeric > 0. Width of the conditional-slope line
#'   (default: 0.80).
#' @param rug Logical. If \code{TRUE}, draw a rug (density strip) of observed
#'   moderator values along the x-axis (default: TRUE).
#' @param plot Logical. If \code{TRUE}, produce the JN plot (default: TRUE).
#' @param digits Integer \code{>= 0}. Decimal digits in printed output
#'   (default: 3).
#' @param return_data Logical. If \code{TRUE}, include the plotting
#'   \code{data.frame} in the returned list (default: FALSE).
#'
#' @return A list of class \code{"lav_jn"} with elements:
#' \describe{
#'   \item{\code{jn_points}}{Numeric vector of length 0, 1, or 2 giving the
#'     Johnson-Neyman boundary values of the moderator (the roots of the
#'     quadratic). \code{numeric(0)} when no real root exists.}
#'   \item{\code{signif_regions}}{A data frame with columns \code{lower},
#'     \code{upper}, and \code{significance} (\code{"significant"} or
#'     \code{"non-significant"}), describing the regions over the observed
#'     moderator range.}
#'   \item{\code{plot}}{A \code{ggplot} object (or \code{NULL} when
#'     \code{plot = FALSE}).}
#'   \item{\code{observed_support}}{Numeric length-2: the minimum and maximum of
#'     the observed moderator values.}
#'   \item{\code{interaction_test}}{List with the unstandardized interaction
#'     coefficient (\code{b}), its \code{se}, \code{z}, \code{p}, Wald
#'     confidence interval (\code{ci}), and standardized beta (\code{beta_std}).}
#'   \item{\code{labels}}{List of the user-supplied variable names.}
#'   \item{\code{conf.level}}{Confidence level used.}
#'   \item{\code{digits}}{Default digits for printing.}
#'   \item{\code{call}}{Matched call.}
#'   \item{\code{plot_data}}{Only when \code{return_data = TRUE}: data frame
#'     used to build the plot.}
#' }
#'
#' @section Notes:
#' All estimates are unstandardized; a standardized coefficient for the interaction is
#' also reported for reference. Wald tests assume large-sample normality of the
#' parameter estimates. When the discriminant of the JN quadratic is negative,
#' the conditional slope is either significant or non-significant everywhere;
#' the function reports which case applies rather than returning a boundary.
#'
#' @references
#' Johnson, P. O., & Neyman, J. (1936). Tests of certain linear hypotheses and
#' their application to some educational problems. \emph{Statistical Research
#' Memoirs}, \emph{1}, 57-93.
#'
#' Bauer, D. J., & Curran, P. J. (2005). Probing interactions in fixed and
#' multilevel regression: Inferential and graphical techniques.
#' \emph{Multivariate Behavioral Research}, \emph{40}(3), 373-400.
#' \doi{10.1207/s15327906mbr4003_5}
#'
#' Carden, S. W., Holtzman, N. S., & Strube, M. J. (2017). CAHOST: An Excel
#' workbook for facilitating the Johnson-Neyman technique for two-way
#' interactions in multiple regression. \emph{Frontiers in Psychology},
#' \emph{8}, Article 1293. \doi{10.3389/fpsyg.2017.01293}
#'
#' @seealso \code{\link{lav_slopes}} for pick-a-point simple slopes.
#'
#' @examples
#' set.seed(42)
#' X <- rnorm(100); Z <- rnorm(100); X_Z <- X*Z
#' Y <- 0.6*X + 0.6*Z + 0.3*X_Z + rnorm(100, sd = 0.7) 
#' dataset <- data.frame(Y, X, Z, X_Z)
#' fit <- lavaan::sem("Y ~ X + Z + X_Z", data = dataset)
#' lav_jn(
#' fit = fit,
#' outcome = "Y",
#' pred = "X",
#' modx = "Z",
#' interaction = "X_Z"
#' )
#'
#' @importFrom lavaan parameterEstimates parTable lavInspect lavNames
#' @importFrom stats vcov qnorm pnorm 
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon geom_hline geom_vline
#'   geom_rect geom_rug labs scale_fill_manual scale_x_continuous
#' @importFrom rlang sym %||%
#' @export
lav_jn <- function(
    fit,
    outcome,
    pred,
    modx,
    interaction,
    data = NULL,
    conf.level = 0.95,
    modx.range = NULL,
    n_grid = 1000L,
    x.label = NULL,
    y.label = NULL,
    sig.color = "#009E73",
    nonsig.color = "#D55E00",
    line.color = "#0072B2",
    alpha = 0.20,
    line.size = 0.80,
    rug = TRUE,
    plot = TRUE,
    digits = 3L,
    return_data = FALSE
) {

  if (!inherits(fit, "lavaan"))
    stop("'fit' must be a 'lavaan' object.", call. = FALSE)
  if (missing(interaction) || is.null(interaction))
    stop("'interaction' (product-term name) must be supplied.", call. = FALSE)

  fetch_data <- function(f) {
    out <- tryCatch(lavaan::lavInspect(f, "data"), error = function(e) NULL)
    if (is.null(out)) {
      out <- tryCatch(lavaan::lavInspect(f, "data.original"),
                       error = function(e) NULL)
    }
    if (is.null(out)) {
      out <- tryCatch(f@Data@X, error = function(e) NULL)
    }
    if (is.null(out)) return(NULL)

    ov_names <- tryCatch(lavaan::lavNames(f, type = "ov"),
                          error = function(e) NULL)
    if (is.data.frame(out)) {
      out <- as.data.frame(out)
      return(out)
    }
    if (is.matrix(out)) {
      out <- as.data.frame(out)
      if (!is.null(ov_names) && length(ov_names) == ncol(out)) {
        names(out) <- ov_names
      }
      return(out)
    }
    if (is.list(out)) {
      ok_list <- all(vapply(out,
                            function(x) is.data.frame(x) || is.matrix(x),
                            logical(1)))
      if (!ok_list) return(NULL)

      group_labels_fd <- tryCatch(lavaan::lavInspect(f, "group.label"),
                                   error = function(e) NULL)
      group_var_fd <- NULL
      group_call <- tryCatch(f@call$group, error = function(e) NULL)
      if (!is.null(group_call)) {
        group_var_fd <- paste(as.character(group_call), collapse = "")
        group_var_fd <- gsub('^"|"$', "", group_var_fd)
      }
      if (is.null(group_var_fd) || length(group_var_fd) != 1L ||
          is.na(group_var_fd) || !nzchar(group_var_fd)) {
        tmp_group <- tryCatch(f@Options$group, error = function(e) NULL)
        if (length(tmp_group) == 1L && !is.na(tmp_group) && nzchar(tmp_group)) {
          group_var_fd <- tmp_group
        } else {
          group_var_fd <- NULL
        }
      }
      out_list <- vector("list", length(out))
      for (g in seq_along(out)) {
        dg <- out[[g]]
        if (is.matrix(dg)) {
          dg <- as.data.frame(dg)
          if (!is.null(ov_names) && length(ov_names) == ncol(dg)) {
            names(dg) <- ov_names
          }
        } else {
          dg <- as.data.frame(dg)
        }
        if (!is.null(group_var_fd) && nzchar(group_var_fd) &&
            !(group_var_fd %in% names(dg))) {
          if (!is.null(group_labels_fd) && length(group_labels_fd) >= g) {
            lab_g <- group_labels_fd[g]
          } else {
            lab_g <- g
          }
          num_lab <- suppressWarnings(as.numeric(lab_g))
          if (!is.na(num_lab)) {
            dg[[group_var_fd]] <- num_lab
          } else {
            dg[[group_var_fd]] <- as.character(lab_g)
          }
        }
        out_list[[g]] <- dg
      }
      out <- do.call(rbind, out_list)
      rownames(out) <- NULL
      return(out)
    }
    NULL
  }

  dat <- fetch_data(fit)
  if (is.null(dat) && !is.null(data)) {
    dat <- as.data.frame(data)
  }

  if (!is.null(dat) && modx %in% names(dat)) {
    modx_obs <- dat[[modx]]
    if (!is.numeric(modx_obs)) {
      stop(
        "The moderator '", modx, "' is not numeric. ",
        "lav_jn() is designed for continuous moderators only. ",
        "For categorical moderators, use lav_slopes().",
        call. = FALSE
      )
    }
    n_unique <- length(unique(modx_obs[!is.na(modx_obs)]))
    if (n_unique < 4L) {
      stop(
        "The moderator '", modx, "' has only ", n_unique, " unique value(s). ",
        "The Johnson-Neyman technique requires a continuous moderator. ",
        "For categorical or quasi-categorical moderators, use lav_slopes().",
        call. = FALSE
      )
    }
  } else {

        lv_names <- tryCatch(lavaan::lavNames(fit, type = "lv"),
                          error = function(e) character(0))
    if (!(modx %in% lv_names)) {
      stop(
        "'modx' ('", modx, "') was not found as an observed or latent variable ",
        "in 'fit'. Supply a valid moderator name.",
        call. = FALSE
      )
    }
    modx_obs <- NULL
  }

  if (!is.null(modx_obs)) {
    obs_min <- min(modx_obs, na.rm = TRUE)
    obs_max <- max(modx_obs, na.rm = TRUE)
    observed_support <- c(obs_min, obs_max)
  } else {

    ngroups <- tryCatch(lavaan::lavInspect(fit, "ngroups"),
                         error = function(e) 1L)
    mean_lv <- tryCatch(lavaan::lavInspect(fit, "mean.lv"),
                          error = function(e) NULL)
    cov_lv  <- tryCatch(lavaan::lavInspect(fit, "cov.lv"),
                          error = function(e) NULL)
    if (is.list(mean_lv)) mean_lv <- mean_lv[[1L]]
    if (is.list(cov_lv))  cov_lv  <- cov_lv[[1L]]
    if (is.matrix(mean_lv) && nrow(mean_lv) >= 1L) {
      tmp <- as.numeric(mean_lv[1L, ])
      names(tmp) <- colnames(mean_lv)
      mean_lv <- tmp
    }
    mu_z <- if (!is.null(mean_lv) && modx %in% names(mean_lv)) {
      as.numeric(mean_lv[modx])
    } else { 0 }
    sd_z <- if (!is.null(cov_lv) && is.matrix(cov_lv) &&
                modx %in% rownames(cov_lv)) {
      sqrt(as.numeric(cov_lv[modx, modx]))
    } else { 1 }
    observed_support <- c(mu_z - 3 * sd_z, mu_z + 3 * sd_z)
    obs_min <- observed_support[1L]
    obs_max <- observed_support[2L]
  }

  if (is.null(modx.range)) {
    pad <- 0.05 * (obs_max - obs_min)
    if (pad < 1e-8) pad <- 0.5
    modx.range <- c(obs_min - pad, obs_max + pad)
  }

  pe <- lavaan::parameterEstimates(fit, standardized = FALSE)
  pt <- lavaan::parTable(fit)
  vc <- tryCatch(stats::vcov(fit),
                 error = function(e) lavaan::lavInspect(fit, "vcov"))
  if (!is.matrix(vc))
    stop("Could not retrieve covariance matrix from 'fit'.", call. = FALSE)

  find_par <- function(lhs, rhs, op = "~", rhs_regex = NULL,
                       required = TRUE, default_est = 0) {
    idx_pe <- which(pe$lhs == lhs & pe$rhs == rhs & pe$op == op)
    if (length(idx_pe) == 0L && !is.null(rhs_regex))
      idx_pe <- which(pe$lhs == lhs & pe$op == op &
                        grepl(rhs_regex, pe$rhs))
    if (length(idx_pe) == 0L) {
      if (!required) {
        est_val <- default_est
        idx_pe  <- NA_integer_
      } else {
        stop(sprintf("Parameter '%s %s %s' not found.", lhs, op, rhs),
             call. = FALSE)
      }
    } else {
      idx_pe <- idx_pe[1L]
      est_val <- pe$est[idx_pe]
    }
    idx_pt <- which(pt$lhs == lhs & pt$rhs == rhs & pt$op == op)
    if (length(idx_pt) == 0L && !is.null(rhs_regex))
      idx_pt <- which(pt$lhs == lhs & pt$op == op &
                        grepl(rhs_regex, pt$rhs))
    free_idx <- if (length(idx_pt) == 0L) 0L else {
      fi <- pt$free[idx_pt[1L]]
      if (is.na(fi)) 0L else as.integer(fi)
    }
    list(est = est_val, free = free_idx, idx_pe = idx_pe)
  }

  p1 <- find_par(outcome, pred, op = "~")
  p2 <- find_par(outcome, modx, op = "~")  
  p3 <- find_par(outcome, interaction, op = "~",
                 rhs_regex = paste0("(", pred, ").*(", modx, ")|(",
                                    modx, ").*(", pred, ")"))

  safe_free <- function(x) {
    if (is.na(x) || length(x) == 0L) 0L else as.integer(x)
  }
  p1$free <- safe_free(p1$free)
  p3$free <- safe_free(p3$free)

  vc_ <- function(a, b) {
    a <- safe_free(a); b <- safe_free(b)
    if (a <= 0L || b <= 0L) return(0)
    if (a > nrow(vc) || b > ncol(vc)) return(0)
    val <- vc[a, b]
    if (is.na(val)) 0 else val
  }

  b1 <- p1$est
  b3 <- p3$est

  var_b1 <- vc_(p1$free, p1$free)
  var_b3 <- vc_(p3$free, p3$free)
  cov_b1b3 <- vc_(p1$free, p3$free)

  # quadratica di johnson neyman!!!
  z_crit <- stats::qnorm(1 - (1 - conf.level) / 2)

  A <- b3^2 - z_crit^2 * var_b3
  B <- 2 * b1 * b3 - 2 * z_crit^2 * cov_b1b3
  C <- b1^2 - z_crit^2 * var_b1

  discriminant <- B^2 - 4 * A * C
  jn_points <- numeric(0)

  if (abs(A) < 1e-14) {
    # degenera
    if (abs(B) > 1e-14) {
      jn_points <- -C / B
    }
  } else if (discriminant >= 0) {
    sqrt_disc <- sqrt(discriminant)
    root1 <- (-B - sqrt_disc) / (2 * A)
    root2 <- (-B + sqrt_disc) / (2 * A)
    jn_points <- sort(c(root1, root2))
  }

  slope_sig_at <- function(z_val) {
    slope <- b1 + b3 * z_val
    var_slope <- var_b1 + 2 * z_val * cov_b1b3 + z_val^2 * var_b3
    se_slope  <- sqrt(max(var_slope, 0))
    if (se_slope < 1e-15) return(TRUE)
    abs(slope / se_slope) > z_crit
  }

  jn_in_range <- jn_points[jn_points >= obs_min & jn_points <= obs_max]
  breaks <- sort(unique(c(obs_min, jn_in_range, obs_max)))

  signif_regions <- data.frame(
    lower = numeric(0),
    upper = numeric(0),
    significance = character(0),
    stringsAsFactors = FALSE
  )
  for (k in seq_len(length(breaks) - 1L)) {
    midpoint <- (breaks[k] + breaks[k + 1L]) / 2
    is_sig <- slope_sig_at(midpoint)
    signif_regions <- rbind(signif_regions, data.frame(
      lower = breaks[k],
      upper = breaks[k + 1L],
      significance = if (is_sig) "significant" else "non-significant",
      stringsAsFactors = FALSE
    ))
  }

  interaction_test <- list(
    b = p3$est,
    se = if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_,
    z = {
      se_int <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      if (!is.na(se_int) && se_int > 0) p3$est / se_int else NA_real_
    },
    p = {
      se_int <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      if (!is.na(se_int) && se_int > 0) {
        2 * stats::pnorm(abs(p3$est / se_int), lower.tail = FALSE)
      } else { NA_real_ }
    },
    ci = {
      se_int <- if (!is.na(p3$idx_pe)) pe$se[p3$idx_pe] else NA_real_
      c(p3$est - z_crit * se_int, p3$est + z_crit * se_int)
    },
    beta_std = {
      pe_std <- lavaan::parameterEstimates(fit, standardized = TRUE)
      idx_std <- which(pe_std$lhs == outcome &
                         pe_std$rhs == interaction &
                         pe_std$op == "~")
      if (length(idx_std)) pe_std$std.all[idx_std[1L]] else NA_real_
    }
  )

  # griglia di plot
  z_seq <- seq(modx.range[1L], modx.range[2L], length.out = n_grid)

  plot_df <- data.frame(
    modx_value = z_seq,
    slope = NA_real_,
    se = NA_real_,
    ci_lower = NA_real_,
    ci_upper = NA_real_,
    stringsAsFactors = FALSE
  )
  for (i in seq_len(n_grid)) {
    z0 <- z_seq[i]
    slope <- b1 + b3 * z0
    var_slope <- var_b1 + 2 * z0 * cov_b1b3 + z0^2 * var_b3
    se_slope <- sqrt(max(var_slope, 0))
    plot_df$slope[i] <- slope
    plot_df$se[i] <- se_slope
    plot_df$ci_lower[i] <- slope - z_crit * se_slope
    plot_df$ci_upper[i] <- slope + z_crit * se_slope
  }

  p_out <- NULL
  if (isTRUE(plot)) {

    y_lo <- min(plot_df$ci_lower, na.rm = TRUE)
    y_hi <- max(plot_df$ci_upper, na.rm = TRUE)
    y_pad <- 0.05 * (y_hi - y_lo)
    if (y_pad < 1e-8) y_pad <- 0.1
    rect_ymin <- y_lo - y_pad
    rect_ymax <- y_hi + y_pad

    all_breaks <- sort(unique(c(modx.range[1L], jn_in_range, modx.range[2L])))
    region_df <- data.frame(
      xmin = numeric(0), xmax = numeric(0),
      fill = character(0), stringsAsFactors = FALSE
    )
    for (k in seq_len(length(all_breaks) - 1L)) {
      mid  <- (all_breaks[k] + all_breaks[k + 1L]) / 2
      is_s <- slope_sig_at(mid)
      region_df <- rbind(region_df, data.frame(
        xmin = all_breaks[k],
        xmax = all_breaks[k + 1L],
        fill = if (is_s) "Significant" else "Non-significant",
        stringsAsFactors = FALSE
      ))
    }
    region_df$fill <- factor(region_df$fill,
                             levels = c("Significant", "Non-significant"))

    region_colors <- c("Significant"     = sig.color,
                       "Non-significant"  = nonsig.color)

    p_out <- ggplot2::ggplot(
      plot_df,
      ggplot2::aes(x = !!rlang::sym("modx_value"))
    ) +
      ggplot2::geom_rect(
        data = region_df,
        ggplot2::aes(
          xmin = !!rlang::sym("xmin"),
          xmax = !!rlang::sym("xmax"),
          fill = !!rlang::sym("fill"),
          ymin = rect_ymin,
          ymax = rect_ymax
        ),
        alpha = alpha,
        inherit.aes = FALSE
      ) +
      ggplot2::scale_fill_manual(
        values = region_colors,
        name = "Region"
      ) +
      ggplot2::geom_ribbon(
        ggplot2::aes(
          ymin = !!rlang::sym("ci_lower"),
          ymax = !!rlang::sym("ci_upper")
        ),
        fill = line.color, alpha = alpha
      ) +
      ggplot2::geom_line(
        ggplot2::aes(y = !!rlang::sym("slope")),
        color = line.color,
        linewidth = line.size
      ) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed",
                          color = "grey40", linewidth = 0.4)

    if (length(jn_in_range) > 0L) {
      p_out <- p_out +
        ggplot2::geom_vline(
          xintercept = jn_in_range,
          linetype = "dotted",
          color = "grey30",
          linewidth = 0.5
        )
    }

    if (isTRUE(rug) && !is.null(modx_obs)) {
      rug_df <- data.frame(modx_rug = modx_obs[!is.na(modx_obs)])
      p_out <- p_out +
        ggplot2::geom_rug(
          data = rug_df,
          ggplot2::aes(x = !!rlang::sym("modx_rug")),
          sides = "b",
          alpha = 0.3,
          color = "grey30",
          inherit.aes = FALSE
        )
    }

    x_lab <- x.label %||% modx
    y_lab <- y.label %||% paste0("Conditional slope of ", pred)

    p_out <- p_out +
      ggplot2::labs(x = x_lab, y = y_lab) +
      ggplot2::scale_x_continuous(expand = c(0, 0))
  }

  res <- list(
    jn_points = jn_points,
    signif_regions = signif_regions,
    plot = p_out,
    observed_support = observed_support,
    interaction_test = interaction_test,
    labels = list(outcome = outcome,
                  pred = pred,
                  modx = modx,
                  interaction = interaction),
    conf.level = conf.level,
    digits = digits,
    call = match.call()
  )
  if (isTRUE(return_data)) res$plot_data <- plot_df

  class(res) <- "lav_jn"
  return(res)
}


### metodi S3 print e summary:

#' @param x A 'lav_jn' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_jn
#' @export
print.lav_jn <- function(x, ...) {
  lab <- x$labels
  it <- x$interaction_test
  digits <- x$digits
  cl <- x$conf.level

  fmt <- function(v, k = digits) {
    if (is.na(v)) "NA" else formatC(v, format = "f", digits = k)
  }
  fmtp <- function(p, k = digits) {
    if (is.na(p)) return("NA")
    cutoff <- 10^(-k)
    cutoff_txt <- sub("^0\\.", ".", formatC(cutoff, digits = k, format = "f"))
    if (p < cutoff) return(paste0("< ", cutoff_txt))
    out <- formatC(p, digits = k, format = "f")
    sub("^0\\.", ".", out)
  }
  zfmt <- function(z) {
    if (is.na(z)) "NA" else formatC(z, format = "f", digits = digits)
  }

  cat("\nInteraction effect (", lab$pred, " * ", lab$modx,
      " -> ", lab$outcome, "): ",
      "b = ", fmt(it$b), ", SE = ", fmt(it$se),
      ", beta = ", fmt(it$beta_std), ", z = ", zfmt(it$z),
      ", p = ", fmtp(it$p),
      ", ", sprintf("%.0f%% CI", cl * 100),
      " [", fmt(it$ci[1L]), ", ", fmt(it$ci[2L]), "]\n",
      sep = "")

  cat("\nJohnson-Neyman boundaries (alpha = ",
      formatC(1 - cl, format = "f", digits = 2), ")\n", sep = "")
  cat(rep("-", 72), "\n", sep = "")

  obs_sup <- x$observed_support
  jn <- x$jn_points
  jn_in <- jn[jn >= obs_sup[1L] & jn <= obs_sup[2L]]

  if (length(jn) == 0L) {
    test_sig <- x$signif_regions$significance[1L] == "significant"
    if (test_sig) {
      cat("No JN boundary found. The conditional slope of ", lab$pred,
          " is significant across the entire observed range of ", lab$modx,
          " [", fmt(obs_sup[1L]), ", ", fmt(obs_sup[2L]), "].\n", sep = "")
    } else {
      cat("No JN boundary found. The conditional slope of ", lab$pred,
          " is non-significant across the entire observed range of ", lab$modx,
          " [", fmt(obs_sup[1L]), ", ", fmt(obs_sup[2L]), "].\n", sep = "")
    }
  } else {
    if (length(jn_in) > 0L) {
      for (j in seq_along(jn_in)) {
        cat("  JN boundary ", j, ": ", lab$modx, " = ", fmt(jn_in[j]),
            "\n", sep = "")
      }
    } else {
      cat("Both roots fall outside the observed range.\n")
    }
    cat("Observed range of ", lab$modx, ": [",
        fmt(obs_sup[1L]), ", ", fmt(obs_sup[2L]), "]\n", sep = "")
  }

  sr <- x$signif_regions
  if (nrow(sr) > 0L) {
    cat("\nSignificance regions (within observed range):\n")
    for (k in seq_len(nrow(sr))) {
      cat("  ", lab$modx, " in [", fmt(sr$lower[k]),
          ", ", fmt(sr$upper[k]), "]: slope is ",
          sr$significance[k], "\n", sep = "")
    }
  }
  cat(rep("-", 72), "\n", sep = "")

  cat("\n")
  if (!is.null(x$plot)) print(x$plot)
  invisible(x)
}

#' @param object A 'lav_jn' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_jn
#' @export
summary.lav_jn <- function(object, ...) {
  print.lav_jn(object, ...)
  invisible(object)
}

#' Variance Inflation Factors for 'lavaan' Structural Predictors
#'
#' Compute VIF for each predictor that appears in structural regressions
#' with two or more predictors, refitting the necessary sub-models so that
#' latent predictors are handled at the latent level (i.e., with their original
#' measurement models). It returns also the \eqn{R^2} of each eligible endogenous variable 
#' from the original fit for context. 
#' 
#' Each auxiliary refitted model includes the measurement structure for latent 
#' predictors, rebuilt from the original model syntax, together with observed 
#' residual covariances among the involved indicators when specified in the 
#' original model. Then, regresses the focal predictor on the remaining predictors 
#' at the latent level when applicable.
#' 
#' \eqn{\mathrm{VIF}_i = 1 / (1 - R_i^2)} generalizes VIF to SEM while respecting 
#' measurement models.
#'   
#' The function reuses the estimator, missing-data handling, and several options 
#' from \code{fit}. 
#' In multigroup models, VIF is computed separately within each group. 
#' Higher-order latent predictors are supported by recursively rebuilding
#' lower-order measurement blocks in the auxiliary refitted models.
#' 
#' @usage
#' lav_vif(
#'   fit,
#'   data = NULL,
#'   quiet = TRUE
#' )
#'
#' @param fit A fitted \code{lavaan} object.
#' @param data Optional. The data frame used to fit \code{fit}. If \code{NULL},
#'   the function attempts to extract the data from \code{fit} via
#'   \code{lavInspect(fit, "data")}. For multigroup models, grouped data are
#'   reconstructed into a single data frame when possible.
#' @param quiet Logical. If \code{TRUE} suppresses lavaan refit messages.
#'
#' @return A list with:
#' \itemize{
#'   \item \code{vif_table}: data.frame with columns
#'         \code{outcome}, \code{predictor}, \code{group}, \code{r2_predictor},
#'         \code{vif}, \code{k_predictors}.
#'   \item \code{outcome_r2}: data.frame with \eqn{R^2} per eligible endogenous
#'         \code{outcome} and \code{group} from the original \code{fit}.
#' }
#'
#' @references
#' Fox, J., & Monette, G. (1992). Generalized collinearity diagnostics.
#' \emph{Journal of the American Statistical Association}, \emph{87}(417),
#' 178-183. \doi{10.1080/01621459.1992.10475190}
#'
#' Belsley, D. A., Kuh, E., & Welsch, R. E. (1980).
#' \emph{Regression diagnostics: Identifying influential data and sources of
#' collinearity}. Wiley.
#'
#' @examples
#' set.seed(42)
#' x1 <- rnorm(100); x2 <- 0.85*x1 + rnorm(100, sd = sqrt(1 - 0.85^2)); x3 <- rnorm(100)
#' y <- 0.5*x1 + 0.3*x2 + 0.1*x3 + rnorm(100, sd = 0.7)
#' dataset <- data.frame(y, x1, x2, x3)
#' fit <- lavaan::sem("y ~ x1 + x2 + x3", data = dataset)
#' lav_vif(
#' fit = fit,
#' data = dataset)
#' 
#' @importFrom rlang %||%
#' @export
lav_vif <- function(fit, data = NULL, quiet = TRUE) {
  if (!inherits(fit, "lavaan")) stop("`fit` must be a fitted lavaan object.")
  
  # trova i dati e lascia errori se non va
  group_labels <- tryCatch(lavaan::lavInspect(fit, "group.label"), error = function(e) NULL)
  group_var <- NULL
  group_call <- tryCatch(fit@call$group, error = function(e) NULL)
  if (!is.null(group_call)) {
    group_var <- paste(as.character(group_call), collapse = "")
    group_var <- gsub('^"|"$', "", group_var)
  }
  if (is.null(group_var) || length(group_var) != 1L || is.na(group_var) || !nzchar(group_var)) {
    tmp_group <- tryCatch(fit@Options$group, error = function(e) NULL)
    if (length(tmp_group) == 1L && !is.na(tmp_group) && nzchar(tmp_group)) {
      group_var <- tmp_group
    } else {
      group_var <- NULL
    }
  }
  data_in <- data %||%
    tryCatch(lavaan::lavInspect(fit, "data"), error = function(e) NULL)
  if (is.null(data_in)) {
    stop("Could not locate data from `fit`; pass it via `data =`.")
  }
  if (is.data.frame(data_in)) {
    data_in <- as.data.frame(data_in)
  } else if (is.matrix(data_in)) {
    data_in <- as.data.frame(data_in)
    ov_names <- tryCatch(lavaan::lavNames(fit, type = "ov"), error = function(e) NULL)
    if (!is.null(ov_names) && length(ov_names) == ncol(data_in)) {
      names(data_in) <- ov_names
    }
  } else if (is.list(data_in)) {
    ok_list <- all(vapply(data_in, function(x) is.data.frame(x) || is.matrix(x), logical(1)))
    if (!ok_list) {
      stop("Recovered grouped data from `fit`, but could not coerce it to a data frame; please pass it via `data =`.")
    }
    ov_names <- tryCatch(lavaan::lavNames(fit, type = "ov"), error = function(e) NULL)
    data_list <- vector("list", length(data_in))
    for (g in seq_along(data_in)) {
      dg <- data_in[[g]]
      if (is.matrix(dg)) {
        dg <- as.data.frame(dg)
        if (!is.null(ov_names) && length(ov_names) == ncol(dg)) {
          names(dg) <- ov_names
        }
      } else {
        dg <- as.data.frame(dg)
      }
      if (!is.null(group_var) && !(group_var %in% names(dg))) {
        if (!is.null(group_labels) && length(group_labels) >= g) {
          lab_g <- group_labels[g]
        } else {
          lab_g <- g
        }
        num_lab <- suppressWarnings(as.numeric(lab_g))
        if (!is.na(num_lab)) {
          dg[[group_var]] <- num_lab
        } else {
          dg[[group_var]] <- as.character(lab_g)
        }
      }
      data_list[[g]] <- dg
    }
    data_in <- do.call(rbind, data_list)
    rownames(data_in) <- NULL
  } else {
    stop("Recovered data is not a data.frame, matrix, or supported grouped list; please pass it via `data =`.")
  }
  
  # estrai tutto
  OPT <- fit@Options
  estimator <- OPT$estimator
  missing_opt <- OPT$missing
  std_lv <- isTRUE(OPT$std.lv)
  parameterization <- OPT$parameterization
  aux_se <- "none"
  aux_test <- "none"

  ngroups <- tryCatch(
    as.integer(lavaan::lavInspect(fit, "ngroups")),
    error = function(e) 1L
  )
  if (length(ngroups) != 1L || is.na(ngroups)) ngroups <- 1L

  group_labels <- tryCatch(
    lavaan::lavInspect(fit, "group.label"),
    error = function(e) NULL
  )
  
  group_var <- NULL
  
  group_call <- tryCatch(fit@call$group, error = function(e) NULL)
  if (!is.null(group_call)) {
    group_var <- paste(as.character(group_call), collapse = "")
    group_var <- gsub('^"|"$', "", group_var)
  }
  
  if (is.null(group_var) || !nzchar(group_var)) {
    tmp_group <- tryCatch(OPT$group, error = function(e) NULL)
    if (length(tmp_group) == 1L && !is.na(tmp_group) && nzchar(tmp_group)) {
      group_var <- tmp_group
    } else {
      group_var <- NULL
    }
  }
  ordered_vars <- tryCatch(lavaan::lavNames(fit, type = "ov.ord"), error = function(e) character(0))
  
  # mappa strutturale
  pt <- lavaan::parTable(fit)
  is_struct <- pt$op == "~" & pt$rhs != "1"
  is_interaction <- grepl(":", pt$rhs, fixed = TRUE) | pt$op == "XWITH"
  struct <- pt[is_struct & !is_interaction, c("lhs", "rhs", "group", "free")]
  struct$rhs_free <- struct$free != 0
  struct_k <- stats::aggregate(rhs_free ~ lhs + group, data = struct, sum)
  eligible <- struct_k[struct_k$rhs_free >= 2, c("lhs", "group")]
  if (!nrow(eligible)) stop("No endogenous variables with two or more predictors.")
  key <- unique(struct[struct$rhs_free, c("lhs", "rhs", "group")])
  
  # mapping di misura nuovo con anche second'ordine
  latents_all <- tryCatch(lavaan::lavNames(fit, type = "lv"), error = function(e) character(0))
  loadings <- pt[pt$op == "=~", c("lhs", "rhs")]
  
  latent_lhs <- unique(loadings$lhs)
  indicators_by_latent <- lapply(latent_lhs, function(L) {
    unique(loadings$rhs[loadings$lhs == L])})
  names(indicators_by_latent) <- latent_lhs
  
  ov_all <- tryCatch(lavaan::lavNames(fit, type = "ov"), error = function(e) unique(c(pt$lhs, pt$rhs)))
  resid_cov <- pt[pt$op == "~~" & pt$lhs != pt$rhs, c("lhs", "rhs")]
  resid_cov <- resid_cov[resid_cov$lhs %in% ov_all & resid_cov$rhs %in% ov_all, , drop = FALSE]
  espandi_blocco_latenti <- function(lat_set) {
    lat_set <- intersect(unique(lat_set), names(indicators_by_latent))
    if (!length(lat_set)) return(character(0))
    
    out <- character(0)
    seen <- character(0)
    
    walk <- function(L) {
      if (L %in% seen) return(invisible(NULL))
      seen <<- c(seen, L)
      
      rhs <- indicators_by_latent[[L]]
      lower_latents <- intersect(rhs, names(indicators_by_latent))
      
      if (length(lower_latents)) {
        for (j in seq_along(lower_latents)) {
          walk(lower_latents[j])
        }
      }
      
      out <<- c(out, L)
      invisible(NULL)
    }
    
    for (j in seq_along(lat_set)) {
      walk(lat_set[j])
    }
    
    out
  }
  
  build_measurement_syntax <- function(lat_set) {
    lat_set <- espandi_blocco_latenti(lat_set)
    if (!length(lat_set)) return(character(0))
    
    meas_lines <- vapply(lat_set, function(L) {
      inds <- indicators_by_latent[[L]]
      paste0(L, " =~ ", paste(inds, collapse = " + "))
    }, character(1))
    
    used_inds <- unique(unlist(lapply(lat_set, function(L) {
      rhs <- indicators_by_latent[[L]]
      rhs[rhs %in% ov_all]
    }), use.names = FALSE))
    
    rc_lines <- character(0)
    if (length(used_inds) > 1L && nrow(resid_cov)) {
      rc_keep <- resid_cov[
        resid_cov$lhs %in% used_inds & resid_cov$rhs %in% used_inds,
        ,
        drop = FALSE
      ]
      if (nrow(rc_keep)) {
        rc_lines <- unique(paste0(rc_keep$lhs, " ~~ ", rc_keep$rhs))
      }
    }
    
    unique(c(meas_lines, rc_lines))
  } # stop blocco nuovo fixato
  
  # R2 ottenuto dal fit originale!!!
  r2_orig <- lavaan::inspect(fit, "r2")
  outcome_r2 <- list()
  
  if (is.list(r2_orig)) {
    for (i in seq_len(nrow(eligible))) {
      y <- eligible$lhs[i]
      g <- eligible$group[i]
      r2_g <- r2_orig[[g]]
      
      if (length(r2_g) && y %in% names(r2_g)) {
        outcome_r2[[length(outcome_r2) + 1L]] <- data.frame(
          outcome = y,
          group = g,
          r2_outcome = unname(r2_g[y])
        )
      }
    }
  } else {
    for (i in seq_len(nrow(eligible))) {
      y <- eligible$lhs[i]
      
      if (length(r2_orig) && y %in% names(r2_orig)) {
        outcome_r2[[length(outcome_r2) + 1L]] <- data.frame(
          outcome = y,
          group = 1L,
          r2_outcome = unname(r2_orig[y])
        )
      }
    }
  }
  
  outcome_r2 <- if (length(outcome_r2)) {
    do.call(rbind, outcome_r2)
  } else {
    data.frame(
      outcome = character(0),
      group = integer(0),
      r2_outcome = numeric(0)
    )
  }
  
  # VIF ottenuti da regressioni accessorie
  out_rows <- list()
  for (i in seq_len(nrow(eligible))) {
    y <- eligible$lhs[i]; g <- eligible$group[i]
    preds <- unique(key$rhs[key$lhs == y & key$group == g])
    if (length(preds) < 2L) next
    lat_preds <- intersect(preds, latents_all)
    meas_syntax <- build_measurement_syntax(lat_preds)
    
    for (px in preds) {
      if (grepl(":", px, fixed = TRUE)) next
      others <- setdiff(preds, px); if (!length(others)) next
      reg_line <- paste0(px, " ~ ", paste(others, collapse = " + "))
      aux_model <- paste(c(meas_syntax, reg_line), collapse = "\n")
      
      aux_fit <- lavaan::sem(
        model = aux_model,
        data = data_in,
        group = group_var,
        estimator = estimator,
        missing = missing_opt,
        se = aux_se,
        test = aux_test,
        std.lv = std_lv,
        parameterization = parameterization,
        ordered = if (length(ordered_vars)) ordered_vars else NULL,
        warn = !quiet
      )
      
      r2_aux <- lavaan::inspect(aux_fit, "r2")
      
      if (is.list(r2_aux)) {
        if (!is.null(r2_aux[[g]]) && px %in% names(r2_aux[[g]])) {
          r2v <- unname(r2_aux[[g]][px])
        } else {
          r2v <- NA_real_
        }
        
        r2v <- if (is.na(r2v)) NA_real_ else min(max(r2v, 0), 0.999999)
        vifv <- if (is.na(r2v)) NA_real_ else 1 / (1 - r2v)
        
        out_rows[[length(out_rows) + 1L]] <- data.frame(
          outcome = y,
          predictor = px,
          group = g,
          r2_predictor = r2v,
          vif = vifv,
          k_predictors = length(preds)
        )
      } else {
        if (px %in% names(r2_aux)) {
          r2v <- unname(r2_aux[px])
        } else {
          r2v <- NA_real_
        }
        
        r2v <- if (is.na(r2v)) NA_real_ else min(max(r2v, 0), 0.999999)
        vifv <- if (is.na(r2v)) NA_real_ else 1 / (1 - r2v)
        
        out_rows[[length(out_rows) + 1L]] <- data.frame(
          outcome = y,
          predictor = px,
          group = 1L,
          r2_predictor = r2v,
          vif = vifv,
          k_predictors = length(preds)
        )
      }
    }
  }
  
  vif_table <- if (length(out_rows)) do.call(rbind, out_rows) else
    data.frame(outcome = character(0), predictor = character(0), group = integer(0),
               r2_predictor = numeric(0), vif = numeric(0), k_predictors = integer(0))
  if (nrow(vif_table)) {
    vif_table <- vif_table[order(vif_table$outcome, vif_table$group, -vif_table$vif), ]
    rownames(vif_table) <- NULL
  }
  if (nrow(outcome_r2)) {
    keep <- unique(vif_table$outcome)
    outcome_r2 <- outcome_r2[outcome_r2$outcome %in% keep, , drop = FALSE]
    rownames(outcome_r2) <- NULL
  }
  
  res <- list(
    vif_table    = vif_table,
    outcome_r2   = outcome_r2,
    group_var    = group_var,
    group_labels = group_labels,
    call         = match.call()
  )
  class(res) <- c("lav_vif", "list")
  res
}

### metodi S3 print e summary:

#' @param x A 'lav_vif' object.
#' @param digits Integer number of digits to print.
#' @param cutoff Numeric length-2 thresholds used to flag VIF values.
#' @param ... Additional arguments; unused.
#' @rdname lav_vif
#' @export
print.lav_vif <- function(x, digits = 3, cutoff = c(5, 10), ...) {
  stopifnot(is.list(x), !is.null(x$vif_table))
  vt  <- x$vif_table
  or2 <- x$outcome_r2
  gl  <- x$group_labels
  u2  <- if (isTRUE(l10n_info()[["UTF-8"]])) "\u00B2" else "^2"
  
  if (!nrow(vt)) { cat("No eligible structural regressions (k < 2).\n"); return(invisible(x)) }
  
  fmt  <- function(z) ifelse(is.na(z), "NA", formatC(z, digits = digits, format = "f"))
  keys <- unique(vt[, c("outcome", "group")])
  
  for (ii in seq_len(nrow(keys))) {
    y <- keys$outcome[ii]; g <- keys$group[ii]
    gname <- if (length(gl) && !is.na(g) && g <= length(gl)) gl[g] else paste0("Group ", g)
    
    cat(sprintf("Dependent variable: %s  |  Group: %s\n", y, gname))
    
    r2y <- or2$r2_outcome[or2$outcome == y & or2$group == g]
    if (length(r2y)) cat(sprintf("  Model R%s = %s\n\n", u2, fmt(r2y)))
    
    sub <- vt[vt$outcome == y & vt$group == g, c("predictor", "r2_predictor", "vif")]
    sub <- sub[order(-sub$vif), , drop = FALSE]
    for (i in seq_len(nrow(sub))) {
      v <- sub$vif[i]
      flag <- if (is.na(v)) "" else if (v >= cutoff[2]) " [VIF>=10]" else if (v >= cutoff[1]) " [VIF>=5]" else ""
      tol <- if (is.na(v)) NA_real_ else 1 / v
      cat(sprintf("  %s: Predictor R%s = %s, VIF = %s, Tolerance = %s%s\n",
                  sub$predictor[i], u2, fmt(sub$r2_predictor[i]), fmt(v), fmt(tol), flag))
    }
    cat("\n")
  }
  ge <- if (isTRUE(l10n_info()[["UTF-8"]])) "\u2265" else ">="
  cat(sprintf(
    "Note. VIF = 1/(1 - Predictor R%s); Tolerance = 1 - Predictor R%s = 1/VIF. Flags mark VIF %s %s and %s %s.\n",
    u2, u2, ge, cutoff[1], ge, cutoff[2]
  ))
  invisible(x)
}

#' @param object A 'lav_vif' object.
#' @param ... Passed to `print.lav_vif()` (e.g., `digits`, `cutoff`).
#' @rdname lav_vif
#' @export
summary.lav_vif <- function(object, ...) {
  print.lav_vif(object, ...)
  invisible(object)
}

#' False Discovery Rate (FDR) Correction for 'lavaan' parameter p-values
#'
#' Apply a false discovery rate correction (Benjamini-Yekutieli by default) to
#' the p-values of selected parameters from a fitted \code{lavaan} object.
#' 
#' Useful when a SEM includes many structural paths (or many other parameters 
#' of substantive interest) and there is the need to control the expected 
#' proportion of false positives among the parameters declared 'statistically 
#' significant'.
#'  
#' With many simultaneous tests, using \code{p < .05} for each parameter 
#' inflates the expected number of false positives (about \code{m * .05} under 
#' all true null hypotheses, where \code{m} is the number of tested parameters).
#' Benjamini-Yekutieli (BY) controls the False Discovery Rate (FDR) under 
#' arbitrary dependence structures, which is suitable for SEMs where structural 
#' paths are inherently dependent through shared latent variables, covariance 
#' matrices, and model constraints.
#'
#' @usage
#' lav_fdr(
#'   fit,
#'   ops = c("reg", "load", "var.cov"),
#'   family = c("by_group", "selected"),
#'   method = "BY",
#'   alpha = 0.05,
#'   standardized = c("std.all", "std.lv", "std.nox", "none")
#' )
#'
#' @param fit A fitted \code{lavaan} object.
#' @param ops Character. One of \code{"reg"} (regressions), \code{"load"} (factor loadings),
#'   or \code{"var.cov"} (variances/covariances/residual variances). Default is \code{"reg"}.
#' @param family Character. If \code{"selected"}, FDR is applied across all selected
#'   parameters jointly. If \code{"by_group"}, FDR is applied separately within each group.
#'   Default is \code{"by_group"}.
#' @param method Character method passed to \code{stats::p.adjust} (default \code{"BY"}).
#' @param alpha Numeric significance threshold for adjusted p-values (default 0.05).
#' @param standardized Which standardized column to include, or \code{"none"}.
#'
#' @return A list with:
#' \itemize{
#'   \item \code{fdr_table}: data.frame with raw and FDR-adjusted p-values.
#'   \item \code{settings}: list of settings used.
#'   \item \code{group_var}: group variable name (or \code{NULL}).
#'   \item \code{group_labels}: group labels if available.
#'   \item \code{call}: matched call.
#' }
#' The returned object has class \code{"lav_fdr"}.
#'
#' @references
#' Benjamini, Y., & Hochberg, Y. (1995). Controlling the false discovery rate:
#' A practical and powerful approach to multiple testing. \emph{Journal of the
#' Royal Statistical Society: Series B (Methodological)}, \emph{57}(1),
#' 289-300. \doi{10.1111/j.2517-6161.1995.tb02031.x}
#'
#' Benjamini, Y., & Yekutieli, D. (2001). The control of the false discovery
#' rate in multiple testing under dependency. \emph{The Annals of Statistics},
#' \emph{29}(4), 1165-1188. \doi{10.1214/aos/1013699998}
#'
#' @examples 
#' model <- "
#' ind60 =~ x1 + x2 + x3
#' dem60 =~ y1 + y2 + y3 + y4
#' dem65 =~ y5 + y6 + y7 + y8
#' dem60 ~ ind60
#' dem65 ~ ind60 + dem60
#' y1 ~~ y5
#' y2 ~~ y6
#' "
#' fit <- lavaan::sem(
#' model = model, 
#' data = lavaan::PoliticalDemocracy,
#' std.lv = TRUE, 
#' estimator = "MLR", 
#' meanstructure = TRUE)
#' lav_fdr(fit = fit)
#'
#' @importFrom rlang %||%
#' @export
lav_fdr <- function(
    fit,
    ops = c("reg", "load", "var.cov"),
    family = c("by_group", "selected"),
    method = "BY",
    alpha = 0.05,
    standardized = c("std.all", "std.lv", "std.nox", "none")
) {
  if (!inherits(fit, "lavaan")) stop("`fit` must be a fitted lavaan object.")
  
  ops <- match.arg(ops)
  family <- match.arg(family)
  standardized <- match.arg(standardized)
  
  if (!is.character(method) || length(method) != 1L) stop("`method` must be a single character value.")
  if (!is.numeric(alpha) || length(alpha) != 1L || is.na(alpha) || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).")
  }
  
  OPT <- fit@Options
  group_var <- OPT$group
  if (is.null(group_var) || !nzchar(group_var)) group_var <- NULL
  group_labels <- tryCatch(lavaan::lavInspect(fit, "group.label"), error = function(e) NULL)
  
  # estrai stime parametri
  pe <- lavaan::parameterEstimates(fit, standardized = TRUE)
  
  op_map <- list(
    reg = "~",
    load = "=~",
    var.cov = "~~"
  )
  op_target <- op_map[[ops]]

  sel <- rep(TRUE, nrow(pe))
  sel <- sel & (pe$op == op_target)
  
  # no intercette di default
  sel <- sel & !(pe$op == "~1" | (pe$op == "~" & pe$rhs == "1"))
  # no threshold
  sel <- sel & !(pe$op == "|")
  # no parametri definiti
  sel <- sel & !(pe$op == ":=")
  # no interazioni
  is_int <- (pe$op == "XWITH") |
    (!is.na(pe$rhs) & grepl(":", pe$rhs, fixed = TRUE)) |
    (!is.na(pe$lhs) & grepl(":", pe$lhs, fixed = TRUE))
  sel <- sel & !is_int
  
  pe2 <- pe[sel, , drop = FALSE]
  if (!nrow(pe2)) stop("No parameters matched the selection criteria for FDR correction.")
  
  # cerca che esistano le colonne
  if (!("group" %in% names(pe2))) pe2$group <- 1L
  if (all(is.na(pe2$group))) pe2$group <- 1L
  
  # trova i pvalue che vanno corretti
  ok_p <- !is.na(pe2$pvalue) & is.finite(pe2$pvalue)
  p_adj <- rep(NA_real_, nrow(pe2))
  
  if (family == "by_group" && length(unique(pe2$group)) > 1L) {
    groups <- sort(unique(pe2$group))
    for (g in groups) {
      idx <- which(pe2$group == g & ok_p)
      if (length(idx)) p_adj[idx] <- stats::p.adjust(pe2$pvalue[idx], method = method)
    }
  } else {
    idx <- which(ok_p)
    if (length(idx)) p_adj[idx] <- stats::p.adjust(pe2$pvalue[idx], method = method)
  }
  
  # selezione della colonna standardizzata
  std_col <- NA_character_
  std_val <- rep(NA_real_, nrow(pe2))
  if (standardized != "none") {
    std_col <- standardized
    if (std_col %in% names(pe2)) std_val <- pe2[[std_col]]
  }
  
  fdr_table <- data.frame(
    lhs = pe2$lhs,
    op = pe2$op,
    rhs = pe2$rhs,
    group = pe2$group,
    est = pe2$est,
    se = pe2$se,
    z = pe2$z,
    p = pe2$pvalue,
    p_fdr = p_adj,
    sig_fdr = (!is.na(p_adj)) & (p_adj < alpha),
    stringsAsFactors = FALSE
  )
  
  if (standardized != "none") {
    fdr_table[[std_col]] <- std_val
  }
  
  fdr_table <- fdr_table[order(fdr_table$group, fdr_table$lhs, fdr_table$op, fdr_table$rhs), , drop = FALSE]
  rownames(fdr_table) <- NULL
  
  res <- list(
    fdr_table = fdr_table,
    settings = list(
      ops = ops,
      family = family,
      method = method,
      alpha = alpha,
      standardized = standardized
    ),
    group_var = group_var,
    group_labels = group_labels %||% NULL,
    call = match.call()
  )
  class(res) <- c("lav_fdr", "list")
  res
}

### metodi S3 print e summary:

#' @param x A 'lav_fdr' object.
#' @param ... Additional arguments; unused.
#' @rdname lav_fdr
#' @export
print.lav_fdr <- function(x, ...) {
  tbl <- x$fdr_table
  gl <- x$group_labels
  st <- x$settings
  
  if (!nrow(tbl)) {
    cat("No parameters in fdr_table.\n")
    return(invisible(x))
  }
  
  fmt  <- function(v, k = 3) if (is.na(v)) "NA" else formatC(v, format = "f", digits = k)
  fmtp <- function(p) {
    if (is.na(p)) return("NA")
    if (p < .001) return("< .001")
    out <- formatC(p, digits = 3, format = "f")
    sub("^0\\.", ".", out)
  }
  zfmt <- function(z) if (is.na(z)) "NA" else formatC(z, format = "f", digits = 3)
  
  std_name <- intersect(names(tbl), c("std.all", "std.lv", "std.nox"))
  std_name <- if (length(std_name)) std_name[1] else NULL
  
  groups <- sort(unique(tbl$group))
  for (g in groups) {
    gname <- if (length(gl) && !is.na(g) && g <= length(gl)) gl[g] else paste0("Group ", g)
    sub <- tbl[tbl$group == g, , drop = FALSE]
    par_texts <- paste(sub$lhs, sub$op, sub$rhs)
    par_width <- max(nchar(par_texts) + 2, 9)
    
    cat("\nFDR-corrected parameters\n")
    cat(sprintf("Ops: %s | Family: %s | Method: %s | alpha: %.3f | %s\n",
                st$ops, st$family, st$method, st$alpha, gname))
    
    dash_n <- (if (is.null(std_name)) 76 else 88) + 12 + (par_width - 30)
    
    cat(rep("-", dash_n), "\n", sep = "")
    if (is.null(std_name)) {
      cat(sprintf("%-*s %10s %8s %8s %10s %10s\n", par_width,
                  "Parameter", "est", "SE", "z", "p", "p_FDR"))
    } else {
      cat(sprintf("%-*s %10s %8s %8s %10s %10s %10s\n", par_width,
                  "Parameter", "est", "SE", "z", "p", "p_FDR", std_name))
    }
    cat(rep("-", dash_n), "\n", sep = "")
    
    for (i in seq_len(nrow(sub))) {
      par_txt <- paste(sub$lhs[i], sub$op[i], sub$rhs[i])
      flag <- if (isTRUE(sub$sig_fdr[i])) "*" else ""
      if (is.null(std_name)) {
        cat(sprintf("%-*s %10s %8s %8s %10s %10s %s\n", par_width,
                    par_txt,
                    fmt(sub$est[i]),
                    fmt(sub$se[i]),
                    zfmt(sub$z[i]),
                    fmtp(sub$p[i]),
                    fmtp(sub$p_fdr[i]),
                    flag))
      } else {
        cat(sprintf("%-*s %10s %8s %8s %10s %10s %10s %s\n", par_width,
                    par_txt,
                    fmt(sub$est[i]),
                    fmt(sub$se[i]),
                    zfmt(sub$z[i]),
                    fmtp(sub$p[i]),
                    fmtp(sub$p_fdr[i]),
                    fmt(sub[[std_name]][i]),
                    flag))
      }
    }
    cat(rep("-", dash_n), "\n", sep = "")
    cat(sprintf("FDR-significant (p_FDR < alpha): %d / %d\n",
                sum(sub$sig_fdr, na.rm = TRUE), nrow(sub)))
  }
  
  cat("\nNote. '*' marks parameters with FDR-adjusted p-value < alpha.\n")
  invisible(x)
}

#' @param object A 'lav_fdr' object.
#' @param ... Passed to \code{print.lav_fdr()}.
#' @rdname lav_fdr
#' @export
summary.lav_fdr <- function(object, ...) {
  print.lav_fdr(object, ...)
  invisible(object)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_slopes.R
\name{lav_slopes}
\alias{lav_slopes}
\alias{print.lav_slopes}
\alias{summary.lav_slopes}
\title{Simple slopes and interaction plots for fitted 'lavaan' models}
\usage{
lav_slopes(
  fit,
  outcome,
  pred,
  modx,
  interaction,
  data = NULL,
  modx.values = NULL,
  modx.labels = NULL,
  pred.range = NULL,
  conf.level = 0.95,
  x.label = NULL,
  y.label = NULL,
  legend.title = NULL,
  colors = NULL,
  line.size = 0.80,
  alpha = 0.20,
  table = TRUE,
  digits = 3L,
  modx_n_unique_cutoff = 4L,
  return_data = FALSE
)

\method{print}{lav_slopes}(x, ...)

\method{summary}{lav_slopes}(object, ...)
}
\arguments{
\item{fit}{A fitted 'lavaan' object that includes the product term (required).}

\item{outcome}{Character. Name of the dependent variable in \code{fit} (required).}

\item{pred}{Character. Name of the focal predictor whose simple slopes are probed (required).}

\item{modx}{Character. Name of the moderator. The moderator must appear in
the fitted model as a single numeric variable. General nominal moderators
with more than two categories are not supported.}

\item{interaction}{Character. Name of the product term in \code{fit} (e.g., \code{"X_Z"}) (required).}

\item{data}{\code{data.frame}. Optional raw data (not needed, retained for backward compatibility). 
The function automatically recovers observed data from \code{fit} when needed.}

\item{modx.values}{Numeric vector of moderator values at which to compute
simple slopes. If \code{NULL} and \code{modx} is observed and numeric, the
function uses mean minus 1 SD, the mean, and mean plus 1 SD for moderators
with more than \code{modx_n_unique_cutoff} unique values, and otherwise
uses the observed numeric values as discrete probe points. If \code{modx}
is latent in a single-group model, the function uses the estimated latent
mean minus 1 latent SD, the latent mean, and the latent mean plus 1 latent
SD.}

\item{modx.labels}{Character vector. Legend and table labels for \code{modx.values}.
By default, the labels are \code{c("-1 SD", "Mean", "+1 SD")} when values are derived as 
mean plus or minus 1 SD, and \code{as.character(modx.values)} otherwise.}

\item{pred.range}{Numeric vector of length 2. Range \code{c(min, max)} for the x-axis
for the focal predictor. If \code{NULL} and \code{pred} is observed, the observed 
range recovered from \code{fit} is used. If \code{pred} is latent in a single-group model, 
the function uses the estimated latent mean minus 2 latent SD and the latent mean plus 2 
latent SD. Otherwise, \code{c(-2, 2)} is used.}

\item{conf.level}{Numeric in (0,1). Confidence level for Wald confidence intervals and ribbons (default: 0.95).}

\item{x.label}{Character. X-axis label (default: \code{pred}).}

\item{y.label}{Character. Y-axis label (default: \code{outcome}).}

\item{legend.title}{Character. Legend title; if \code{NULL}, the legend shows only levels (default: NULL).}

\item{colors}{Character vector. Colors for lines and ribbons; named vector recommended with names matching \code{modx.labels} (default: Okabe-Ito palette).}

\item{line.size}{Numeric > 0. Line width (default: 0.80).}

\item{alpha}{Numeric in (0,1). Ribbon opacity (default 0.20).}

\item{table}{Logical. Print APA-style interaction test and simple-slopes table (default: \code{TRUE}).}

\item{digits}{Integer \code{>= 0}. Decimal digits in printed output (default: 3).}

\item{modx_n_unique_cutoff}{Integer \code{>= 1}. Threshold for treating a numeric moderator
as continuous and using mean ± SD (default: 4).}

\item{return_data}{Logical. If \code{TRUE}, include the plotting data.frame in the returned list (default: FALSE).}

\item{x}{A 'lav_slopes' object.}

\item{...}{Additional arguments; unused.}

\item{object}{A 'lav_slopes' object.}
}
\value{
A list of class \code{"lav_slopes"} with elements:
\describe{
  \item{\code{plot}}{\code{ggplot} object with lines and confidence ribbons.}
  \item{\code{slope_table}}{Data frame with moderator levels, simple slopes, SE, z, and CI.}
  \item{\code{plot_data}}{Only when \code{return_data = TRUE}: data used to build the plot.}
}
}
\description{
Computes conditional (simple) slopes of a focal predictor across values
of a moderator from a fitted 'lavaan' model that includes their explicit 
product term. Plots predicted lines with Wald confidence ribbons and prints
an APA-style test of the interaction for easy reporting and interpretation,
together with a simple slopes table.
}
\details{
The model should include a main effect for the predictor, a main effect for
the moderator, and one explicit product term between them.

The moderator must enter the fitted model as a single numeric
variable, which may be continuous, a binary observed moderator coded as a
single numeric dummy variable, an observed numeric moderator with a small
number of values treated as discrete probe points, or a latent moderator
treated as a single continuous latent variable. 

Standard errors use the delta method with the model covariance matrix of the
estimates. When moderator values are derived automatically for latent
moderators, probe points are based on the estimated latent mean and
model-implied latent standard deviation.
}
\section{Notes}{

Estimates are unstandardized; a standardized coefficient for the interaction is
also reported for reference. Wald tests assume large-sample normality of the
parameter estimates. Multigroup fitted models are not supported.
}

\examples{
set.seed(42)
X <- rnorm(100); Z <- rnorm(100); X_Z <- X*Z
Y <- 0.6*X + 0.6*Z + 0.3*X_Z + rnorm(100, sd = 0.7) 
dataset <- data.frame(Y, X, Z, X_Z)
fit <- lavaan::sem("Y ~ X + Z + X_Z", data = dataset)
lav_slopes(
fit = fit, 
outcome = "Y", 
pred = "X", 
modx = "Z", 
interaction = "X_Z")

}
\references{
Aiken, L. S., & West, S. G. (1991). \emph{Multiple regression: Testing and
interpreting interactions}. Sage.

Preacher, K. J., Curran, P. J., & Bauer, D. J. (2006). Computational tools
for probing interactions in multiple linear regression, multilevel modeling,
and latent curve analysis. \emph{Journal of Educational and Behavioral
Statistics}, \emph{31}(4), 437-448.
\doi{10.3102/10769986031004437}

Rogosa, D. (1980). Comparing nonparallel regression lines.
\emph{Psychological Bulletin}, \emph{88}(2), 307-321.
\doi{10.1037/0033-2909.88.2.307}
}
\seealso{
\code{\link{lav_jn}} for Johnson-Neyman regions of significance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_cv.R
\name{lav_cv}
\alias{lav_cv}
\alias{print.lav_cv}
\alias{summary.lav_cv}
\title{Repeated holdout (Monte Carlo) cross-validation of \eqn{R^2} for structural equation models ('lavaan' objects)}
\usage{
lav_cv(
  fit,
  data = NULL,
  times = "auto",
  train_prop = 0.8,
  seed = 42L,
  quiet = TRUE,
  digits = 3L,
  plot = TRUE,
  tol = 0.001,
  window = 50L,
  max_times = 3000L,
  min_r2_for_pct = 0.05
)

\method{print}{lav_cv}(x, digits = x$digits \%||\% 3L, ...)

\method{summary}{lav_cv}(object, ...)
}
\arguments{
\item{fit}{A fitted 'lavaan' object (required).}

\item{data}{The data frame used to fit the model; if NULL, it is extracted from 'fit' when available (default: NULL).}

\item{times}{Integer indicating the number of random splits, or "auto" for stabilization-based early stopping (default: "auto").}

\item{train_prop}{Numeric in (0,1). Proportion of cases in the training split for each repetition (default: 0.8).}

\item{seed}{Integer. Random seed for reproducibility of the splits (default: 42).}

\item{quiet}{Logical. Suppress 'lavaan' refit messages when TRUE (default: TRUE).}

\item{digits}{Integer. Number of digits to print in summaries (default: 3).}

\item{plot}{Logical. Show convergence plots of the running mean \eqn{R^2} per outcome (default: TRUE).}

\item{tol}{Numeric. Tolerance for the auto-stop rule on the running mean (default: 0.001).}

\item{window}{Integer. Trailing window size (number of successful splits) used by the auto-stop rule (default: 50).}

\item{max_times}{Integer. Maximum number of splits when \code{times} = "auto" (default: 3000).}

\item{min_r2_for_pct}{Numeric in (0,1). Minimum in-sample \eqn{R^2} required to compute percent drop; below this, \%_drop is set to NA (default: 0.05).}

\item{x}{A 'lav_cv' object.}

\item{...}{Additional arguments; unused.}

\item{object}{A 'lav_cv' object.}
}
\value{
A list with class 'lav_cv' and elements:
\describe{
  \item{\code{table}}{Data frame with columns:
    \code{outcome}, \code{type} ("observed" or "latent"),
    \code{r2_in}, \code{r2_cv_mean}, \code{r2_cv_median}, \code{r2_cv_sd},
    \code{drop_mean_pct}, \code{drop_med_pct}, \code{splits_used}.}
  \item{\code{split_matrix}}{Matrix of split-wise test-set \eqn{R^2} values (rows = splits, columns = outcomes).}
  \item{\code{times}}{Character or integer indicating the number of splits used (e.g., \code{"auto(534)"} or \code{500}).}
  \item{\code{train_prop}}{Numeric. Training proportion used in each split.}
  \item{\code{N}}{Integer. Number of rows in the input data.}
  \item{\code{seed}}{Integer. Random seed used to generate the splits.}
  \item{\code{tol}}{Numeric. Tolerance used by the auto-stop rule.}
  \item{\code{window}}{Integer. Trailing window size for the auto-stop rule.}
  \item{\code{min_r2_for_pct}}{Numeric. Minimum in-sample \eqn{R^2} required to compute percent drop.}
  \item{\code{call}}{\code{match.call()} of the function call.}
  \item{\code{digits}}{Integer. Default number of digits for printing.}
}
}
\description{
Estimate out-of-sample predictive performance for structural relations in a
fitted 'lavaan' model using repeated holdout (Monte Carlo cross-validation,
leave-group-out CV). At each repetition, the model is refitted on a random
training subset and evaluated on a disjoint test subset.
}
\details{
For observed outcomes, \eqn{R^2} is computed by comparing test-set observed values
with predictions obtained by applying the training-set structural coefficients
to the test-set predictors.

For latent outcomes, the outcome is not directly observed in the test set.
Factor scores for the outcome are first computed in the test set using the
measurement model learned on the training set; these scores serve as the
outcome values. Predictions are then formed by applying the training-set
structural coefficients to the test-set predictors (including factor scores
for any latent predictors). \eqn{R^2} is computed by comparing the test-set factor
scores of the outcome with these predicted scores.

The in-sample baseline \eqn{R^2} is computed on the full dataset using the same
metric as in cross-validation: observed outcomes use observed-versus-predicted
\eqn{R^2}; latent outcomes use score-versus-predicted-score R^2.

By default, repetitions continue until the running mean \eqn{R^2} for each outcome
stabilizes within a specified tolerance over a trailing window of successful
splits, or until a maximum number of splits is reached.

The summary table reports the in-sample baseline \eqn{R^2}, the median cross-validated
\eqn{R^2}, its standard deviation, and the percent drop (baseline vs. median CV) with
heuristic threshold markers. The percent drop is suppressed when the in-sample
\eqn{R^2} is very small.
}
\examples{
set.seed(42)
model <- "
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8

dem60 ~ ind60
dem65 ~ ind60 + dem60

y1 ~~ y5
y2 ~~ y6
"
fit <- lavaan::sem(
model = model, 
data = lavaan::PoliticalDemocracy,
std.lv = TRUE, 
estimator = "MLR", 
meanstructure = TRUE)
result <- lav_cv(
fit = fit, 
data = lavaan::PoliticalDemocracy, 
times = 5)
print(result)

}
\references{
Cudeck, R., & Browne, M. W. (1983). Cross-validation of covariance
structures. \emph{Multivariate Behavioral Research}, \emph{18}(2), 147-167.
\doi{10.1207/s15327906mbr1802_2}

Hastie, T., Tibshirani, R., & Friedman, J. (2001). \emph{The elements of
statistical learning: Data mining, inference, and prediction}. Springer.
\doi{10.1007/978-0-387-21606-5}

Kvålseth, T. O. (1985). Cautionary note about \eqn{R^2}.
\emph{The American Statistician}, \emph{39}(4), 279-285.
\doi{10.1080/00031305.1985.10479448}

Shmueli, G. (2010). To explain or to predict?
\emph{Statistical Science}, \emph{25}(3), 289-310.
\doi{10.1214/10-STS330}

Yarkoni, T., & Westfall, J. (2017). Choosing prediction over explanation in
psychology: Lessons from machine learning. \emph{Perspectives on
Psychological Science}, \emph{12}(6), 1100-1122.
\doi{10.1177/1745691617693393}
}
\seealso{
\code{\link[lavaan]{sem}}, \code{\link[lavaan]{lavPredict}},
  \code{\link[lavaan]{inspect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_vif.R
\name{lav_vif}
\alias{lav_vif}
\alias{print.lav_vif}
\alias{summary.lav_vif}
\title{Variance Inflation Factors for 'lavaan' Structural Predictors}
\usage{
lav_vif(
  fit,
  data = NULL,
  quiet = TRUE
)

\method{print}{lav_vif}(x, digits = 3, cutoff = c(5, 10), ...)

\method{summary}{lav_vif}(object, ...)
}
\arguments{
\item{fit}{A fitted \code{lavaan} object.}

\item{data}{Optional. The data frame used to fit \code{fit}. If \code{NULL},
the function attempts to extract the data from \code{fit} via
\code{lavInspect(fit, "data")}. For multigroup models, grouped data are
reconstructed into a single data frame when possible.}

\item{quiet}{Logical. If \code{TRUE} suppresses lavaan refit messages.}

\item{x}{A 'lav_vif' object.}

\item{digits}{Integer number of digits to print.}

\item{cutoff}{Numeric length-2 thresholds used to flag VIF values.}

\item{...}{Passed to `print.lav_vif()` (e.g., `digits`, `cutoff`).}

\item{object}{A 'lav_vif' object.}
}
\value{
A list with:
\itemize{
  \item \code{vif_table}: data.frame with columns
        \code{outcome}, \code{predictor}, \code{group}, \code{r2_predictor},
        \code{vif}, \code{k_predictors}.
  \item \code{outcome_r2}: data.frame with \eqn{R^2} per eligible endogenous
        \code{outcome} and \code{group} from the original \code{fit}.
}
}
\description{
Compute VIF for each predictor that appears in structural regressions
with two or more predictors, refitting the necessary sub-models so that
latent predictors are handled at the latent level (i.e., with their original
measurement models). It returns also the \eqn{R^2} of each eligible endogenous variable 
from the original fit for context.
}
\details{
Each auxiliary refitted model includes the measurement structure for latent 
predictors, rebuilt from the original model syntax, together with observed 
residual covariances among the involved indicators when specified in the 
original model. Then, regresses the focal predictor on the remaining predictors 
at the latent level when applicable.

\eqn{\mathrm{VIF}_i = 1 / (1 - R_i^2)} generalizes VIF to SEM while respecting 
measurement models.
  
The function reuses the estimator, missing-data handling, and several options 
from \code{fit}. 
In multigroup models, VIF is computed separately within each group. 
Higher-order latent predictors are supported by recursively rebuilding
lower-order measurement blocks in the auxiliary refitted models.
}
\examples{
set.seed(42)
x1 <- rnorm(100); x2 <- 0.85*x1 + rnorm(100, sd = sqrt(1 - 0.85^2)); x3 <- rnorm(100)
y <- 0.5*x1 + 0.3*x2 + 0.1*x3 + rnorm(100, sd = 0.7)
dataset <- data.frame(y, x1, x2, x3)
fit <- lavaan::sem("y ~ x1 + x2 + x3", data = dataset)
lav_vif(
fit = fit,
data = dataset)

}
\references{
Fox, J., & Monette, G. (1992). Generalized collinearity diagnostics.
\emph{Journal of the American Statistical Association}, \emph{87}(417),
178-183. \doi{10.1080/01621459.1992.10475190}

Belsley, D. A., Kuh, E., & Welsch, R. E. (1980).
\emph{Regression diagnostics: Identifying influential data and sources of
collinearity}. Wiley.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_localfit.R
\name{lav_localfit}
\alias{lav_localfit}
\alias{print.lav_localfit}
\alias{summary.lav_localfit}
\alias{plot.lav_localfit}
\title{Local fit diagnostics for fitted \code{lavaan} models}
\usage{
lav_localfit(
  fit,
  group = 1L,
  type = "cor.bentler",
  thresholds = c(1.96, 2.58),
  top_n = 10L,
  triangle = "lower",
  include_diagonal = FALSE,
  plot = TRUE,
  plot_style = "trafficlight",
  show_values = FALSE,
  digits = 3L,
  good_color = "#009E73",
  moderate_color = "#F0E442",
  poor_color = "#D55E00",
  neg_color = "#3B4CC0",
  pos_color = "#B40426",
  na_color = "grey90",
  return_data = FALSE
)

\method{print}{lav_localfit}(x, ...)

\method{summary}{lav_localfit}(object, ...)

\method{plot}{lav_localfit}(x, y = NULL, ...)
}
\arguments{
\item{fit}{A fitted \code{lavaan} object.}

\item{group}{Group to inspect in a multigroup model. May be either a numeric
group index or a character group label. Ignored for single-group models.}

\item{type}{Character string passed to \code{lavaan::lavResiduals()}.
Common choices are \code{"cor.bentler"} (default), \code{"raw"},
\code{"cor"}, and \code{"cor.bollen"}.}

\item{thresholds}{Numeric vector of length 2 giving the descriptive cutoffs
used for standardized residual magnitudes. The first value separates
\code{"minor"} from \code{"moderate"} discrepancies; the second separates
\code{"moderate"} from \code{"notable"} discrepancies. Used for the
traffic-light heatmap and for printed summaries. Defaults to
\code{c(1.96, 2.58)}, as per common z-based guidelines.}

\item{top_n}{Integer. Number of largest absolute standardized covariance
residuals to report in the output table.}

\item{triangle}{Character string. Either \code{"lower"} or \code{"full"}.
If \code{"lower"}, only the lower triangle of the covariance residual
matrix is used in the heatmap and top-residual table.}

\item{include_diagonal}{Logical. If \code{TRUE}, include diagonal elements
(variance residuals) in the covariance residual diagnostics.}

\item{plot}{Logical. If \code{TRUE}, produce a heatmap of covariance residual
misfit.}

\item{plot_style}{Character string. Either \code{"trafficlight"} or
\code{"signed"}. The default \code{"trafficlight"} plots the absolute
standardized covariance residuals in descriptive magnitude categories. The
\code{"signed"} option plots signed standardized residuals on a diverging
color scale.}

\item{show_values}{Logical. If \code{TRUE}, overlay numeric residual values on
the heatmap tiles.}

\item{digits}{Non-negative integer giving the default number of digits used
in printed output and numeric labels.}

\item{good_color}{Fill color for \code{"minor"} discrepancies in the
traffic-light heatmap.}

\item{moderate_color}{Fill color for \code{"moderate"} discrepancies in the
traffic-light heatmap.}

\item{poor_color}{Fill color for \code{"notable"} discrepancies in the
traffic-light heatmap.}

\item{neg_color}{Fill color for negative residuals in the signed heatmap.}

\item{pos_color}{Fill color for positive residuals in the signed heatmap.}

\item{na_color}{Fill color for cells omitted from the heatmap, such as the
upper triangle when \code{triangle = "lower"}.}

\item{return_data}{Logical. If \code{TRUE}, include the long-format heatmap
data frame in the returned object.}

\item{x}{A \code{"lav_localfit"} object.}

\item{...}{Additional arguments; unused.}

\item{object}{A \code{"lav_localfit"} object.}

\item{y}{Ignored.}
}
\value{
A list of class \code{"lav_localfit"} with elements:
\itemize{
  \item \code{summary}: residual summary returned by
    \code{lavaan::lavResiduals()} for the selected group;
  \item \code{cov_residuals}: covariance residual matrix for the selected
    group;
  \item \code{cov_z}: standardized covariance residual matrix;
  \item \code{mean_residuals}: mean residual vector, if available;
  \item \code{mean_z}: standardized mean residual vector, if available;
  \item \code{top_cov_residuals}: data frame of the largest absolute
    standardized covariance residuals;
  \item \code{top_mean_residuals}: data frame of the largest absolute
    standardized mean residuals;
  \item \code{counts}: list with counts of covariance residuals exceeding the
    descriptive thresholds, the maximum absolute standardized covariance
    residual, and the corresponding variable pair;
  \item \code{plot}: a \code{ggplot} heatmap object, or \code{NULL} if
    \code{plot = FALSE};
  \item \code{group}: numeric index of the selected group;
  \item \code{group_label}: character label of the selected group;
  \item \code{type}: residual type used;
  \item \code{thresholds}: thresholds used for descriptive classification;
  \item \code{digits}: default number of digits for printing;
  \item \code{call}: matched function call;
  \item \code{plot_data}: only when \code{return_data = TRUE}, the long-format
    data used to build the heatmap.
}
}
\description{
Evaluate local fit in a fitted \code{lavaan} model from residual-based
diagnostics. The function extracts residual summaries, computes standardized 
covariance and mean residual diagnostics, identifies the largest local 
discrepancies, and builds a heatmap of covariance residual misfit. 
By default, the function computes Bentler-standardized residual summaries and
standardized covariance residual diagnostics suitable for screening localized
strain in the model. 
The default plot is a traffic-light heatmap of the absolute standardized
covariance residuals.
}
\section{Notes}{

In this function, local fit is evaluated from residuals of the sample summary
statistics rather than from casewise prediction errors. Standardized residuals
are used to screen where the model reproduces specific covariances or means
poorly. The default thresholds are descriptive aids only. They should not be
treated as formal decision rules or as substitutes for theory-based model
evaluation. When the fitted object uses bootstrap standard errors,
standardized residual diagnostics are obtained from an auxiliary refit of the
same model without bootstrap.
}

\examples{
model <- "
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8
dem60 ~ ind60
dem65 ~ ind60 + dem60
y1 ~~ y5
y2 ~~ y6
"
fit <- lavaan::sem(
model = model, 
data = lavaan::PoliticalDemocracy,
std.lv = TRUE, 
estimator = "MLR", 
meanstructure = TRUE)

lav_localfit(fit)

}
\references{
Rosseel, Y. (2012). lavaan: An R package for structural equation modeling.
\emph{Journal of Statistical Software}, \emph{48}(2), 1-36.
\doi{10.18637/jss.v048.i02}

Schermelleh-Engel, K., Moosbrugger, H., & Muller, H. (2003). Evaluating the
fit of structural equation models: Tests of significance and descriptive
goodness-of-fit measures. \emph{Methods of Psychological Research Online},
\emph{8}(2), 23-74.
}
\seealso{
\code{\link[lavaan]{lavResiduals}} for residual extraction and
  \code{\link{lav_deltaR2}} for local structural effect-size diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_deltaR2.R
\name{lav_deltaR2}
\alias{lav_deltaR2}
\alias{print.lav_deltaR2}
\alias{summary.lav_deltaR2}
\title{Incremental effect sizes for structural predictors in fitted \code{lavaan} models}
\usage{
lav_deltaR2(
  fit,
  data = NULL,
  outcome = NULL,
  terms = NULL,
  block = NULL,
  quiet = TRUE,
  digits = 3L
)

\method{print}{lav_deltaR2}(x, ...)

\method{summary}{lav_deltaR2}(object, ...)
}
\arguments{
\item{fit}{A fitted \code{lavaan} object.}

\item{data}{Optional data frame used to fit \code{fit}. If supplied, this data
frame is used to refit reduced models. If \code{NULL}, reduced models are
refit using the internal data and options stored in \code{fit}.}

\item{outcome}{Optional character vector naming endogenous variables for which
incremental effect sizes should be computed. If \code{NULL}, all endogenous
variables with at least one free structural predictor are used.}

\item{terms}{Optional character vector naming predictors to test one at a
time. Ignored if \code{block} is supplied. If \code{NULL}, all eligible
structural predictors are tested individually.}

\item{block}{Optional character vector naming predictors to remove jointly.
When supplied, one reduced-model comparison is computed per selected outcome
for the entire predictor block. A block is evaluated only for outcomes in
which all named predictors appear as free structural regressors.}

\item{quiet}{Logical. If \code{TRUE}, suppress reduced-model refit messages.}

\item{digits}{Non-negative integer giving the default number of digits used in
printing.}

\item{x}{A 'lav_deltaR2' object.}

\item{...}{Additional arguments; unused.}

\item{object}{A 'lav_deltaR2' object.}
}
\value{
A list with:
\itemize{
  \item \code{delta_r2_table}: A data frame containing one row per
  outcome-by-test comparison, with columns:
  \itemize{
    \item \code{outcome}: endogenous variable whose \eqn{R^2} is evaluated;
    \item \code{tested_set}: predictor or predictor block fixed to zero in the
      reduced model;
    \item \code{test_type}: either \code{"single"} or \code{"block"};
    \item \code{n_terms}: number of predictors removed jointly;
    \item \code{group}: numeric group index;
    \item \code{group_label}: group label, if available;
    \item \code{r2_full}: \eqn{R^2} from the fitted model;
    \item \code{r2_reduced}: \eqn{R^2} from the reduced model;
    \item \code{delta_r2}: \eqn{R^2_{full} - R^2_{reduced}};
    \item \code{part_r2}: equal to \code{delta_r2} in this implementation;
    \item \code{f2}: Cohen's \eqn{f^2};
    \item \code{f2_magnitude}: conventional descriptive interpretation of
      \code{f2} based on Cohen's benchmarks (\code{"negligible"},
      \code{"small"}, \code{"medium"}, or \code{"large"});
    \item \code{converged}: logical indicator of whether the reduced model
      converged.
  }
  \item \code{settings}: list of user-supplied function settings;
  \item \code{group_var}: grouping variable name, or \code{NULL};
  \item \code{group_labels}: group labels, if available;
  \item \code{call}: matched function call;
  \item \code{digits}: default number of digits used for printing.
}
}
\description{
Compute outcome-specific incremental effect sizes for structural predictors in
a fitted \code{lavaan} model by comparing the fitted model with nested reduced
models in which one predictor, or a block of predictors, is fixed to zero.
For each tested predictor set, the function returns the reduction in explained
variance (\eqn{\Delta R^2}), the corresponding part \eqn{R^2}, and Cohen's
\eqn{f^2 = \Delta R^2 / (1 - R^2_{full})}.
}
\details{
Reduced models preserve all untargeted model parameters and differ from the
fitted model only in that the tested structural regression path(s) are fixed
to zero. Accordingly, the resulting \eqn{\Delta R^2} quantifies the unique
contribution of the tested predictor or predictor block to the explained
variance of a given endogenous variable, conditional on the rest of the model.

If \code{data} is supplied, reduced models are refit from that data set.
Otherwise, the function reuses the internal data and fitting options stored in
\code{fit}. The function is intended for converged \code{lavaan} models with
free structural regressions.
}
\section{Notes}{

For a given endogenous variable, \eqn{\Delta R^2} is the reduction in
explained variance obtained when the tested predictor or predictor block is
fixed to zero, and the reduced model is refit with all untargeted parameters
left free.
In this implementation, part \eqn{R^2} is numerically identical to \eqn{\Delta R^2}. 
Cohen's \eqn{f^2} expresses the same quantity relative to the residual variance of the
full model. The \code{f2_magnitude} column applies Cohen's conventional 
benchmarks to \eqn{f^2} and is intended only as a descriptive aid.
}

\examples{
library("lavaan")
data("PoliticalDemocracy", 
package = "lavaan")
model <- "
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8
dem60 ~ ind60
dem65 ~ ind60 + dem60
y1 ~~ y5
y2 ~~ y4 + y6
y3 ~~ y7
y4 ~~ y8
y6 ~~ y8
"
fit <- lavaan::sem(model, 
data = PoliticalDemocracy)
lav_deltaR2(fit = fit)
lav_deltaR2(fit = fit, outcome = "dem65")
lav_deltaR2(fit = fit, outcome = "dem65", terms = "ind60")
lav_deltaR2(fit = fit, outcome = "dem65", block = c("ind60", "dem60"))

}
\references{
Cohen, J. (1988). \emph{Statistical power analysis for the behavioral sciences}
(2nd ed.). Lawrence Erlbaum Associates.

Rosseel, Y. (2012). lavaan: An R package for structural equation modeling.
\emph{Journal of Statistical Software}, \emph{48}(2), 1-36.
\doi{10.18637/jss.v048.i02}
}
\seealso{
\code{\link{lav_cv}} for cross-validated \eqn{R^2}, \code{\link{lav_vif}}
  for variance inflation factors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lavinteract-package.R
\docType{package}
\name{lavinteract}
\alias{lavinteract}
\alias{lavinteract-package}
\title{Post-Estimation Utilities for 'lavaan' Fitted Models}
\description{
\if{html}{\figure{logo.png}{options: style="float: right; margin: 0 0 8px 12px;" alt="lavinteract logo" width="110"}}

Post-estimation tools for structural equation models fitted with 'lavaan'.
Provides methods for probing observed and latent interactions, diagnosing local
misfit and multicollinearity, quantifying incremental effect sizes for 
structural predictors, assessing predictive performance by repeated holdout 
cross-validation, and adjusting selected parameter p-values for multiple 
testing. Functions operate from a fitted model object and, when needed, refit 
auxiliary or reduced models while preserving the original SEM specification.
}
\details{
The functions are:
\itemize{
  \item \code{\link{lav_slopes}}: simple slopes and interaction plots from a fitted 'lavaan' model.
  \item \code{\link{lav_jn}}: Johnson-Neyman regions of significance for continuous moderators in a fitted 'lavaan' model.
  \item \code{\link{lav_deltaR2}}: incremental effect sizes (part \eqn{R^2} and Cohen's \eqn{f^2}) for structural predictors via reduced-model comparisons.
  \item \code{\link{lav_localfit}}: residual-based local fit diagnostics and heatmaps for fitted 'lavaan' models.
  \item \code{\link{lav_vif}}: variance inflation factors for structural predictors with measurement preserved.
  \item \code{\link{lav_cv}}: repeated holdout (Monte Carlo) cross-validation of \eqn{R^2} for SEM outcomes.
  \item \code{\link{lav_fdr}}: false discovery rate correction for selected 'lavaan' parameter p-values.
}
}
\section{Note}{

The development of this package grew from ongoing discussions and interactions (sic)
with colleagues, in particular Dr. Cataldo Giuliano Gemmano, whose steady 
feedback and support helped shape it.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/g-corbelli/lavinteract}
  \item Report bugs at \url{https://github.com/g-corbelli/lavinteract/issues}
}

}
\author{
Giuseppe Corbelli (<giuseppe.corbelli@uninettunouniversity.net>)
}
\keyword{R2}
\keyword{SEM}
\keyword{diagnostics}
\keyword{interactions}
\keyword{lavaan}
\keyword{localfit}
\keyword{moderation}
\keyword{multicollinearity}
\keyword{multipletesting}
\keyword{plotting}
\keyword{residuals}
\keyword{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_fdr.R
\name{lav_fdr}
\alias{lav_fdr}
\alias{print.lav_fdr}
\alias{summary.lav_fdr}
\title{False Discovery Rate (FDR) Correction for 'lavaan' parameter p-values}
\usage{
lav_fdr(
  fit,
  ops = c("reg", "load", "var.cov"),
  family = c("by_group", "selected"),
  method = "BY",
  alpha = 0.05,
  standardized = c("std.all", "std.lv", "std.nox", "none")
)

\method{print}{lav_fdr}(x, ...)

\method{summary}{lav_fdr}(object, ...)
}
\arguments{
\item{fit}{A fitted \code{lavaan} object.}

\item{ops}{Character. One of \code{"reg"} (regressions), \code{"load"} (factor loadings),
or \code{"var.cov"} (variances/covariances/residual variances). Default is \code{"reg"}.}

\item{family}{Character. If \code{"selected"}, FDR is applied across all selected
parameters jointly. If \code{"by_group"}, FDR is applied separately within each group.
Default is \code{"by_group"}.}

\item{method}{Character method passed to \code{stats::p.adjust} (default \code{"BY"}).}

\item{alpha}{Numeric significance threshold for adjusted p-values (default 0.05).}

\item{standardized}{Which standardized column to include, or \code{"none"}.}

\item{x}{A 'lav_fdr' object.}

\item{...}{Passed to \code{print.lav_fdr()}.}

\item{object}{A 'lav_fdr' object.}
}
\value{
A list with:
\itemize{
  \item \code{fdr_table}: data.frame with raw and FDR-adjusted p-values.
  \item \code{settings}: list of settings used.
  \item \code{group_var}: group variable name (or \code{NULL}).
  \item \code{group_labels}: group labels if available.
  \item \code{call}: matched call.
}
The returned object has class \code{"lav_fdr"}.
}
\description{
Apply a false discovery rate correction (Benjamini-Yekutieli by default) to
the p-values of selected parameters from a fitted \code{lavaan} object.
}
\details{
Useful when a SEM includes many structural paths (or many other parameters 
of substantive interest) and there is the need to control the expected 
proportion of false positives among the parameters declared 'statistically 
significant'.
 
With many simultaneous tests, using \code{p < .05} for each parameter 
inflates the expected number of false positives (about \code{m * .05} under 
all true null hypotheses, where \code{m} is the number of tested parameters).
Benjamini-Yekutieli (BY) controls the False Discovery Rate (FDR) under 
arbitrary dependence structures, which is suitable for SEMs where structural 
paths are inherently dependent through shared latent variables, covariance 
matrices, and model constraints.
}
\examples{
model <- "
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8
dem60 ~ ind60
dem65 ~ ind60 + dem60
y1 ~~ y5
y2 ~~ y6
"
fit <- lavaan::sem(
model = model, 
data = lavaan::PoliticalDemocracy,
std.lv = TRUE, 
estimator = "MLR", 
meanstructure = TRUE)
lav_fdr(fit = fit)

}
\references{
Benjamini, Y., & Hochberg, Y. (1995). Controlling the false discovery rate:
A practical and powerful approach to multiple testing. \emph{Journal of the
Royal Statistical Society: Series B (Methodological)}, \emph{57}(1),
289-300. \doi{10.1111/j.2517-6161.1995.tb02031.x}

Benjamini, Y., & Yekutieli, D. (2001). The control of the false discovery
rate in multiple testing under dependency. \emph{The Annals of Statistics},
\emph{29}(4), 1165-1188. \doi{10.1214/aos/1013699998}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lav_jn.R
\name{lav_jn}
\alias{lav_jn}
\alias{print.lav_jn}
\alias{summary.lav_jn}
\title{Johnson-Neyman regions of significance for fitted 'lavaan' models}
\usage{
lav_jn(
  fit,
  outcome,
  pred,
  modx,
  interaction,
  data = NULL,
  conf.level = 0.95,
  modx.range = NULL,
  n_grid = 1000L,
  x.label = NULL,
  y.label = NULL,
  sig.color = "#009E73",
  nonsig.color = "#D55E00",
  line.color = "#0072B2",
  alpha = 0.20,
  line.size = 0.80,
  rug = TRUE,
  plot = TRUE,
  digits = 3L,
  return_data = FALSE
)

\method{print}{lav_jn}(x, ...)

\method{summary}{lav_jn}(object, ...)
}
\arguments{
\item{fit}{A fitted 'lavaan' object that includes the product term (required).}

\item{outcome}{Character. Name of the dependent variable in \code{fit} (required).}

\item{pred}{Character. Name of the focal predictor whose conditional slope
is probed (required).}

\item{modx}{Character. Name of the continuous moderator (required). If
\code{modx} is categorical (fewer than 4 unique values), the function
stops with a message directing the user to \code{\link{lav_slopes}}.}

\item{interaction}{Character. Name of the product term in \code{fit}
(e.g., \code{"X_Z"}) (required).}

\item{data}{\code{data.frame}. Optional raw data (not needed; retained for
backward compatibility). The function automatically recovers observed data
from \code{fit} when needed.}

\item{conf.level}{Numeric in (0, 1). Confidence level for the Wald band and
the JN critical value (default: 0.95).}

\item{modx.range}{Numeric vector of length 2. Range \code{c(min, max)} for
the moderator axis. If \code{NULL}, the observed range of \code{modx} in
the data is used, extended by 5\% on each side for visual padding
(default: NULL).}

\item{n_grid}{Integer. Number of moderator values at which the conditional
slope is evaluated for plotting (default: 1000).}

\item{x.label}{Character. X-axis label (default: \code{modx}).}

\item{y.label}{Character. Y-axis label (default: \code{"Conditional slope of <pred>"}).}

\item{sig.color}{Character. Fill color for significant-slope regions
(default: \code{"#009E73"}, Okabe-Ito bluish green).}

\item{nonsig.color}{Character. Fill color for non-significant regions
(default: \code{"#D55E00"}, Okabe-Ito vermillion).}

\item{line.color}{Character. Color for the conditional-slope line
(default: \code{"#0072B2"}, Okabe-Ito blue).}

\item{alpha}{Numeric in (0, 1). Opacity of the confidence ribbon and the
region shading (default: 0.20).}

\item{line.size}{Numeric > 0. Width of the conditional-slope line
(default: 0.80).}

\item{rug}{Logical. If \code{TRUE}, draw a rug (density strip) of observed
moderator values along the x-axis (default: TRUE).}

\item{plot}{Logical. If \code{TRUE}, produce the JN plot (default: TRUE).}

\item{digits}{Integer \code{>= 0}. Decimal digits in printed output
(default: 3).}

\item{return_data}{Logical. If \code{TRUE}, include the plotting
\code{data.frame} in the returned list (default: FALSE).}

\item{x}{A 'lav_jn' object.}

\item{...}{Additional arguments; unused.}

\item{object}{A 'lav_jn' object.}
}
\value{
A list of class \code{"lav_jn"} with elements:
\describe{
  \item{\code{jn_points}}{Numeric vector of length 0, 1, or 2 giving the
    Johnson-Neyman boundary values of the moderator (the roots of the
    quadratic). \code{numeric(0)} when no real root exists.}
  \item{\code{signif_regions}}{A data frame with columns \code{lower},
    \code{upper}, and \code{significance} (\code{"significant"} or
    \code{"non-significant"}), describing the regions over the observed
    moderator range.}
  \item{\code{plot}}{A \code{ggplot} object (or \code{NULL} when
    \code{plot = FALSE}).}
  \item{\code{observed_support}}{Numeric length-2: the minimum and maximum of
    the observed moderator values.}
  \item{\code{interaction_test}}{List with the unstandardized interaction
    coefficient (\code{b}), its \code{se}, \code{z}, \code{p}, Wald
    confidence interval (\code{ci}), and standardized beta (\code{beta_std}).}
  \item{\code{labels}}{List of the user-supplied variable names.}
  \item{\code{conf.level}}{Confidence level used.}
  \item{\code{digits}}{Default digits for printing.}
  \item{\code{call}}{Matched call.}
  \item{\code{plot_data}}{Only when \code{return_data = TRUE}: data frame
    used to build the plot.}
}
}
\description{
Computes the Johnson-Neyman (JN) interval for a continuous moderator from a
fitted 'lavaan' model that includes an explicit product term. 
Plots the conditional slope with a confidence band, shaded significance
regions, and an observed-moderator density strip.
}
\details{
The model should include a main effect for the predictor, a main effect for
the moderator, and their product term. Standard errors are obtained via the
delta method using the model-implied covariance matrix of the parameter
estimates. If the model was fitted with a robust estimator (e.g., MLR),
the robust covariance matrix returned by \code{vcov(fit)} is used
automatically, so no separate correction is needed.

The JN boundary values are the real roots of the quadratic equation that
equates the squared conditional slope to the squared critical value times
its variance. When the discriminant is negative, the conditional slope is
either significant or non-significant across the entire moderator range and
no finite boundary exists.

This function is restricted to continuous moderators. For categorical
moderators, use \code{\link{lav_slopes}}.
}
\section{Notes}{

All estimates are unstandardized; a standardized coefficient for the interaction is
also reported for reference. Wald tests assume large-sample normality of the
parameter estimates. When the discriminant of the JN quadratic is negative,
the conditional slope is either significant or non-significant everywhere;
the function reports which case applies rather than returning a boundary.
}

\examples{
set.seed(42)
X <- rnorm(100); Z <- rnorm(100); X_Z <- X*Z
Y <- 0.6*X + 0.6*Z + 0.3*X_Z + rnorm(100, sd = 0.7) 
dataset <- data.frame(Y, X, Z, X_Z)
fit <- lavaan::sem("Y ~ X + Z + X_Z", data = dataset)
lav_jn(
fit = fit,
outcome = "Y",
pred = "X",
modx = "Z",
interaction = "X_Z"
)

}
\references{
Johnson, P. O., & Neyman, J. (1936). Tests of certain linear hypotheses and
their application to some educational problems. \emph{Statistical Research
Memoirs}, \emph{1}, 57-93.

Bauer, D. J., & Curran, P. J. (2005). Probing interactions in fixed and
multilevel regression: Inferential and graphical techniques.
\emph{Multivariate Behavioral Research}, \emph{40}(3), 373-400.
\doi{10.1207/s15327906mbr4003_5}

Carden, S. W., Holtzman, N. S., & Strube, M. J. (2017). CAHOST: An Excel
workbook for facilitating the Johnson-Neyman technique for two-way
interactions in multiple regression. \emph{Frontiers in Psychology},
\emph{8}, Article 1293. \doi{10.3389/fpsyg.2017.01293}
}
\seealso{
\code{\link{lav_slopes}} for pick-a-point simple slopes.
}
